@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: backgammon.h,v 1.280 2004/03/18 23:29:14 oysteijo Exp $
+ * $Id: backgammon.h,v 1.285 2004/04/22 19:14:34 thyssen Exp $
  */
 
 #ifndef _BACKGAMMON_H_
@@ -541,6 +541,7 @@ extern evalsetup esAnalysisCube, esAnalysisChequer;
 extern movefilter aamfEval[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
 extern movefilter aamfAnalysis[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ];
 
+extern void *rngctxRollout;
 extern rolloutcontext rcRollout;
 
 extern int fCubeEqualChequer, fPlayersAreSame, fTruncEqualPlayer0;
@@ -901,6 +902,10 @@ extern void CommandAccept( char * ),
     CommandRecordErase( char * ),
     CommandRecordEraseAll( char * ),
     CommandRecordShow( char * ),
+    CommandRelationalAddMatch( char * ),
+    CommandRelationalShowEnvironments( char * ),
+    CommandRelationalTest( char * ),
+    CommandRelationalHelp( char * ),
     CommandResign( char * ),
     CommandRoll( char * ),
     CommandRollout( char * ),
@@ -1048,6 +1053,7 @@ extern void CommandAccept( char * ),
     CommandSetGUIWindowPositions( char * ),
     CommandSetGUIShowIDs( char * ),
     CommandSetGUIShowPips( char * ),
+    CommandSetGUIShowEPCs( char * ),
     CommandSetGUIDragTargetHelp( char * ),
     CommandSetGUIUseStatsPanel( char * ),
     CommandSetHighlight ( char * ),
@@ -1260,6 +1266,8 @@ extern void CommandAccept( char * ),
     CommandShowJacoby( char * ),
     CommandShowKleinman( char * ),
     CommandShowLang( char * ),
+    CommandShowManualGUI( char * ),
+    CommandShowManualWeb( char * ),
     CommandShowMarketWindow( char * ),
     CommandShowMatchInfo( char * ),
     CommandShowMatchLength( char * ),
@@ -1365,7 +1373,7 @@ ShowBearoff( char* sz, matchstate* pms, bearoffcontext* pbc );
 
 extern int
 EPC( int anBoard[ 2 ][ 25 ], float *arEPC, float *arMu, float *arSigma, 
-     int *pfSource );
+     int *pfSource, const int fOnlyBearoff );
 
 extern char *
 ShowEPC( int anBoard[ 2 ][ 25 ] );
