@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: bearoff.h,v 1.11 2003/08/13 11:52:26 Superfly_Jon Exp $
+ * $Id: bearoff.h,v 1.13 2006/06/22 22:50:29 Superfly_Jon Exp $
  */
 
 #ifndef _BEAROFF_H_
@@ -99,7 +99,7 @@ enum _bearoffoptions {
 
 extern bearoffcontext *
 BearoffInit ( const char *szFilename, const char *szDir,
-              const int bo, void *p );
+              const int bo, void (*p)() );
 
 extern bearoffcontext *
 BearoffInitBuiltin ( void );
@@ -139,4 +139,7 @@ BearoffHyper( bearoffcontext *pbc,
               const unsigned int iPos,
               float arOutput[], float arEquity[] );
 
+extern bearoffcontext *
+BearoffAlloc( void );
+
 #endif /* _BEAROFF_H_ */
