@@ -18,7 +18,7 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: inc3d.h,v 1.19 2004/03/23 10:20:41 Superfly_Jon Exp $
+* $Id: inc3d.h,v 1.24 2006/05/01 20:17:32 oysteijo Exp $
 */
 #ifndef _INC3D_H_
 #define _INC3D_H_
@@ -84,19 +84,15 @@ typedef struct _diceTest
 } diceTest;
 
 /* Setup functions */
-void InitBoard3d(BoardData *bd);
-void InitBoardPreview(BoardData *bd);
 void InitGL(BoardData *bd);
 
 /* font functions */
-#ifndef __cplusplus
-void glPrintPointNumbers(BoardData* bd, const char *text, int mode);
-void glPrintCube(BoardData* bd, const char *text, int mode);
-void glPrintNumbersRA(BoardData* bd, const char *text, int mode);
-float getFontHeight(BoardData* bd);
-void BuildFont(BoardData* bd);
-void KillFont(BoardData* bd);
-#endif
+void glPrintPointNumbers(BoardData* bd, const char *text);
+void glPrintCube(BoardData* bd, const char *text);
+void glPrintNumbersRA(BoardData* bd, const char *text);
+int BuildFont3d(BoardData* bd);
+int RenderString3d(OGLFont *pFont, const char* str);
+float getTextLen3d(OGLFont *pFont, const char* text);
 
 /* Drawing functions */
 void drawBoard(BoardData* bd);
@@ -143,14 +139,14 @@ void AddGameData(GraphData* pgd, int game, statcontext *psc);
 void TidyGraphData(GraphData* pgd);
 
 /* Misc functions */
-void SetTexture(BoardData* bd, Material* pMat, const char* filename, TextureFormat format);
+void GetTexture(BoardData* bd, Material* pMat);
 void SetupSimpleMatAlpha(Material* pMat, float r, float g, float b, float a);
 void SetupSimpleMat(Material* pMat, float r, float g, float b);
 void SetupMat(Material* pMat, float r, float g, float b, float dr, float dg, float db, float sr, float sg, float sb, int shin, float a);
 void setMaterial(Material* pMat);
 void SetColour(float r, float g, float b, float a);
-unsigned char *LoadDIBTexture(FILE* fp, int *width, int *height);
-unsigned char *LoadPNGTexture(FILE *fp, int *width, int *height);
+/* unsigned char *LoadDIBTexture(FILE* fp, int *width, int *height); */
+/* unsigned char *LoadPNGTexture(FILE *fp, int *width, int *height); */
 float randRange(float range);
 void setupPath(BoardData *bd, Path* p, float* pRotate, int fClockwise, int fromPoint, int fromDepth, int toPoint, int toDepth);
 int movePath(Path* p, float d, float* pRotate, float v[3]);
@@ -164,9 +160,8 @@ int IsSet(int flags, int bit);
 float Dist2d(float a, float b);
 float ***Alloc3d(int x, int y, int z);
 void Free3d(float ***array, int x, int y);
-int LoadTexture(Texture* texture, const char* Filename, TextureFormat format);
+int LoadTexture(Texture* texture, const char* Filename);
 void CheckOpenglError();
-void freeEigthPoints(float ****boardPoints, int accuracy);
 
 typedef int idleFunc(BoardData* bd);
 void setIdleFunc(BoardData* bd, idleFunc* pFun);
