@@ -18,7 +18,7 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: widget3d.c,v 1.15 2004/03/12 10:56:07 Superfly_Jon Exp $
+* $Id: widget3d.c,v 1.19 2004/04/06 11:36:54 Superfly_Jon Exp $
 */
 
 #include <config.h>
@@ -42,6 +42,14 @@
 
 #if HAVE_GTKGLEXT
 GdkGLConfig *glconfig;
+
+GdkGLConfig *getGlConfig()
+{
+	if (!glconfig)
+		glconfig = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH | GDK_GL_MODE_DOUBLE | GDK_GL_MODE_STENCIL);
+
+	return glconfig;
+}
 #endif
 
 guint idleId = 0;
@@ -92,7 +100,7 @@ void setIdleFunc(BoardData* bd, idleFunc* pFun)
 
 void Draw(BoardData* bd)
 {	/* Render board - need multiple passes for shadows */
-	if (bd->showShadows)
+	if (bd->rd->showShadows)
 		shadowDisplay(drawBoard, bd);
 	else
 		drawBoard(bd);
@@ -116,7 +124,7 @@ static gboolean configure_event(GtkWidget *widget, GdkEventConfigure *notused, B
 	height = widget->allocation.height;
 
 	glViewport(0, 0, width, height);
-	SetupViewingVolume3d(bd, &rdAppearance);
+	SetupViewingVolume3d(bd);
 
 	RestrictiveRedraw();
 
@@ -142,8 +150,7 @@ void realize(GtkWidget *widget, BoardData* bd)
 #endif
 
 	InitGL(bd);
-	Set3dSettings(bd, &rdAppearance);
-	SetupViewingVolume3d(bd, &rdAppearance);
+	SetupViewingVolume3d(bd);
 	GetTextures(bd);
 	preDraw3d(bd);
 
@@ -171,17 +178,13 @@ static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, BoardData
 	if (!gdk_gl_drawable_gl_begin(gldrawable, gtk_widget_get_gl_context(widget)))
 		return TRUE;
 #else
-	/* Draw only last expose. */
-	if (event->count > 0)
-		return TRUE;
-
 	/* OpenGL functions can be called only if make_current returns true */
 	if (!gtk_gl_area_make_current(GTK_GL_AREA(widget)))
 		return TRUE;
 #endif
 	CheckOpenglError();
 
-	if (!bd->quickDraw)
+	if (!bd->rd->quickDraw)
 		Draw(bd);
 	else
 	{	/* Quick drawing mode */
@@ -208,37 +211,34 @@ static gboolean expose_event(GtkWidget *widget, GdkEventExpose *event, BoardData
 	return TRUE;
 }
 
-void CreateGLWidget(BoardData* bd, GtkWidget **drawing_area)
+extern GtkWidget* CreateGLWidget(BoardData* bd)
 {
+	GtkWidget* drawing_area;
 #if HAVE_GTKGLEXT
 	/* Drawing area for OpenGL */
-	*drawing_area = gtk_drawing_area_new();
+	drawing_area = gtk_drawing_area_new();
 	/* Set OpenGL-capability to the widget - no list sharing */
-	gtk_widget_set_gl_capability(*drawing_area, glconfig, NULL, TRUE, GDK_GL_RGBA_TYPE);
+	gtk_widget_set_gl_capability(drawing_area, getGlConfig(), NULL, TRUE, GDK_GL_RGBA_TYPE);
 #else
-	*drawing_area = gtk_gl_area_new_vargs(NULL, /* no sharing of lists */
+	drawing_area = gtk_gl_area_new_vargs(NULL, /* no sharing of lists */
 			 GDK_GL_RGBA, GDK_GL_DOUBLEBUFFER, GDK_GL_DEPTH_SIZE, 1, GDK_GL_STENCIL_SIZE, 1, GDK_GL_NONE);
 #endif
-	if (*drawing_area == NULL)
+	if (drawing_area == NULL)
 	{
 		g_print("Can't create opengl drawing widget\n");
-		return;
+		return NULL;
 	}
 
-	gtk_widget_set_events(*drawing_area, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | 
+	gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | 
 			GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "button_press_event", GTK_SIGNAL_FUNC(button_press_event), bd);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "button_release_event", GTK_SIGNAL_FUNC(button_release_event), bd);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "motion_notify_event", GTK_SIGNAL_FUNC(motion_notify_event), bd);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "realize", GTK_SIGNAL_FUNC(realize), bd);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "configure_event", GTK_SIGNAL_FUNC(configure_event), bd);
-	gtk_signal_connect(GTK_OBJECT(*drawing_area), "expose_event", GTK_SIGNAL_FUNC(expose_event), bd);
-}
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "button_press_event", GTK_SIGNAL_FUNC(button_press_event), bd);
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "button_release_event", GTK_SIGNAL_FUNC(button_release_event), bd);
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "motion_notify_event", GTK_SIGNAL_FUNC(motion_notify_event), bd);
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "realize", GTK_SIGNAL_FUNC(realize), bd);
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "configure_event", GTK_SIGNAL_FUNC(configure_event), bd);
+	gtk_signal_connect(GTK_OBJECT(drawing_area), "expose_event", GTK_SIGNAL_FUNC(expose_event), bd);
 
-void CreateBoard3d(BoardData* bd, GtkWidget** drawing_area)
-{
-	CreateGLWidget(bd, drawing_area);
-	InitBoard3d(bd);
+	return drawing_area;
 }
 
 void InitGTK3d(int *argc, char ***argv)
@@ -260,9 +260,7 @@ void Init3d()
 
 	/* Check for opengl support */
 #if HAVE_GTKGLEXT
-	/* Configure OpenGL-capable visual */
-	glconfig = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB | GDK_GL_MODE_DEPTH | GDK_GL_MODE_DOUBLE | GDK_GL_MODE_STENCIL);
-	if (!glconfig)
+	if (!getGlConfig())
 #else
 	if (!gdk_gl_query())
 #endif
@@ -395,9 +393,9 @@ GdkGLContext *glPixmapContext = NULL;
 
 void SetupPreview(BoardData* bd, renderdata* prd)
 {
-	ClearTextures(bd, TRUE);
+	ClearTextures(bd);
 	GetTextures(bd);
-	SetupViewingVolume3d(bd, prd);
+	SetupViewingVolume3d(bd);
 	preDraw3d(bd);
 }
 
@@ -437,7 +435,6 @@ void *CreatePreviewBoard3d(BoardData* bd, GdkPixmap *ppm)
 
 void RenderBoard3d(BoardData* bd, renderdata* prd, void *glpixmap, unsigned char* buf)
 {
-#if !LINUX
 	GLint viewport[4];
 	/*** OpenGL BEGIN ***/
 	GdkGLDrawable *gldrawable = GDK_GL_DRAWABLE((GdkGLPixmap *)glpixmap);
@@ -454,7 +451,6 @@ void RenderBoard3d(BoardData* bd, renderdata* prd, void *glpixmap, unsigned char
 
 	gdk_gl_drawable_gl_end(gldrawable);
 	/*** OpenGL END ***/
-#endif
 }
 
 #else
@@ -490,7 +486,6 @@ void *CreatePreviewBoard3d(BoardData* bd, GdkPixmap *ppm)
 
 void RenderBoard3d(BoardData* bd, renderdata* prd, void *ppm, unsigned char* buf)
 {
-#if !LINUX
 	GLint viewport[4];
 	GdkGLPixmap *glpixmap;
 
@@ -511,7 +506,6 @@ void RenderBoard3d(BoardData* bd, renderdata* prd, void *ppm, unsigned char* buf
 
 	gdk_gl_pixmap_unref(glpixmap);
 	gdk_gl_context_unref(glPixmapContext);
-#endif
 }
 
 #endif
