@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: eval.h,v 1.125 2004/02/24 10:20:47 uid68519 Exp $
+ * $Id: eval.h,v 1.131 2006/04/12 23:45:50 c_anthon Exp $
  */
 
 #ifndef _EVAL_H_
@@ -33,8 +33,8 @@
 #define TRUE 1
 #endif
 
-#define WEIGHTS_VERSION "0.14"
-#define WEIGHTS_VERSION_BINARY 0.14f
+#define WEIGHTS_VERSION "0.15"
+#define WEIGHTS_VERSION_BINARY 0.15f
 #define WEIGHTS_MAGIC_BINARY 472.3782f
 
 #define NUM_OUTPUTS 5
@@ -102,18 +102,31 @@ typedef struct {
        classes */
     unsigned int fCubeful : 1; /* cubeful evaluation */
     unsigned int nPlies   : 3;
+#if defined( REDUCTION_CODE )
     unsigned int nReduced : 3; /* this will need to be expanded if we add
 				  support for nReduced != 3 */
+#else
+    unsigned int fUsePrune : 1;
+#endif
     unsigned int fDeterministic : 1;
     float        rNoise;       /* standard deviation */
 } evalcontext;
 
-/* identifies the Rollout Context being written in sgf files. This should
-   be changed if the rollout context changes such that previous .sgf files
-   can't have their rollouts extended
+/* identifies the format of evaluation info in .sgf files
+   early (pre extending rollouts) had no version numbers
+   extendable rollouts have a version number of 1
+   with pruning nets, we have two possibilities - reduction could still
+   be enabled (version = 2) or, given the speedup and performance 
+   improvements, I assume we will drop reduction entirely. (ver = 3 or more)
+   When presented with an .sgf file, gnubg will attempt to work out what
+   data is present in the file based on the version number
 */
 
-#define SGF_ROLLOUT_VER 1
+#if defined( REDUCTION_CODE )
+#define SGF_FORMAT_VER 2
+#else
+#define SGF_FORMAT_VER 3
+#endif
 
 typedef struct {
 
@@ -569,6 +582,11 @@ getCurrentGammonRates ( float aarRates[ 2 ][ 2 ],
                         cubeinfo *pci,
                         evalcontext *pec );
 
+extern void
+calculate_gammon_rates( float aarRates[ 2 ][ 2 ],
+                        float arOutput[],
+                        cubeinfo *pci );
+
 extern void
 getMoneyPoints ( float aaarPoints[ 2 ][ 7 ][ 2 ],
                  const int fJacoby, const int fBeavers,
