@@ -3,6 +3,7 @@
  *
  * by Gary Wong, 1997
  *
+ * $Id: extwin.c,v 1.4 2005/10/29 15:47:01 Superfly_Jon Exp $
  */
 
 #include "config.h"
@@ -12,6 +13,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#if EXT
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 #include <X11/Xatom.h>
@@ -21,6 +23,7 @@
 
 #include <ext.h>
 #include <extwin.h>
+#endif
 
 #define AllButtonMask ( Button1Mask | Button2Mask | Button3Mask | Button4Mask \
 			| Button5Mask )
