@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format.c,v 1.10 2004/02/12 10:31:27 uid68519 Exp $
+ * $Id: format.c,v 1.16 2006/03/28 22:35:45 c_anthon Exp $
  */
 
 #include "config.h"
@@ -121,11 +121,17 @@ OutputEvalContext ( const evalcontext *pec, const int fChequer ) {
             pec->nPlies, 
             ( ! fChequer || pec->fCubeful ) ? _("cubeful") : _("cubeless") );
 
+#if defined( REDUCTION_CODE )
   if ( pec->nPlies == 2 ) 
     sprintf ( pc = strchr ( sz, 0 ),
               " %d%% speed",
               (pec->nReduced) ? 100 / pec->nReduced : 100 );
-
+#else
+  if( pec->fUsePrune ) {
+    sprintf( pc = strchr ( sz, 0 ), " prune" );
+  }
+#endif
+	    
   if ( fChequer && pec->nPlies ) {
     /* FIXME: movefilters!!! */
   }
@@ -386,14 +392,25 @@ OutputRolloutContext ( const char *szIndent, const evalsetup *pes ) {
               gettext( aszRNG[ prc->rngRollout ] ),
               prc->nSeed );
 
-  /* stop on std.err */
+   if ( ( prc->fStopOnJsd || prc->fStopMoveOnJsd || prc->fStopOnSTD )
+        && szIndent && *szIndent )
+     strcat ( sz, szIndent );
 
-  if ( prc->fStopOnSTD )
+    /* stop on std.err */
+
+   if ( prc->fStopOnSTD && !prc->fStopMoveOnJsd )
     sprintf( strchr( sz, 0 ),
              _("Stop when std.errs. are small enough: ratio "
                "%.4g (min. %d games)\n"),
              prc->rStdLimit, prc->nMinimumGames );
 
+   /* stop on JSD */
+   if ( prc->fStopOnJsd || prc->fStopMoveOnJsd )
+     sprintf( strchr( sz, 0 ),
+              _("Stop when best play is enough JSDs ahead: limit "
+                "%.4g (min. %d games)\n"),
+              prc->rJsdLimit, prc->nMinimumJsdGames );
+
   /* first play */
 
   OutputEvalContextsForRollout( sz, szIndent, 
@@ -450,7 +467,7 @@ OutputEquity ( const float r, const cubeinfo *pci, const int f ) {
   else {
     if ( fOutputMatchPC ) {
       /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits - 1 );
+      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
       sprintf ( sz, fmt,
                 100.0f * ( f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) ) );
     }
@@ -519,7 +536,7 @@ OutputEquityScale ( const float r, const cubeinfo *pci,
 
       if ( fOutputMatchPC ) {
         /* fmt: "%6.2f%%" */
-        sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits - 1 );
+        sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
         sprintf ( sz, fmt,
                   100.0f * ( f ? eq2mwc ( r, pci ) : se_eq2mwc ( r, pci ) ) );
       }
@@ -574,7 +591,7 @@ OutputEquityDiff ( const float r1, const float r2, const cubeinfo *pci ) {
   else {
     if ( fOutputMatchPC ) {
       /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits - 1 );
+      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
       sprintf ( sz, fmt,
                 100.0f * eq2mwc ( r1, pci ) - 100.0f * eq2mwc ( r2, pci ) );
     }
@@ -620,7 +637,7 @@ OutputMWCDiff ( const float r1, const float r2, const cubeinfo *pci ) {
   else {
     if ( fOutputMatchPC ) {
       /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits - 1 );
+      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
       sprintf ( sz, fmt,
                 100.0f * r1 - 100.0f * r2 );
     }
@@ -668,7 +685,7 @@ OutputMWC ( const float r, const cubeinfo *pci, const int f ) {
     }
     else if ( fOutputMatchPC ) {
       /* fmt: "%6.2f%%" */
-      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits - 1 );
+      sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 3, fOutputDigits > 1 ? fOutputDigits - 1 : 0 );
       sprintf ( sz, fmt, 100.0f * r );
     }
     else {
@@ -692,7 +709,7 @@ OutputPercent ( const float r ) {
 
   if ( fOutputWinPC ) {
     /* fmt: "%5.1f" */
-    sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 2, fOutputDigits - 2 );
+	  sprintf( fmt, "%%%d.%df%%%%", fOutputDigits + 2, fOutputDigits > 2 ? fOutputDigits - 2 : 0 );
     sprintf ( sz, fmt, 100.0 * r );
   }
   else {
@@ -757,7 +774,7 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
 
   float r;
 
-  int fClose, fMissed;
+  int fMissed;
   int fAnno = FALSE;
 
   float arDouble[ 4 ];
@@ -771,13 +788,11 @@ OutputCubeAnalysisFull ( const float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ],
   /* check if cube analysis should be printed */
 
   if ( pes->et == EVAL_NONE ) return NULL; /* no evaluation */
-  if ( ! GetDPEq ( NULL, NULL, pci ) ) return NULL; /* cube not available */
 
   FindCubeDecision ( arDouble, aarOutput, pci );
 
   /* print alerts */
 
-  fClose = isCloseCubedecision ( arDouble ); 
   fMissed = 
     fDouble > -1 &&
     isMissedDouble ( arDouble, aarOutput, fDouble, pci );
