@@ -16,26 +16,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gnubgmodule.c,v 1.28 2004/03/15 06:50:11 thyssen Exp $
+ * $Id: gnubgmodule.c,v 1.48 2005/02/10 10:31:12 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
 #include "config.h"
+#endif
 #if USE_PYTHON
-#undef HAVE_FSTAT
-#endif
-#endif
-
-#if USE_PYTHON
-#include <Python.h>
-#if HAVE_CONFIG_H
-#undef HAVE_FSTAT
-#include "config.h"
-#endif
+#include "gnubgmodule.h"
 
 #include <signal.h>
 #include <assert.h>
 #include <string.h>
+#include <math.h>
 
 #if HAVE_LIBREADLINE
 #include <readline/history.h>
@@ -43,13 +36,17 @@
 #endif
 
 #include <stdio.h>
+#include <glib.h>
 #include "i18n.h"
 #include "backgammon.h"
 #include "eval.h"
 #include "matchequity.h"
 #include "path.h"
 #include "positionid.h"
+#include "analysis.h"
+#include "md5.h"
 
+#undef IGNORE	// Ignore msdev define
 #define IGNORE __attribute__ ((unused))
 
 static PyObject *
@@ -119,6 +116,22 @@ PyToBoard(PyObject* p, int anBoard[ 2 ][ 25 ])
   return 0;
 }
 
+static int
+PyToDice( PyObject* p, int anDice[ 2 ] )
+{
+  if( PySequence_Check(p) && PySequence_Size(p) == 2 ) {
+    int j;
+
+    for ( j = 0; j < 2; ++j ) {
+      PyObject* pi = PySequence_Fast_GET_ITEM(p, j);
+    
+      anDice[ j ] = (int) PyInt_AsLong( pi );
+    }
+    return 1;
+  }
+
+  return 0;
+}
 
 static PyObject *
 CubeInfoToPy( const cubeinfo *pci )
@@ -232,10 +245,16 @@ PyToCubeInfo( PyObject *p, cubeinfo *pci ) {
 static PyObject*
 EvalContextToPy( const evalcontext* pec)
 {
-  return Py_BuildValue( "{s:i,s:i,s:i,s:i,s:f}",
+  return Py_BuildValue( "{s:i,s:i"
+#if defined( REDUCTION_CODE )
+			",s:i"
+#endif
+			",s:i,s:f}",
                         "cubeful", pec->fCubeful,
                         "plies", pec->nPlies,
+#if defined( REDUCTION_CODE )
                         "reduced", pec->nReduced,
+#endif
                         "deterministic", pec->fDeterministic,
                         "noise", pec->rNoise );
 }
@@ -291,9 +310,11 @@ PyToEvalContext( PyObject *p, evalcontext *pec ) {
         pec->fCubeful = i ? 1 : 0;
       else if ( iKey == 1 )
         pec->nPlies = ( i < 8 ) ? i : 7;
-      else if ( iKey == 2 )
+      else if ( iKey == 2 ) {
+#if defined( REDUCTION_CODE )
         pec->nReduced = ( i < 8 ) ? i : 7;
-      else 
+#endif
+      } else 
         pec->fDeterministic = i ? 1 : 0;
 
       break;
@@ -384,7 +405,9 @@ PythonEvalContext( PyObject* self IGNORE, PyObject *args ) {
 
   ec.fCubeful = fCubeful ? 1 : 0;
   ec.nPlies = ( nPlies < 8 ) ? nPlies : 7;
+#if defined( REDUCTION_CODE )
   ec.nReduced = ( nReduced < 8 ) ? nReduced : 7;
+#endif
   ec.fDeterministic = fDeterministic ? 1 : 0;
   ec.rNoise = rNoise;
                            
@@ -406,7 +429,9 @@ PythonCommand( PyObject* self IGNORE, PyObject *args ) {
   PortableSignal( SIGINT, HandleInterrupt, &sh, FALSE );
   HandleCommand( sz, acTop );
 
-  PythonNextTurn(0, 0);
+   if( ms.gs != GAME_NONE ) {  /* HACK, no idea if this is right or not */
+  PythonNextTurn(0, 0); 
+   }
 
   outputx();
   free( sz );
@@ -436,6 +461,27 @@ PythonBoard( PyObject* self IGNORE, PyObject *args ) {
   return BoardToPy( ms.anBoard );
 }
 
+static PyObject *
+PythonLuckRating( PyObject* self IGNORE, PyObject *args ) {
+
+  float r;
+  if ( ! PyArg_ParseTuple( args, "f", &r ) )
+    return NULL;
+
+  return PyInt_FromLong( getLuckRating( r ) );
+
+}
+
+static PyObject *
+PythonErrorRating( PyObject* self IGNORE, PyObject *args ) {
+
+  float r;
+  if ( ! PyArg_ParseTuple( args, "f", &r ) )
+    return NULL;
+
+  return PyInt_FromLong( GetRating( r ) );
+
+}
 
 static PyObject *
 PythonEvaluate( PyObject* self IGNORE, PyObject *args ) {
@@ -481,6 +527,113 @@ PythonEvaluate( PyObject* self IGNORE, PyObject *args ) {
   }
 }
 
+static PyObject *
+PythonEvaluateCubeful( PyObject* self IGNORE, PyObject *args ) {
+
+  PyObject *pyBoard = NULL;
+  PyObject *pyCubeInfo = NULL;
+  PyObject *pyEvalContext = NULL;
+
+  int anBoard[ 2 ][ 25 ];
+  float aarOutput[ 2 ][ NUM_ROLLOUT_OUTPUTS ], arCube[ NUM_CUBEFUL_OUTPUTS ];
+  cubeinfo ci;
+  evalcontext ec = { 0, 0, 0, 1, 0.0f };
+  int cp;
+
+  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  
+  GetMatchStateCubeInfo( &ci, &ms );
+
+  if ( ! PyArg_ParseTuple( args, "|OOO", 
+                           &pyBoard, &pyCubeInfo, &pyEvalContext ) )
+    return NULL;
+
+  if ( pyBoard && !PyToBoard( pyBoard, anBoard ) )
+    return NULL;
+
+  if ( pyCubeInfo && PyToCubeInfo( pyCubeInfo, &ci ) )
+    return NULL;
+
+  if ( pyEvalContext && PyToEvalContext( pyEvalContext, &ec ) )
+    return NULL;
+
+  if ( GeneralCubeDecisionE ( aarOutput, anBoard, &ci, &ec, 0 ) < 0 ){
+    PyErr_SetString( PyExc_StandardError, 
+                     _("interupted/errno in GeneralCubeDecisionE") );
+    return NULL;
+  }
+
+  cp = FindCubeDecision ( arCube, GCCCONSTAHACK aarOutput, &ci );
+
+  {
+    PyObject* p = PyTuple_New( NUM_CUBEFUL_OUTPUTS + 2 );
+    int k;
+    for(k = 0; k < NUM_CUBEFUL_OUTPUTS; ++k) {
+      PyTuple_SET_ITEM(p, k, PyFloat_FromDouble(arCube[k]));
+    }
+	PyTuple_SET_ITEM(p, NUM_CUBEFUL_OUTPUTS, PyInt_FromLong(cp));
+	PyTuple_SET_ITEM(p, NUM_CUBEFUL_OUTPUTS+1, PyString_FromString(GetCubeRecommendation(cp)));
+
+    return p;
+  }
+}
+
+static PyObject *
+PythonFindBestMove( PyObject* self IGNORE, PyObject *args ) {
+
+  PyObject *pyDice = NULL;
+  PyObject *pyBoard = NULL;
+  PyObject *pyCubeInfo = NULL;
+  PyObject *pyEvalContext = NULL;
+
+  int anBoard[ 2 ][ 25 ];
+  int anDice[ 2 ];
+  int anMove[ 8 ];
+  cubeinfo ci;
+  evalcontext ec = { 0, 0, 0, 1, 0.0f };
+
+  memcpy( anBoard, ms.anBoard, sizeof anBoard );
+  memcpy( anDice, ms.anDice, sizeof anDice );
+  GetMatchStateCubeInfo( &ci, &ms );
+  if ( ! PyArg_ParseTuple( args, "|OOOO", 
+                           &pyBoard, &pyCubeInfo, &pyEvalContext, &pyDice ) )
+    return NULL;
+
+  if ( pyDice && !PyToDice( pyDice, anDice ) )
+    return NULL;
+
+  if (!anDice){
+	  printf("What? No dice?\n");
+	  return NULL;
+  }
+
+  if ( pyBoard && !PyToBoard( pyBoard, anBoard ) )
+    return NULL;
+
+  if ( pyCubeInfo && PyToCubeInfo( pyCubeInfo, &ci ) )
+    return NULL;
+
+  if ( pyEvalContext && PyToEvalContext( pyEvalContext, &ec ) )
+    return NULL;
+
+  /* 
+   * FIXME: The function will use the eval movefilter. The function
+   * should take a movefilter as an argument.
+   */
+  
+  if( FindBestMove( anMove, anDice[ 0 ], anDice[ 1 ],
+                        anBoard, &ci, &ec, aamfEval ) < 0 )
+        return NULL;
+  {
+    PyObject* p = PyTuple_New( 8 );
+    int k;
+    for(k = 0; k < 8; ++k) {
+      PyTuple_SET_ITEM(p, k, Py_BuildValue( "i", anMove[k]+1));
+    }
+    return p;
+  }
+
+}
 
 static PyObject *
 METRow( float ar[ MAXSCORE ], const int n ) {
@@ -592,6 +745,83 @@ PythonEq2mwc( PyObject* self IGNORE, PyObject *args ) {
 
 }
 
+#define STRBUF_SIZE 1024
+
+void AddString(list* buffers, char* str)
+{
+	list* pCurrent = buffers->plPrev;
+	if (!pCurrent->p || strlen(str) + strlen(pCurrent->p) > STRBUF_SIZE)
+	{
+		char* newBuf = malloc(STRBUF_SIZE + 1);
+		*newBuf = '\0';
+		pCurrent = ListInsert(buffers, newBuf);
+	}
+	strcat(pCurrent->p, str);
+}
+
+extern char* GameAsString()
+{
+	char *ret;
+	int size;
+	char buf[1024];
+	list *pList, *plGame, *plMove;
+	list buffers;
+	ListCreate(&buffers);
+
+	sprintf(buf, "%s vs %s (%d)", ap[0].szName, ap[1].szName, ms.nMatchTo);
+	AddString(&buffers, buf);
+
+	for (plGame = lMatch.plNext; plGame->p; plGame = plGame->plNext)
+	{
+		list* plStart = plGame->p;
+		int move = 1;
+		for (plMove = plStart->plNext; plMove->p; plMove = plMove->plNext)
+		{
+			char playerStr[3] = ".AB";
+			int player;
+			moverecord* pmr = plMove->p;
+			char* moveString = GetMoveString(pmr, &player);
+			if (moveString)
+			{
+				sprintf(buf, " %d%c %s", move, playerStr[player + 1], moveString);
+				AddString(&buffers, buf);
+				if (player == 1)
+					move++;
+			}
+		}
+	}
+
+	/* Create single string from buffers */
+	size = 0;
+	for (pList = buffers.plNext; pList->p; pList = pList->plNext)
+		size += strlen(pList->p);
+
+	ret = malloc(size + 1);
+	*ret = '\0';
+	for (pList = buffers.plNext; pList->p; pList = pList->plNext)
+		strcat(ret, pList->p);
+
+	ListDeleteAll(&buffers);
+
+	return ret;
+}
+
+extern PyObject *
+PythonMatchChecksum( PyObject* self IGNORE, PyObject *args )
+{
+	unsigned char auch[16], auchHex[33];
+	int i;
+	// Work out md5 checksum
+	char* gameStr = GameAsString();
+	md5_buffer(gameStr, strlen(gameStr), auch);
+	free(gameStr);
+	/* Convert to hex so stores easily in database - is there a better way? */
+	for (i = 0; i < 16; i++)
+		sprintf(auchHex + (i * 2), "%02x", auch[i]);
+
+	return PyString_FromString(auchHex);
+}
+
 static PyObject *
 PythonMwc2eq( PyObject* self IGNORE, PyObject *args ) {
 
@@ -723,7 +953,7 @@ PythonPositionBearoff( PyObject* self IGNORE, PyObject *args )
   int nPoints = 6;
   int anBoard[ 25 ];
 
-  memcpy( anBoard, 0, sizeof anBoard );
+  memcpy( anBoard, ms.anBoard, sizeof anBoard );
 
   if ( ! PyArg_ParseTuple( args, "|Oii:positionbearoff", 
                            &pyBoard, &nPoints, &nChequers ) )
@@ -817,9 +1047,11 @@ diffContext(const evalcontext* c, PyMatchState* ms)
       DictSetItemSteal(context, "plies", PyInt_FromLong(c->nPlies));
     }
 
+#if defined( REDUCTION_CODE )
     if( c->nReduced != s->nReduced ) {
       DictSetItemSteal(context, "reduced", PyInt_FromLong(c->nReduced));
     }
+#endif
 
     if( c->fDeterministic != s->fDeterministic ) {
       DictSetItemSteal(context, "deterministic",
@@ -1209,13 +1441,17 @@ PyDoubleAnalysis(const evalsetup* pes,
 }
 
 static PyObject*
-PyGameStats(const statcontext* sc)
+PyGameStats(const statcontext* sc, const int fIsMatch, const int nMatchTo)
 {
   PyObject* p[2];
+  float aaaar[ 3 ][ 2 ][ 2 ][ 2 ];
+
   if( ! (sc->fMoves || sc->fCube || sc->fDice) ) {
     return 0;
   }
 
+  getMWCFromError( sc, aaaar );
+
   p[0] = PyDict_New();
   p[1] = PyDict_New();
   
@@ -1248,40 +1484,46 @@ PyGameStats(const statcontext* sc)
   if( sc->fCube ) {
     int side;
     for(side = 0; side < 2; ++side) {
+      float errorskill = 
+         aaaar[ CUBEDECISION ][ PERMOVE ][ side ][ NORMALISED ];
+      float errorcost = 
+         aaaar[ CUBEDECISION ][ PERMOVE ][ side ][ UNNORMALISED ];
+
       PyObject* d =
 	Py_BuildValue("{s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i,s:i"
-		      "s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f}",
+		      "s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,s:f,"
+                      "s:f,s:f}",
 		      "total-cube",sc->anTotalCube[side],
                       "close-cube", sc->anCloseCube[side],
 		      "n-doubles",sc->anDouble[side],
 		      "n-takes", sc->anTake[side],
 		      "n-drops", sc->anPass[side],
 
-		      "missed-double-dp",sc->anCubeMissedDoubleDP[side],
-		      "missed-double-tg", sc->anCubeMissedDoubleTG[side],
-		      "wrong-double-dp", sc->anCubeWrongDoubleDP[side],
-		      "wrong-double-tg",sc->anCubeWrongDoubleTG[side],
+		      "missed-double-below-cp",sc->anCubeMissedDoubleDP[side],
+		      "missed-double-above-cp", sc->anCubeMissedDoubleTG[side],
+		      "wrong-double-below-dp", sc->anCubeWrongDoubleDP[side],
+		      "wrong-double-above-tg",sc->anCubeWrongDoubleTG[side],
 		      "wrong-take", sc->anCubeWrongTake[side],
 		      "wrong-drop", sc->anCubeWrongPass[side],
 
-		      "err-missed-double-dp-skill",
+		      "err-missed-double-below-cp-skill",
 		      sc->arErrorMissedDoubleDP[side][0],
-		      "err-missed-double-dp-cost",
+		      "err-missed-double-below-cp-cost",
 		      sc->arErrorMissedDoubleDP[side][1],
 
-		      "err-missed-double-tg-skill",
+		      "err-missed-double-above-cp-skill",
 		      sc->arErrorMissedDoubleTG[side][ 0 ],
-		      "err-missed-double-tg-cost",
+		      "err-missed-double-above-cp-cost",
 		      sc->arErrorMissedDoubleTG[side][ 1 ],
 
-		      "err-wrong-double-dp-skill",
+		      "err-wrong-double-below-dp-skill",
 		      sc->arErrorWrongDoubleDP[side][ 0 ],
-		      "err-wrong-double-dp-cost",
+		      "err-wrong-double-below-dp-cost",
 		      sc->arErrorWrongDoubleDP[side][ 1 ],
 
-		      "err-wrong-double-tg-skill",
+		      "err-wrong-double-above-tg-skill",
 		      sc->arErrorWrongDoubleTG[side][ 0 ],
-		      "err-wrong-double-tg-cost",
+		      "err-wrong-double-above-tg-cost",
 		      sc->arErrorWrongDoubleTG[side][ 1 ],
 
 		      "err-wrong-take-skill",
@@ -1292,7 +1534,12 @@ PyGameStats(const statcontext* sc)
 		      "err-wrong-drop-skill",
 		      sc->arErrorWrongPass[side][ 0 ],
 		      "err-wrong-drop-cost",
-		      sc->arErrorWrongPass[side][ 1 ] );
+		      sc->arErrorWrongPass[side][ 1 ],
+
+                      "error-skill",
+                      errorskill,
+                      "error-cost",
+                      errorcost );
 
       DictSetItemSteal(p[side], "cube", d);
     }
@@ -1302,14 +1549,14 @@ PyGameStats(const statcontext* sc)
     int side;
     for(side = 0; side < 2; ++side) {
       PyObject* d =
-	Py_BuildValue("{s:f,s:f,s:f,s:f}",
+	Py_BuildValue("{s:f,s:f,s:f,s:f,s:f}",
 		      "luck", sc->arLuck[side][0],
 		      "luck-cost", sc->arLuck[side][1], 
                       "actual-result", sc->arActualResult[side],
                       "luck-adjusted-result", sc->arLuckAdj[side]);
     
       PyObject* m = PyDict_New();
-
+  
       {
 	lucktype lt;
 	for( lt = LUCK_VERYBAD; lt <= LUCK_VERYGOOD; lt++ ) {
@@ -1317,6 +1564,17 @@ PyGameStats(const statcontext* sc)
 			   PyInt_FromLong(sc->anLuck[0][lt]));
 	}
       }
+
+      /* luck based fibs rating difference */
+  
+      if ( nMatchTo && fIsMatch ) {
+         float r = 0.5f + sc->arActualResult[ side ] -
+              sc->arLuck[ side ][ 1 ] + sc->arLuck[ !side ][ 1 ];
+         if ( r > 0.0f && r < 1.0f )
+            DictSetItemSteal( d, "fibs-rating-difference",
+                              PyFloat_FromDouble( relativeFibsRating( r,
+                                                  nMatchTo ) ) );
+      }
       
       DictSetItemSteal(d, "marked-rolls", m);
 
@@ -1324,6 +1582,78 @@ PyGameStats(const statcontext* sc)
     }
   }
 
+#if USE_TIMECONTROL
+  {
+     int side;
+     for ( side = 0; side < 2; ++side ) {
+        PyObject *d =
+            Py_BuildValue( "{s:i,s:f,s:f}",
+                           "time-penalty", sc->anTimePenalties[ side ],
+                           "time-penalty-skill", sc->aarTimeLoss[ side ][ 0 ],
+                           "time-penalty-cost", sc->aarTimeLoss[ side ][ 1 ] );
+
+         DictSetItemSteal( p[ side ], "time", d );
+      }
+  }
+#endif /* USE_TIMECONTROL */
+
+  /* advantage */
+
+  if ( sc->fDice && fIsMatch && !nMatchTo && sc->nGames > 1 ) {
+
+     int side;
+     for ( side = 0; side < 2; ++side ) {
+        PyObject *d =
+            Py_BuildValue( "{s:f,s:f,s:f,s:f}",
+                           "actual", sc->arActualResult[ side ] / sc->nGames,
+                           "actual-ci", 1.95996f * 
+                           sqrt( sc->arVarianceActual[ side ] / sc->nGames ),
+                           "luck-adjusted", 
+                           sc->arLuckAdj[ side ] / sc->nGames,
+                           "luck-adjusted-ci", 1.95996f * 
+                           sqrt( sc->arVarianceLuckAdj[ side ] / sc->nGames ) );
+
+         DictSetItemSteal( p[ side ], "ppg-advantage", d );
+      }
+
+  }
+
+  /* error based fibs ratings */
+
+  if ( ( sc->fCube || sc->fMoves ) && fIsMatch && nMatchTo ) {
+
+     int side;
+     for ( side = 0; side < 2; ++side ) {
+
+        PyObject* d = PyDict_New();
+
+        DictSetItemSteal( d, "total", 
+                          PyFloat_FromDouble( absoluteFibsRating( aaaar[ CHEQUERPLAY ][ PERMOVE ][ side ][ NORMALISED ],
+                                                                  aaaar[ CUBEDECISION ][ PERMOVE ][ side ][ NORMALISED ],
+                                                                  nMatchTo, rRatingOffset ) ) );
+ 
+        if ( sc->anUnforcedMoves[ side ] ) {
+           DictSetItemSteal( d, "chequer", 
+                             PyFloat_FromDouble( absoluteFibsRatingChequer( aaaar[ CHEQUERPLAY ][ PERMOVE ][ side ][ NORMALISED ],
+                             nMatchTo ) ) );
+ 
+        }
+
+        if ( sc->anCloseCube[ side ] ) {
+           DictSetItemSteal( d, "cube", 
+                             PyFloat_FromDouble( absoluteFibsRatingCube( aaaar[ CUBEDECISION ][ PERMOVE ][ side ][ NORMALISED ],
+                             nMatchTo ) ) );
+ 
+        }
+
+        DictSetItemSteal( p[ side ], "error-based-fibs-rating", d );
+
+     }
+
+  }
+  
+  /* */
+
   {
     PyObject* d = PyDict_New();
 
@@ -1350,7 +1680,7 @@ PythonGame(const list*    plGame,
 {
   const list* pl = plGame->plNext;
   const moverecord* pmr = pl->p;            
-  const movegameinfo* g = &pmr->g;
+  const xmovegameinfo* g = &pmr->g;
 
   PyObject* gameDict = PyDict_New();
   PyObject* gameInfoDict = PyDict_New();
@@ -1400,7 +1730,7 @@ PythonGame(const list*    plGame,
     AddStatcontext(&g->sc, scMatch);
     
     {
-      PyObject* s = PyGameStats(&g->sc);
+      PyObject* s = PyGameStats(&g->sc,FALSE,g->nMatch);
 
       if( s ) {
 	DictSetItemSteal(gameDict, "stats", s);
@@ -1439,31 +1769,33 @@ PythonGame(const list*    plGame,
       switch( pmr->mt ) {
 	case MOVE_NORMAL:
 	{
-	  const movenormal* n = &pmr->n;
-
 	  action = "move";
-	  player = n->fPlayer;
+	  player = pmr->fPlayer;
 
 	  {
-	    PyObject* dice = Py_BuildValue("(ii)", n->anRoll[0], n->anRoll[1]);
+	    PyObject* dice = Py_BuildValue("(ii)", 
+                                           pmr->anDice[0], pmr->anDice[1]);
+
 	    
 	    DictSetItemSteal(recordDict, "dice", dice);
 	  }
 
-	  DictSetItemSteal(recordDict, "move", PyMove(n->anMove));
+	  DictSetItemSteal(recordDict, "move", PyMove(pmr->n.anMove));
 
 	  if( includeBoards ) {
 	    DictSetItemSteal(recordDict, "board",
 			     PyString_FromString(PositionID(anBoard)));
 	    
-	    ApplyMove(anBoard, n->anMove, 0);
+	    ApplyMove(anBoard, pmr->n.anMove, 0);
 	    SwapSides(anBoard);
 	  }
 	  
 	  if( analysis ) {
-	    if( n->esDouble.et != EVAL_NONE ) {
+	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE ) {
 	      PyObject* d =
-		PyDoubleAnalysis(&n->esDouble,   n->aarOutput, n->aarStdDev,
+		PyDoubleAnalysis(&pmr->CubeDecPtr->esDouble,   
+                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
+                                 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
 				 ms, verbose);
 	      {
 		int s = PyDict_Merge(analysis, d, 1);     assert( s != -1 );
@@ -1472,28 +1804,27 @@ PythonGame(const list*    plGame,
 	    }
 	  
 	  
-	    if( n->ml.cMoves ) {
-	      PyObject* a = PyMoveAnalysis(&n->ml, ms);
+	    if( pmr->ml.cMoves ) {
+	      PyObject* a = PyMoveAnalysis(&pmr->ml, ms);
 
 	      if( a ) {
 		DictSetItemSteal(analysis, "moves", a);
 
-		DictSetItemSteal(analysis, "imove", PyInt_FromLong(n->iMove));
+		DictSetItemSteal(analysis, "imove", PyInt_FromLong(pmr->n.iMove));
 	      }
 	    }
 
-	    addLuck(analysis, n->rLuck, n->lt);
-	    addSkill(analysis, n->stMove, 0);
-	    addSkill(analysis, n->stCube, "cube-skill");
+	    addLuck(analysis, pmr->rLuck, pmr->lt);
+	    addSkill(analysis, pmr->n.stMove, 0);
+	    addSkill(analysis, pmr->stCube, "cube-skill");
 	  }
 	    
 	  break;
 	}
 	case MOVE_DOUBLE:
 	{
-	  const movedouble* d = &pmr->d;
 	  action = "double";
-	  player = d->fPlayer;
+	  player = pmr->fPlayer;
 
 	  if( includeBoards ) {
 	    DictSetItemSteal(recordDict, "board",
@@ -1501,7 +1832,7 @@ PythonGame(const list*    plGame,
 	  }
 	    
 	  if( analysis ) {
-	    const cubedecisiondata* c = d->CubeDecPtr;
+	    const cubedecisiondata* c = pmr->CubeDecPtr;
 	    if( c->esDouble.et != EVAL_NONE ) {
 	      PyObject* d = PyDoubleAnalysis(&c->esDouble, c->aarOutput,
 					     c->aarStdDev, ms, verbose);
@@ -1511,45 +1842,42 @@ PythonGame(const list*    plGame,
 	      Py_DECREF(d);
 	    }
 	  
-	    addSkill(analysis, d->st, 0);
+	    addSkill(analysis, pmr->stCube, 0);
 	  }
 	    
 	  break;
 	}
 	case MOVE_TAKE:
 	{
-	  const movedouble* d = &pmr->d;
 	  action = "take";
-	  player = d->fPlayer;
+	  player = pmr->fPlayer;
 
 	  /* use nAnimals to point to double analysis ? */
 	  
-	  addSkill(analysis, d->st, 0);
+	  addSkill(analysis, pmr->stCube, 0);
 	    
 	  break;
 	}
 	case MOVE_DROP:
 	{
-	  const movedouble* d = &pmr->d;
 	  action = "drop";
-	  player = d->fPlayer;
+	  player = pmr->fPlayer;
 	    
-	  addSkill(analysis, d->st, 0);
+	  addSkill(analysis, pmr->stCube, 0);
 	    
 	  break;
 	}
 	case MOVE_RESIGN:
 	{
-	  const moveresign* r = &pmr->r;
 	  action = "resign";
-	  player = r->fPlayer;
+	  player = pmr->fPlayer;
 	  break;
 	}
 	
 	case MOVE_SETBOARD:
 	{
-	  const movesetboard* sb = &pmr->sb;
-	  PyObject* id = PyString_FromString(PositionIDFromKey(sb->auchKey));
+	  PyObject* id = 
+            PyString_FromString(PositionIDFromKey(pmr->sb.auchKey));
 
 	  action = "set";
 
@@ -1557,7 +1885,8 @@ PythonGame(const list*    plGame,
 
 	  if( includeBoards ) {
 	    /* (FIXME) what about side? */
-	    PositionFromKey(anBoard, sb->auchKey);
+            /* JTH: the board is always stored as if player 0 was on roll */
+	    PositionFromKey(anBoard, pmr->sb.auchKey);
 	  }
 	  
 	  break;
@@ -1565,34 +1894,31 @@ PythonGame(const list*    plGame,
 	    
 	case MOVE_SETDICE:
 	{
-	  const movesetdice* sd = &pmr->sd;
           PyObject *dice;
 
-	  player = sd->fPlayer;
+	  player = pmr->fPlayer;
 	  action = "set";
 
-	  dice = Py_BuildValue("(ii)", sd->anDice[0], sd->anDice[1]);
+	  dice = Py_BuildValue("(ii)", pmr->anDice[0], pmr->anDice[1]);
 
 	  DictSetItemSteal(recordDict, "dice", dice);
 	    
-	  addLuck(analysis, sd->rLuck, sd->lt);
+	  addLuck(analysis, pmr->rLuck, pmr->lt);
 	    
 	  break;
 	}
 	    
 	case MOVE_SETCUBEVAL:
 	{
-	  const movesetcubeval* scv = &pmr->scv;
 	  action = "set";
-	  DictSetItemSteal(recordDict, "cube", PyInt_FromLong(scv->nCube));
+	  DictSetItemSteal(recordDict, "cube", PyInt_FromLong(pmr->scv.nCube));
 	  break;
 	}
 	    
 	case MOVE_SETCUBEPOS:
 	{
-	  const movesetcubepos* scp = &pmr->scp;
 	  const char* s[] = {"centered", "X", "O"};
-	  const char* o = s[scp->fCubeOwner + 1];
+	  const char* o = s[pmr->scp.fCubeOwner + 1];
 	  
 	  action = "set";
 	  DictSetItemSteal(recordDict, "cube-owner",
@@ -1624,9 +1950,9 @@ PythonGame(const list*    plGame,
 	}
       }
       
-      if( pmr->a.sz ) {
+      if( pmr->sz ) {
 	DictSetItemSteal(recordDict, "comment",
-			     PyString_FromString(pmr->a.sz));
+			     PyString_FromString(pmr->sz));
       }
 
       PyTuple_SET_ITEM(gameTuple, nRecords, recordDict);
@@ -1655,7 +1981,7 @@ PythonMatch(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
   /* take match info from first game */
   const list* firstGame = lMatch.plNext->p;
   const moverecord* pmr;
-  const movegameinfo* g;
+  const xmovegameinfo* g;
   int includeAnalysis = 1;
   int verboseAnalysis = 0;
   int statistics = 0;
@@ -1672,7 +1998,7 @@ PythonMatch(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
   }
   
   pmr = firstGame->plNext->p;
-  {                                       assert( pmr->mt == MOVE_GAMEINFO ); }
+  assert( pmr->mt == MOVE_GAMEINFO );
   g = &pmr->g;
 
   if( !PyArg_ParseTupleAndKeywords(args, keywds, "|iiii", kwlist,
@@ -1691,6 +2017,11 @@ PythonMatch(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
 
   PushLocale("C");
 
+  if (g->i != 0)
+  {
+      PyErr_SetString(PyExc_StandardError, "First game missing from match");
+	  return 0;
+  }
   assert( g->i == 0 );
 
   /* W,X,0 
@@ -1720,6 +2051,19 @@ PythonMatch(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
   addProperty(matchInfoDict, "annotator", mi.pchAnnotator);
   addProperty(matchInfoDict, "comment", mi.pchComment);
 
+  {	/* Work out the result (-1,0,1) - (p0 win, unfinished, p1 win) */
+	int result = 0;
+  	int anFinalScore[2];
+	if (getFinalScore(anFinalScore))
+	{
+		if (anFinalScore[0] > g->nMatch)
+			result = -1;
+		else if (anFinalScore[1] > g->nMatch)
+			result = 1;
+	}
+	DictSetItemSteal(matchInfoDict, "result", PyInt_FromLong(result));
+  }
+
   {
     char* v[] = { "Standard", "Nackgammon", "Hypergammon1", "Hypergammon2",
 		  "Hypergammon3" };
@@ -1785,7 +2129,7 @@ PythonMatch(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
     DictSetItemSteal(matchDict, "games", matchTuple);
 
     if( statistics ) {
-      PyObject* s = PyGameStats(&scMatch);
+      PyObject* s = PyGameStats(&scMatch,TRUE,g->nMatch);
 
       if( s ) {
 	DictSetItemSteal(matchDict, "stats", s);
@@ -1883,7 +2227,7 @@ PythonNavigate(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
       const moverecord* r = (const moverecord*)(plLastMove->plNext->p);
       
       if( r->mt == MOVE_NORMAL ) {
-	memcpy(ms.anDice, r->n.anRoll, sizeof(ms.anDice));
+	memcpy(ms.anDice, r->anDice, sizeof(ms.anDice));
       }
     }
     
@@ -1909,38 +2253,176 @@ PythonNavigate(PyObject* self IGNORE, PyObject* args, PyObject* keywds)
 PyMethodDef gnubgMethods[] = {
 
   { "board", PythonBoard, METH_VARARGS,
-    "Get the current board" },
+    "Get the current board\n"
+    "    arguments: none\n"
+    "    returns: tuple of two lists of 25 ints:\n"
+    "        pieces on points 1..24 and the bar" },
   { "command", PythonCommand, METH_VARARGS,
-    "Execute a command" },
+    "Execute a command\n"
+    "    arguments: string containing command\n"
+    "    returns: nothing" },
+  { "cfevaluate", PythonEvaluateCubeful, METH_VARARGS,
+    "Cubeful evaluation\n"
+    "    arguments: [board] [cube-info] [eval-context]\n"
+    "       board = tuple ( see \"board\" )\n"
+    "       cube-info = dictionary: 'jacoby'=>0/1, 'crawford'=>0/1\n"
+    "           'move'=>0/1, 'beavers'=>0/1, 'cube'=>cube val (int)\n"
+    "           'matchto'=>length (0 for money), 'bgv'=>0..4\n"
+    "           'score'=>(int, int), 'gammonprice'=(float[4])\n"
+    "       eval-context = dictionary: 'cubeful'=>0/1, 'plies'=>int,\n"
+    "           'reduced'=>0/1, 'deterministic'=> 0/1, 'noise'->float\n"
+    "    returns: evaluation = tuple (floats optimal, nodouble, take, drop, int recommendation, String recommendationtext)" },
   { "evaluate", PythonEvaluate, METH_VARARGS,
-    "Cubeless evaluation" },
+    "Cubeless evaluation]n"
+    "    arguments: [board] [cube-info] [eval context]\n"
+    "         see 'cfevaluate'\n"
+    "    returns tuple(floats P(win), P(win gammon), P(win backgammnon)\n"
+    "         P(lose gammon), P(lose backgammon), cubeless equity)" },
   { "evalcontext", PythonEvalContext, METH_VARARGS,
-    "make a evalcontext" },
+    "make an evalcontext\n"
+    "    argument: tuple ( 4 int, float )\n"
+    "    returns:  eval-context ( see 'cfevaluate' )"   },
   { "eq2mwc", PythonEq2mwc, METH_VARARGS,
-    "convert equity to MWC" },
+    "convert equity to MWC\n"
+    "    argument: [float equity], [cube-info]\n"
+    "         defaults equity = 0.0, cube-info see 'cfevaluate'\n"
+    "    return float mwc" },
+  { "findbestmove", PythonFindBestMove, METH_VARARGS,
+    "Find the best move\n"
+    "    arguments: [board] [cube-info] [eval-context]\n"
+    "        see 'cfevaluate'\n"
+    "    returns: tuple( ints point from, point to, \n"
+    "        unused moves are set to zero" },
   { "mwc2eq", PythonMwc2eq, METH_VARARGS,
-    "convert MWC to equity" },
+    "convert MWC to equity\n"
+    "    argument: [float match-winning-chance], [cube-info]\n"
+    "         defaults mwc = 0.0, cube-info see 'cfevaluate'\n"
+    "    returns: float equity" },
+  { "matchchecksum", PythonMatchChecksum, METH_VARARGS,
+    "Calculate checksum for current match\n"
+    "    arguments: none\n"
+    "    returns: MD5 digest as 32 char hex string" },
   { "cubeinfo", PythonCubeInfo, METH_VARARGS,
-    "Make a cubeinfo" },
+    "Make a cubeinfo\n"
+    "    arguments: [cube value, cube owner = 0/1, player on move = 0/1\n"
+    "        match length (0 = money), score (tuple int, int), \n"
+    "        is crawford = 0/1, bg variant = 0/5\n"
+    "    returns cube-info dictionary ( see 'cfevaluate' )" },
   { "met", PythonMET, METH_VARARGS,
-    "return the current match equity table" },
+    "return the current match equity table\n"
+    "   arguments: [max score]\n"
+    "    returns: list of list n of list n (rows of pre-crawford table\n"
+    "        list 2 of list n of post-crawford for player 0/player 1" },
   { "positionid", PythonPositionID, METH_VARARGS,
-    "return position ID from board" },
+    "return position ID from board\n"
+    "    arguments: [board] ( see 'cfevaluate' )\n"
+    "    returns: position ID as string" },
   { "positionfromid", PythonPositionFromID, METH_VARARGS,
-    "return board from position ID" },
+    "return board from position ID\n"
+    "    arguments: [position ID as string]\n"
+    "    returns: board ( see 'cfevaluate' )" },
   { "positionbearoff", PythonPositionBearoff, METH_VARARGS,
-    "return the bearoff id for the given position" },
+    "return the bearoff id for the given position\n"},
   { "positionfrombearoff", PythonPositionFromBearoff, METH_VARARGS,
-    "return the board from the given bearoff id" },
+    "return the board from the given bearoff id\n"
+    "    arguments: [bearoff id] [no. chequers] [no. points]\n"
+    "    returns: board ( see 'cfevaluate' )" },
   { "positionkey", PythonPositionKey, METH_VARARGS,
-    "return key for position" },
+    "return key for position\n"
+    "    arguments: [ board ] ( see 'cfevaluate' )\n"
+    "    returns: tuple (10 ints)" },
   { "positionfromkey", PythonPositionFromKey, METH_VARARGS,
-    "return position from key" },
+    "return position from key\n"
+    "    arguments: [ list of 10 ints] \n"
+    "    returns: board ( see 'cfevaluate' )" },
   { "match", (PyCFunction)PythonMatch, METH_VARARGS|METH_KEYWORDS,
-    "Get the current match" },
+    "Get the current match\n"
+    "    arguments: [ include-analysis = 0/1, include-boards = 0/1,\n"
+    "       include-statistics = 0/1, verbose = 0/1 ]\n"
+    "    returns: dictionary of match info:\n"
+    "       'games' => list of dictionaries, one per game\n"
+    "         'info' => dictionary\n"
+    "           'points-won'=>int, 'score-X'=> int, 'winner'=>'X'/'O'\n"
+    "           'resigned'=> 0/1, 'score-O'=> int\n"
+    "         'stats' => dictionary\n"
+    "           'X' => player 0 dictionary of stats\n"
+    "             'cube'=>dictionary\n"
+    "               'close-cube'=>int, 'err-missed-double-above-cp-cost'=>float\n"
+    "               'err-missed-double-above-cp-skill'=>float\n"
+    "               'err-missed-double-below-cp-cost'=>float\n"
+    "               'err-missed-double-below-cp-skill'=>float\n"
+    "               'err-wrong-double-above-tg-cost'=>float\n"
+    "               'err-wrong-double-above-tg-skill'=>float\n"
+    "               'err-wrong-double-below-dp-cost'=>float\n"
+    "               'err-wrong-double-below-dp-skill'=>float\n"
+    "               'err-wrong-drop-cost'=>float, 'err-wrong-drop-skill'=>float\n"
+    "               'err-wrong-take-cost'=>float, 'err-wrong-take-skill'=>float\n"
+    "               'error-cost'=>float, 'error-skill'=>float\n"
+    "               'missed-double-above-cp'=>int, 'missed-double-below-cp'=>int\n"
+    "               'n-doubles'=>int, 'n-drops'=>int, 'n-takes': =>int,\n"
+    "               'total-cube'=>int, 'wrong-double-above-tg'=>int\n"
+    "               'wrong-double-below-dp'=>int, 'wrong-drop'=>int,\n"
+    "               'wrong-take'=>int\n"
+    "             'moves'=>dictionary\n"
+    "               'marked'=>dictionarly\n"
+    "                  'good'=> int, 'unmarked'->int, 'doubtful'=>int,\n"
+    "                  'bad'=>int, 'very bad'=>int\n"
+    "               'total-moves'=>int, 'unforced-moves'=>int,\n"
+    "               'error-cost'=>float, 'error-skill'=>float\n"
+    "             'dice'=>dictionary\n"
+    "               'actual-result;=>float, 'cube'=>float, 'luck'=>float,\n"
+    "               'luck-cost'=>float, 'luck-adjusted-result'=>float\n"
+    "               'marked-rolls'=>dictionary\n"
+    "                 'verygood'=>int, 'good'=>int, 'unmarked'=>int,\n"
+    "                 'bad'=>int, 'verybad'=>int\n"
+    "              'time'=>dictionary\n"
+    "                'time-penalty'=>int, 'time-penalty-cost'=>float,\n"
+    "                'time-penalty-skill'=>float\n"
+    "            'O'=>  player 1 dicrtionary of stats - see 'X; above\n"
+    "         'game'=>list of dictionaries, one per move\n"
+    "            'dice'=>(int, int), move=>((int, int),[(int, int),...])\n"
+    "            'player'=>'X'/'O', 'board'=>board-id-string,\n"
+    "            'action'=>'move', 'double', 'take', 'drop', 'resign'\n"
+    "            'analysis'=>dictionary\n"
+    "              'imove'=>int index of move in list of analysed moves\n"
+    "              'moves'=>list of dictionaries, one per analysed move\n"
+    "                 'score'=>equity for move, 'type'=>'eval''rollout'\n"
+    "                 'move'=>((int, int),[(int, int),...])\n"
+    "                 'probs'=> tuple (5 floats - P(win),\n"
+    "                 P(win gammon)..P(lose bkgammon)\n"
+    "                 [ 'evalcontext' = dictionary describing eval context\n"
+    "                        if not default\n"
+    "        'match-info' = dictionary \n"
+    "          'X' => player-0 dictionary\n"
+    "            'rating' = rating if known\n"
+    "            'name'   = player name\n"
+    "          'O' => player-1 dictionary, as 'X', above\n" 
+    "          'date'=>(tuple dd, mm, yyyy)\n"
+    "          'default-eval-context' = dictionary\n"
+    "            'plies'=> int, 'deterministic'=>0/1, 'noise'=>float,\n"
+    "            'cubeful'=>0/1\n"
+    "          'match_length' = int\n"
+    "          'result' =>0/1\n"
+    "          'rules' = 'Crawford'/whatever\n"
+    "          'variation' => 'Standard' or whatever\n" },
   { "navigate", (PyCFunction)PythonNavigate, METH_VARARGS|METH_KEYWORDS,
-    "" },
-  { "nextturn", (PyCFunction) PythonNextTurn, METH_VARARGS, "" },
+    "go to a position in a match or session'n"
+    "    arguments: no args = go to start of match/session\n"
+    "         [ game=offset] go forward/backward n games\n"
+    "         [record=offset] go gorward/backward n moves'n"
+    "    returns: None if no change, tuple( games moved, records moved)" },
+  { "nextturn", (PyCFunction) PythonNextTurn, METH_VARARGS, 
+    "play one turn\n"
+    "    arguments: none\n"
+    "    returns: None" },
+  { "luckrating", (PyCFunction) PythonLuckRating, METH_VARARGS, 
+    "convert a luck per move amount into a rating 0..5 for very unlucky to very lucky\n"
+    "    arguments: float luck per move\n"
+    "    returns: int 0..5" },
+  { "errorrating", (PyCFunction) PythonErrorRating, METH_VARARGS, 
+    "convert an error per move amount to a rating 0 = awful..7=supernatural\n"
+    "    arguments: float error per move\n"
+    "    returns: int\n" },
   { NULL, NULL, 0, NULL }
 
 };
@@ -1973,19 +2455,31 @@ PythonReadline( char *p ) {
 #endif
 
 extern void
-PythonInitialise( const char *argv0, const char *szDir ) {
+PythonInitialise( const char *szDir ) {
 
-  FILE *pf;
   char *pch;
-  char scriptDir[BIG_PATH];
-  strcpy( scriptDir, ( szDir && *szDir ) ? szDir : "." );
-  strcat(scriptDir, "/scripts");
+  static char workingDir[BIG_PATH];
+  getcwd(workingDir, BIG_PATH);
 
-  Py_SetProgramName( (char *) argv0 );
+#if WIN32
+{	/* Setup python to look in the pythonlib directory if present */
+	char pythonDir[BIG_PATH];
+	strcpy(pythonDir, workingDir);
+	strcat(pythonDir, "/PythonLib");
+	if (!access(pythonDir, F_OK))
+	{	/* Set Pyton to use this directory */
+		char buf[BIG_PATH + 100];
+		sprintf(buf, "PYTHONPATH=%s", pythonDir);
+		_putenv(buf);
+		sprintf(buf, "PYTHONROOT=%s", pythonDir);
+		_putenv(buf);
+	}
+}
+#endif
+
+  Py_SetProgramName(workingDir);
   Py_Initialize();
 
-  PySys_SetArgv( 1, (char **) &argv0 );
-
   /* ensure that python know about our gnubg module */
   Py_InitModule( "gnubg", gnubgMethods );
   PyRun_SimpleString( "import gnubg\n" );
@@ -1996,14 +2490,9 @@ PythonInitialise( const char *argv0, const char *szDir ) {
 #endif
 
   /* run gnubg.py start up script */
-
-  if ( ( pch = PathSearch( "gnubg.py", scriptDir ) ) &&
-       ( pf = fopen( pch, "r" ) ) ) {
-    PyRun_AnyFile( pf, pch );
-  }
-  if ( pch )
-    free( pch );
-
+  pch = g_strdup( "gnubg.py" );
+  CommandLoadPython( pch );
+  g_free( pch );
 }
 
 extern void
