@@ -16,34 +16,32 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gnubgmodule.h,v 1.5 2003/08/29 18:38:12 jsegrave Exp $
+ * $Id: gnubgmodule.h,v 1.10 2005/02/10 10:31:12 Superfly_Jon Exp $
  */
 
 #ifndef _PYTHONMODULE_H_
 #define _PYTHONMODULE_H_
 
-#if HAVE_CONFIG_H
-#include "config.h"
 #if USE_PYTHON
-#undef HAVE_FSTAT
-#endif
-#endif
 
-#if USE_PYTHON
+/* Python.h defines HAVE_FSTAT (2.3) and HAVE_SETVBUF (2.4) so ignore and redefine below */
+#undef HAVE_SETVBUF
+#undef HAVE_LIMITS_H
+#undef HAVE_FSTAT
 #include <Python.h>
-#if HAVE_CONFIG_H
+#undef HAVE_SETVBUF
+#undef HAVE_LIMITS_H
 #undef HAVE_FSTAT
 #include "config.h"
-#endif
-
-extern PyMethodDef gnubgMethods[];
 
 extern void
-PythonInitialise( const char *arg0, const char *szDir );
+PythonInitialise( const char *szDir );
 
 extern void
 PythonShutdown();
 
+extern PyObject *PythonMatchChecksum( PyObject* self, PyObject *args );
+
 #endif /* USE_PYTHON */
 
 #endif /* _PYTHONMODULE_H_ */
