@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkcube.c,v 1.38 2004/01/29 15:26:13 uid68519 Exp $
+ * $Id: gtkcube.c,v 1.47 2006/09/13 21:55:07 c_anthon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -39,7 +39,8 @@
 #include "rollout.h"
 #include "gtkgame.h"
 #include "gtkcube.h"
-#include "i18n.h"
+#include "gtkwindows.h"
+#include <glib/gi18n.h>
 #include "progress.h"
 #include "format.h"
 
@@ -723,6 +724,7 @@ CubeAnalysisRollout ( GtkWidget *pw, cubehintdata *pchd ) {
   GetMatchStateCubeInfo( &ci, &pchd->ms );
 
   FormatCubePositions( &ci, asz );
+  GTKSetCurrentParent(pw);
   RolloutProgressStart( &ci, 2, aarsStatistics, &pes->rc, asz, &p );
 
   if ( GeneralCubeDecisionR ( aarOutput, aarStdDev, aarsStatistics,
@@ -782,8 +784,14 @@ static void
 CubeAnalysisEvalPly ( GtkWidget *pw, cubehintdata *pchd ) {
 
   char *szPly = gtk_object_get_data ( GTK_OBJECT ( pw ), "user_data" );
-  evalcontext ec = { TRUE, 0, 0, TRUE, 0.0 };
+  evalcontext ec = {0, 0, 0, TRUE, 0.0};
 
+  ec.fCubeful = esAnalysisCube.ec.fCubeful;
+#if defined( REDUCTION_CODE )
+  ec.nReduced = esAnalysisCube.ec.nReduced;
+#else
+  ec.fUsePrune = esAnalysisCube.ec.fUsePrune;
+#endif
   ec.nPlies = atoi ( szPly );
 
   EvalCube ( pchd, &ec );
@@ -798,26 +806,24 @@ CubeAnalysisEval ( GtkWidget *pw, cubehintdata *pchd ) {
 }
 
 static void
-CubeAnalysisEvalSettings ( GtkWidget *pw, void *unused ) {
+CubeAnalysisEvalSettings ( GtkWidget *pw, void *unused )
+{
+	GTKSetCurrentParent(pw);
+	SetEvaluation ( NULL, 0, NULL );
 
-  SetEvaluation ( NULL, 0, NULL );
-
-#if GTK_CHECK_VERSION(2,0,0)
   /* bring the dialog holding this button to the top */
   gtk_window_present ( GTK_WINDOW ( gtk_widget_get_toplevel( pw ) ) );
-#endif
 
 }
 
 static void
-CubeAnalysisRolloutSettings ( GtkWidget *pw, void *unused ) {
+CubeAnalysisRolloutSettings ( GtkWidget *pw, void *unused )
+{
+	GTKSetCurrentParent(pw);
+	SetRollouts ( NULL, 0, NULL );
 
-  SetRollouts ( NULL, 0, NULL );
-
-#if GTK_CHECK_VERSION(2,0,0)
   /* bring the dialog holding this button to the top */
   gtk_window_present ( GTK_WINDOW ( gtk_widget_get_toplevel( pw ) ) );
-#endif
 
 }
 
