@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkoptions.c,v 1.16 2004/01/29 11:30:22 uid68519 Exp $
+ * $Id: gtkoptions.c,v 1.30 2006/09/21 22:24:05 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <assert.h>
 
 #include <gtk/gtk.h>
 
@@ -37,13 +38,14 @@
 #include "eval.h"
 #include "dice.h"
 #include "gtkgame.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "sound.h"
 #include "drawboard.h"
 #include "matchequity.h"
 #include "format.h"
 #include "gtkboard.h"
 #include "renderprefs.h"
+#include "gtkwindows.h"
 
 typedef struct _optionswidget {
 
@@ -70,13 +72,13 @@ typedef struct _optionswidget {
       *pwSoundNAS, *pwSoundNormal, *pwSoundWindows, *pwSoundQuickTime,
       *pwSoundSettings;
 
-  GtkWidget *pwIllegal, *pwUseDiceIcon, *pwShowIDs, *pwShowPips,
+  GtkWidget *pwIllegal, *pwUseDiceIcon, *pwShowIDs, *pwShowPips, *pwShowEPCs,
       *pwAnimateNone, *pwAnimateBlink, *pwAnimateSlide, *pwBeepIllegal,
       *pwHigherDieFirst, *pwSetWindowPos, *pwDragTargetHelp;
   GtkAdjustment *padjSpeed;
 
   GtkWidget *pwCheat, *pwCheatRollBox, *apwCheatRoll[ 2 ];
-  GtkWidget *pwGotoFirstGame;
+  GtkWidget *pwGotoFirstGame, *pwGameListStyles;
   GtkWidget *pwLangMenu;
 
   GtkWidget *pwSconyers15x15DVD;
@@ -175,11 +177,17 @@ static char *aaszLang[][ 2 ] = {
     { N_("German"),	    "de_DE" },
     { N_("Icelandic"),      "is_IS" },
     { N_("Italian"),	    "it_IT" },
-    { N_("Japanese"),	    "ja_JA" },
-    { N_("Turkish"),        "tr_TR" },
+    { N_("Japanese"),	    "ja_JP" },
+    { N_("Russian"),	    "ru_RU" },
+    { N_("Turkish"),	    "tr_TR" },
     { NULL, NULL }
 };
 
+static char *aszTutor[] = {
+	N_("Doubtful"), N_("Bad"), N_("Very bad"), NULL
+};
+
+
 static GtkWidget *OptionsPages( optionswidget *pow ) {
 
     static char *aszRNG[] = {
@@ -204,8 +212,6 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
            "as dice. When the entire file has been read, it will be "
            "rewinded. You will be prompted for a file "
            "when closing the options dialog.")
-    }, *aszTutor[] = {
-	N_("Doubtful"), N_("Bad"), N_("Very bad")
     };
 
   static char *aszCheatRoll[] = {
@@ -238,7 +244,9 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     int cCache;
     int i, nRandom;
 
-    InitRNG( &nRandom, FALSE, rngCurrent );
+    BoardData *bd = BOARD( pwBoard )->board_data;
+
+    InitRNG( &nRandom, NULL, FALSE, rngCurrent );
     
     pwn = gtk_notebook_new();
     gtk_container_set_border_width( GTK_CONTAINER( pwn ), 8 );
@@ -385,6 +393,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "games, not matches."), NULL );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -407,6 +416,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "matches."), NULL);
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -484,22 +494,19 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "the \"Analysis\" settings."), NULL );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
 
     gtk_box_pack_start (GTK_BOX (pwhbox), gtk_label_new( _("Warning level:") ),
 			FALSE, FALSE, 0);
-    pow->pwTutorSkill = gtk_option_menu_new ();
+	pow->pwTutorSkill = gtk_combo_box_new_text();
     gtk_box_pack_start (GTK_BOX (pwhbox), pow->pwTutorSkill, FALSE, FALSE, 0);
-    pwm = gtk_menu_new ();
-    for( ppch = aszTutor; *ppch; ppch++ )
-	gtk_menu_append( GTK_MENU( pwm ),
-			 pw = gtk_menu_item_new_with_label(
-			     gettext( *ppch ) ) );
-    gtk_widget_show_all( pwm );
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (pow->pwTutorSkill), pwm );
-    gtk_option_menu_set_history (GTK_OPTION_MENU (pow->pwTutorSkill), 0);
+	for( ppch = aszTutor; *ppch; ppch++ ) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (pow->pwTutorSkill), gettext( *ppch ));
+	}
+	assert(nTutorSkillCurrent >= 0 && nTutorSkillCurrent <= 2) ;
     gtk_tooltips_set_tip (ptt, pwev,
 			  _("Specify how bad GNU Backgammon must think a "
 			    "decision is before questioning you about a "
@@ -531,6 +538,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			  NULL );
   
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -553,7 +561,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 	gtk_check_button_new_with_label( _("Show dice below board when human "
 					   "player on roll") );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwUseDiceIcon ),
-				  fGUIDiceArea );
+				  bd->rd->fDiceArea );
     gtk_box_pack_start( GTK_BOX( pwvbox ), pow->pwUseDiceIcon, FALSE, FALSE,
 			0 );
     gtk_tooltips_set_tip( ptt, pow->pwUseDiceIcon,
@@ -567,7 +575,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     pow->pwShowIDs = gtk_check_button_new_with_label(
 	_("Show Position ID and Match ID above board") );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwShowIDs ),
-				  fGUIShowIDs );
+				  bd->rd->fShowIDs);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pow->pwShowIDs, FALSE, FALSE, 0 );
     gtk_tooltips_set_tip( ptt, pow->pwShowIDs,
 			  _("Two entry fields will be shown above the board, "
@@ -586,6 +594,16 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "all off) will be shown below the scores."),
 			  NULL );
 	
+    pow->pwShowEPCs = gtk_check_button_new_with_label(
+	_("Show EPCs below board") );
+    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwShowEPCs ),
+				  fGUIShowEPCs );
+    gtk_box_pack_start( GTK_BOX( pwvbox ), pow->pwShowEPCs, FALSE, FALSE, 0 );
+    gtk_tooltips_set_tip( ptt, pow->pwShowEPCs,
+			  _("The \"effective pip counts\" "
+                            "will be shown below the scores."),
+			  NULL );
+	
     pwAnimBox = gtk_hbox_new( FALSE, 0 );
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwAnimBox, FALSE, FALSE, 0 );
     
@@ -626,6 +644,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "the board between the points."), NULL );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwAnimBox ), pwev, FALSE, FALSE, 0 );    
     pwSpeed = gtk_hbox_new( FALSE, 0 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwSpeed );
@@ -633,11 +652,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     pow->padjSpeed = GTK_ADJUSTMENT( gtk_adjustment_new( nGUIAnimSpeed, 0, 7,
 							 1, 1, 0 ) );
     pwScale = gtk_hscale_new( pow->padjSpeed );
-#if GTK_CHECK_VERSION(2,0,0)
     gtk_widget_set_size_request( pwScale, 100, -1 );
-#else
-    gtk_widget_set_usize ( GTK_WIDGET ( pwScale ), 100, -1 );
-#endif
     gtk_scale_set_draw_value( GTK_SCALE( pwScale ), FALSE );
     gtk_scale_set_digits( GTK_SCALE( pwScale ), 0 );
 
@@ -725,6 +740,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     /* number of digits in output */
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -767,6 +783,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     gtk_container_add( GTK_CONTAINER( pwp ), pwvbox );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -799,7 +816,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     gtk_box_pack_start (GTK_BOX (pwhoriz),
 			gtk_label_new( _("Current:") ),
 			FALSE, FALSE, 2);
-    pwLabelFile = gtk_label_new( miCurrent.szFileName );
+    pwLabelFile = gtk_label_new( (char*) miCurrent.szFileName );
     gtk_box_pack_end(GTK_BOX (pwhoriz), pwLabelFile,
 		     FALSE, FALSE, 2);
     pow->pwLoadMET = gtk_button_new_with_label (_("Load..."));
@@ -1156,6 +1173,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     gtk_container_add( GTK_CONTAINER( pwp ), pwvbox );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1175,6 +1193,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "will retain more existing knowledge."), NULL );
 
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1196,6 +1215,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			  NULL );
     
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1250,6 +1270,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     /* path */
     
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwb ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1315,6 +1336,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
     /* path */
     
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwb ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1393,6 +1415,7 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
 			    "matches or sessions, to save memory."), NULL );
   
     pwev = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(pwev), FALSE);
     gtk_box_pack_start( GTK_BOX( pwvbox ), pwev, FALSE, FALSE, 0 );
     pwhbox = gtk_hbox_new( FALSE, 4 );
     gtk_container_add( GTK_CONTAINER( pwev ), pwhbox );
@@ -1428,6 +1451,18 @@ static GtkWidget *OptionsPages( optionswidget *pow ) {
                             "show the first move in the first game"),
                           NULL );
 
+    /* display styles in game list */
+
+    pow->pwGameListStyles = gtk_check_button_new_with_label (
+	_("Display colours for marked moves in game list"));
+    gtk_box_pack_start (GTK_BOX (pwvbox), pow->pwGameListStyles,
+			FALSE, FALSE, 0);
+    gtk_tooltips_set_tip( ptt, pow->pwGameListStyles,
+			  _("This option controls whether moves in the "
+                            "game list window are shown in different "
+                            "colours depending on their analysis"),
+                          NULL );
+
     /* language preference */
 
     pwhbox = gtk_hbox_new( FALSE, 4 );
@@ -1480,6 +1515,7 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
   int n, cCache;
   int i;
   char *pch;
+  BoardData *bd = BOARD( pwBoard )->board_data;
 
   gtk_widget_hide( gtk_widget_get_toplevel( pw ) );
 
@@ -1499,27 +1535,33 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
 	  UserCommand( "set tutor eval on");
   }
 
-  n = gtk_option_menu_get_history(GTK_OPTION_MENU( pow->pwTutorSkill ));
-  switch ( n ) {
-  case 0:
-	if (TutorSkill != SKILL_DOUBTFUL)
-	  UserCommand ("set tutor skill doubtful");
-	break;
+  {
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (pow->pwTutorSkill));
+	
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX(pow->pwTutorSkill), &iter)) {
+		gchar *selection;
+		gtk_tree_model_get (model, &iter, 0, &selection, -1);
 
-  case 1:
-	if (TutorSkill != SKILL_BAD)
-	  UserCommand ("set tutor skill bad");
-	break;
-
-  case 2:
-	if (TutorSkill != SKILL_VERYBAD)
-	  UserCommand ("set tutor skill very bad");
-	break;
-
-  default:
-	UserCommand ("set tutor skill doubtful");
+		if (! strcmp(selection, gettext(aszTutor[0]))) {			/* N_("Doubtful") */
+			if (TutorSkill != SKILL_DOUBTFUL)
+			  UserCommand ("set tutor skill doubtful");
+		} else if (! strcmp(selection, gettext(aszTutor[1]))) {	/* N_("Bad") */
+			if (TutorSkill != SKILL_BAD)
+			  UserCommand ("set tutor skill bad");
+		} else if (! strcmp(selection, gettext(aszTutor[2]))) {	/* N_("Very Bad") */
+			if (TutorSkill != SKILL_VERYBAD)
+				UserCommand ("set tutor skill very bad");
+		} else {
+			/* assert(FALSE); Unknown Selection, defaulting */
+			if (TutorSkill != SKILL_DOUBTFUL)
+				UserCommand ("set tutor skill doubtful");
+		}
+		g_free(selection);
+	}	
   }
-  gtk_widget_set_sensitive( pow->pwTutorSkill, n );
 
   CHECKUPDATE(pow->pwCubeUsecube,fCubeUse, "set cube use %s")
   CHECKUPDATE(pow->pwCubeJacoby,fJacoby, "set jacoby %s")
@@ -1540,7 +1582,7 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
   CHECKUPDATE(pow->pwOutputGWC,fOutputWinPC, "set output winpc %s")
   CHECKUPDATE(pow->pwOutputMWCpst,fOutputMatchPC, "set output matchpc %s")
 
-  if(( n = pow->padjDigits->value ) != fOutputDigits ){
+  if(( n = (int)pow->padjDigits->value ) != fOutputDigits ){
     sprintf(sz, "set output digits %d", n );
     UserCommand(sz); 
   }
@@ -1580,17 +1622,17 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
   CHECKUPDATE(pow->pwConfStart,fConfirm, "set confirm new %s")
   CHECKUPDATE(pow->pwConfOverwrite,fConfirmSave, "set confirm save %s")
   
-  if(( n = pow->padjCubeAutomatic->value ) != cAutoDoubles){
+  if(( n = (int)pow->padjCubeAutomatic->value ) != cAutoDoubles){
     sprintf(sz, "set automatic doubles %d", n );
     UserCommand(sz); 
   }
 
-  if(( n = pow->padjCubeBeaver->value ) != nBeavers){
+  if(( n = (int)pow->padjCubeBeaver->value ) != nBeavers){
     sprintf(sz, "set beavers %d", n );
     UserCommand(sz); 
   }
   
-  if(( n = pow->padjLength->value ) != nDefaultLength){
+  if(( n = (int)pow->padjLength->value ) != nDefaultLength){
     sprintf(sz, "set matchlength %d", n );
     UserCommand(sz); 
   }
@@ -1637,7 +1679,7 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
       break;
     case 8:
       GTKFileCommand( _("Select file with dice"), NULL, 
-                      "set rng file", NULL, 0 );
+                      "set rng file", NULL, FDT_NONE_OPEN, PATH_NULL );
       break;
     default:
       break;
@@ -1650,37 +1692,43 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
 
   EvalCacheStats( NULL, &cCache, NULL, NULL );
 
-  if((n = pow->padjCache->value) != cCache) {
+  if((n = (int)pow->padjCache->value) != cCache) {
     sprintf(sz, "set cache %d", n );
     UserCommand(sz); 
   }
 
-  if((n = pow->padjDelay->value) != nDelay) {
+  if((n = (int)pow->padjDelay->value) != nDelay) {
     sprintf(sz, "set delay %d", n );
     UserCommand(sz); 
   }
 
   if( pow->padjLearning->value != rAlpha ) 
   { 
-     lisprintf(sz, "set training alpha %0.3f", pow->padjLearning->value); 
+     gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
+     sprintf(sz, "set training alpha %s", 
+       g_ascii_formatd( buf, G_ASCII_DTOSTR_BUF_SIZE, "%0.3f", pow->padjLearning->value)); 
      UserCommand(sz); 
   }
 
   if( pow->padjAnnealing->value != rAnneal ) 
   { 
-     lisprintf(sz, "set training anneal %0.3f", pow->padjAnnealing->value); 
+     gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
+     sprintf(sz, "set training anneal %s",
+       g_ascii_formatd( buf, G_ASCII_DTOSTR_BUF_SIZE, "%0.3f", pow->padjAnnealing->value)); 
      UserCommand(sz); 
   }
 
   if( pow->padjThreshold->value != rThreshold ) 
   { 
-     lisprintf(sz, "set training threshold %0.3f", pow->padjThreshold->value); 
+     gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
+     sprintf(sz, "set training threshold %s", 
+       g_ascii_formatd( buf, G_ASCII_DTOSTR_BUF_SIZE, "%0.3f", pow->padjThreshold->value)); 
      UserCommand(sz); 
   }
   
   if( pow->fChanged == 1 ) 
   { 
-     n = pow->padjSeed->value;
+     n = (int)pow->padjSeed->value;
      sprintf(sz, "set seed %d", n); 
      UserCommand(sz); 
   }
@@ -1721,9 +1769,10 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
 #endif
 
   CHECKUPDATE( pow->pwIllegal, fGUIIllegal, "set gui illegal %s" )
-  CHECKUPDATE( pow->pwUseDiceIcon, fGUIDiceArea, "set gui dicearea %s" )
-  CHECKUPDATE( pow->pwShowIDs, fGUIShowIDs, "set gui showids %s" )
+  CHECKUPDATE( pow->pwUseDiceIcon, bd->rd->fDiceArea, "set gui dicearea %s" )
+  CHECKUPDATE( pow->pwShowIDs, bd->rd->fShowIDs, "set gui showids %s" )
   CHECKUPDATE( pow->pwShowPips, fGUIShowPips, "set gui showpips %s" )
+  CHECKUPDATE( pow->pwShowEPCs, fGUIShowEPCs, "set gui showepcs %s" )
   CHECKUPDATE( pow->pwBeepIllegal, fGUIBeep, "set gui beep %s" )
   CHECKUPDATE( pow->pwHigherDieFirst, fGUIHighDieFirst,
 	       "set gui highdiefirst %s" )
@@ -1732,18 +1781,18 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
   CHECKUPDATE( pow->pwDragTargetHelp, fGUIDragTargetHelp,
 	       "set gui dragtargethelp %s" )
 
+{
 #if USE_BOARD3D
-	if (rdAppearance.fDisplayType == DT_3D)
-		updateDiceOccPos(BOARD( pwBoard )->board_data);
+	if (bd->rd->fDisplayType == DT_3D)
+		updateDiceOccPos(bd, &bd->bd3d);
 	else
 #endif
 	if( GTK_WIDGET_REALIZED( pwBoard ) )
 	{
-		BoardData* bd = BOARD( pwBoard )->board_data;
 		board_create_pixmaps( pwBoard, bd );
 		gtk_widget_queue_draw( bd->drawing_area );
 	}
-
+}
   if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( pow->pwAnimateNone ) )
       && animGUI != ANIMATE_NONE )
       UserCommand( "set gui animation none" );
@@ -1756,7 +1805,7 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
       && animGUI != ANIMATE_SLIDE )
       UserCommand( "set gui animation slide" );
 
-  if( ( n = pow->padjSpeed->value ) != nGUIAnimSpeed ) {
+  if( ( n = (int)pow->padjSpeed->value ) != nGUIAnimSpeed ) {
       sprintf( sz, "set gui animation speed %d", n );
       UserCommand( sz );
   }
@@ -1780,6 +1829,7 @@ static void OptionsOK( GtkWidget *pw, optionswidget *pow ){
   }
 
   CHECKUPDATE( pow->pwGotoFirstGame, fGotoFirstGame, "set gotofirstgame %s" )
+  CHECKUPDATE( pow->pwGameListStyles, fStyledGamelist, "set styledgamelist %s" )
       
   /* language preference */
 
@@ -1829,9 +1879,7 @@ OptionsSet( optionswidget *pow) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON( pow->pwTutorEvalAnalysis ),
 								 TRUE );
 
-  gtk_option_menu_set_history(GTK_OPTION_MENU( pow->pwTutorSkill ), 
-                                 nTutorSkillCurrent );
-
+	gtk_combo_box_set_active (GTK_COMBO_BOX (pow->pwTutorSkill), nTutorSkillCurrent);
   gtk_adjustment_set_value ( pow->padjCubeBeaver, nBeavers );
   gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwCubeUsecube ),
                                 fCubeUse );
@@ -1883,6 +1931,8 @@ OptionsSet( optionswidget *pow) {
                                 fConfirmSave );
   gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwGotoFirstGame ),
                                 fGotoFirstGame );
+  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( pow->pwGameListStyles ),
+                                fStyledGamelist );
 
   /* language preference */
 
@@ -1915,7 +1965,7 @@ GTKSetOptions( void ) {
   optionswidget ow;
 
   pwDialog = GTKCreateDialog( _("GNU Backgammon - General options"), DT_QUESTION,
-			     GTK_SIGNAL_FUNC( OptionsOK ), &ow );
+			     NULL, DIALOG_FLAG_MODAL, GTK_SIGNAL_FUNC( OptionsOK ), &ow );
   gtk_container_add( GTK_CONTAINER( DialogArea( pwDialog, DA_MAIN ) ),
  		        pwOptions = OptionsPages( &ow ) );
   gtk_widget_show_all( pwDialog );
@@ -1923,5 +1973,4 @@ GTKSetOptions( void ) {
   OptionsSet ( &ow );
  
   gtk_main();
-
 }
