@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtkrace.c,v 1.13 2004/02/24 10:20:47 uid68519 Exp $
+ * $Id: gtkrace.c,v 1.19 2006/09/21 22:24:05 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -37,10 +37,11 @@
 #include "eval.h"
 #include "gtkrace.h"
 #include "gtkgame.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "onechequer.h"
 #include "osr.h"
 #include "format.h"
+#include "gtkwindows.h"
 
 #if !HAVE_ERF
 extern double erf( double x );
@@ -162,15 +163,15 @@ KleinmanPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
 
     double rK = (double) ( nDiff + 4 ) / ( 2 * sqrt ( nSum -4 ) );
 
-    sz = g_strdup_printf ( "K = (diff+4)/(2 sqrt(sum-4)) = %8.4g", rK );
+    sz = g_strdup_printf ( _("K = (diff+4)/(2 sqrt(sum-4)) = %8.4g"), rK );
     gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
                          pw = gtk_label_new ( sz ),
                          FALSE, FALSE, 4 );
     gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
     g_free ( sz );
 
-    sz = g_strdup_printf ( "Cubeless winning chance (%s on roll) = "
-                           "0.5 (1+erf(K)) = %8.4g%%", 
+    sz = g_strdup_printf ( _("Cubeless winning chance (%s on roll) = "
+                             "0.5 (1+erf(K)) = %8.4g%%"), 
                            ap[ fMove ].szName,
                            100.0 * ( 0.5 * ( 1.0 + erf ( rK ) ) ) );
     gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
@@ -185,6 +186,79 @@ KleinmanPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
 
 }
 
+static GtkWidget *
+KeithPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
+
+  GtkWidget *pwvbox = gtk_vbox_new( FALSE, 4 );
+  GtkWidget *pwTable = gtk_table_new ( 2, 2, FALSE );
+  GtkWidget *pw;
+  GtkWidget *pwp = gtk_alignment_new( 0, 0, 0, 0 );
+  int i, j, nLeader, nTrailer;
+  int anPips[ 2 ];
+  float fL;
+  char *sz;
+  
+  gtk_container_set_border_width( GTK_CONTAINER( pwp ), 4 );
+  gtk_container_add( GTK_CONTAINER( pwp ), pwvbox );
+
+  /* 
+   * pip counts, diffs and sum 
+   */
+
+  gtk_box_pack_start ( GTK_BOX ( pwvbox ), pwTable, FALSE, FALSE, 4 );
+
+  /* pip counts */
+
+  PipCount ( anBoard, anPips );
+
+  for ( i = 0; i < 2; ++i ) {
+
+    j = fMove ? i : !i;
+
+    sz = g_strdup_printf ( _("Player %s"), ap[ i ].szName );
+    gtk_table_attach ( GTK_TABLE ( pwTable ), 
+                       pw = gtk_label_new ( sz ),
+                       0, 1, i, i + 1, 
+                       0, 0, 4, 4 );
+    gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
+    g_free ( sz );
+
+    sz = g_strdup_printf ( _("%d pips"), anPips[ j ] );
+    gtk_table_attach ( GTK_TABLE ( pwTable ), 
+                       gtk_label_new ( sz ),
+                       1, 2, i, i + 1, 
+                       0, 0, 4, 4 );
+    g_free ( sz );
+
+  }
+
+  /* separator */
+
+  gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
+                       gtk_hseparator_new() , FALSE, FALSE, 4 );
+
+  /* Keith count */
+
+  KeithCount ( anBoard, &nLeader, &nTrailer );
+
+  fL = (float) nLeader*8.0f /7.0f;
+  sz = g_strdup_printf ( "L = %d(%.1f)  T = %d -> %s, %s", 
+                         nLeader, fL, nTrailer,
+                         ( nTrailer >= ( fL -3 ) ? _("Redouble") :
+                           ( ( nTrailer >= ( fL -4 ) ) ? _("Double") : 
+                             _("No double") ) ),
+                         ( nTrailer >= ( fL - 2 ) ) ? 
+                         _("drop") : _("take") );
+  gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
+                       pw = gtk_label_new ( sz ),
+                       FALSE, FALSE, 4 );
+  gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
+  g_free ( sz );
+
+  return pwp;
+
+}
+
 static GtkWidget *
 ThorpPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
 
@@ -258,8 +332,8 @@ ThorpPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
     nDiff = 13;
   else if( nDiff < -37 )
     nDiff = -37;
-  sz = g_strdup_printf ( "Bower's interpolation: "
-                         "%d%% cubeless gwc (%s on roll)", 
+  sz = g_strdup_printf ( _("Bower's interpolation: "
+                           "%d%% cubeless gwc (%s on roll)"), 
                          74 + 2 * nDiff, ap[ fMove ].szName );
   gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
                        pw = gtk_label_new ( sz ),
@@ -310,7 +384,7 @@ EffectivePipCount( const float arPips[ 2 ], const float arWastage[ 2 ],
     gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
     g_free ( sz );
 
-    sz = g_strdup_printf ( _("%7.3f"), arPips[ fInvert ? !i : i ] );
+    sz = g_strdup_printf ( "%7.3f", arPips[ fInvert ? !i : i ] );
     gtk_table_attach ( GTK_TABLE ( pwTable ), 
                        pw = gtk_label_new ( sz ),
                        1, 2, i + 1, i + 2, 
@@ -331,13 +405,13 @@ EffectivePipCount( const float arPips[ 2 ], const float arWastage[ 2 ],
   }
 
   gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
-                       pw = gtk_label_new ( "EPC = Effective pip count = "
-                                            "Avg. rolls * 8.167" ),
+                       pw = gtk_label_new ( _("EPC = Effective pip count = "
+                                              "Avg. rolls * 8.167") ),
                        FALSE, FALSE, 0 );
   gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
 
   gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
-                       pw = gtk_label_new ( "Wastage = EPC - Pips" ),
+                       pw = gtk_label_new ( _("Wastage = EPC - Pips") ),
                        FALSE, FALSE, 0 );
   gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
 
@@ -364,7 +438,7 @@ OneChequerPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
   float arWastage[ 2 ];
   const float x = ( 2 * 3 + 3 * 4 + 4 * 5 + 4 * 6 + 6 * 7 +
               5* 8  + 4 * 9 + 2 * 10 + 2 * 11 + 1 * 12 + 
-              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0;
+              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0f;
 
   gtk_container_set_border_width( GTK_CONTAINER( pwp ), 4 );
   gtk_container_add( GTK_CONTAINER( pwp ), pwvbox );
@@ -380,9 +454,9 @@ OneChequerPage ( int anBoard[ 2 ][ 25 ], const int fMove ) {
   /* build widget */
 
   gtk_box_pack_start ( GTK_BOX ( pwvbox ), 
-                       pw = gtk_label_new ( "Number of rolls to bear off, "
-                                            "assuming each player has one "
-                                            "chequer only." ),
+                       pw = gtk_label_new ( _("Number of rolls to bear off, "
+                                              "assuming each player has one "
+                                              "chequer only.") ),
                        FALSE, FALSE, 4 );
   gtk_misc_set_alignment( GTK_MISC( pw ), 0, 0.5 );
 
@@ -469,14 +543,14 @@ static void
 PerformOSR ( GtkWidget *pw, racewidget *prw ) {
 
   GtkWidget *pwOutput = prw->pwOutput;
-  int nTrials = prw->padjTrials->value;
+  int nTrials = (int)prw->padjTrials->value;
   float ar[ 5 ];
   int i, j;
   char sz[ 16 ];
   int anPips[ 2 ];
   const float x = ( 2 * 3 + 3 * 4 + 4 * 5 + 4 * 6 + 6 * 7 +
               5* 8  + 4 * 9 + 2 * 10 + 2 * 11 + 1 * 12 + 
-              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0;
+              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0f;
   float arMu[ 2 ];
   gchar *pch;
 
@@ -502,7 +576,7 @@ PerformOSR ( GtkWidget *pw, racewidget *prw ) {
 
     j = prw->fMove ? i : !i;
 
-    pch = g_strdup_printf ( _("%7.3f"), arMu[ j ] * x );
+    pch = g_strdup_printf ( "%7.3f", arMu[ j ] * x );
     gtk_label_set_text( GTK_LABEL( prw->epcwOSR.apwEPC[ i ] ), pch );
     g_free ( pch );
 
@@ -633,7 +707,7 @@ GTKShowRace ( const int fActivePage, int anBoard[ 2 ][ 25 ] ) {
   /* create dialog */
 
   pwDialog = GTKCreateDialog ( _("GNU Backgammon - Race Theory"), DT_INFO,
-                            NULL, NULL );
+                            NULL, DIALOG_FLAG_MODAL, NULL, NULL );
 
   /* add notebook pages */
 
@@ -661,6 +735,12 @@ GTKShowRace ( const int fActivePage, int anBoard[ 2 ][ 25 ] ) {
                              ThorpPage ( anBoard, prw->fMove ),
                              gtk_label_new ( _("Thorp Count") ) );
 
+  /* Keith */
+
+  gtk_notebook_append_page ( GTK_NOTEBOOK ( pwNotebook ),
+                             KeithPage ( anBoard, prw->fMove ),
+                             gtk_label_new ( _("Keith Count") ) );
+
   /* One sided rollout */
 
   gtk_notebook_append_page ( GTK_NOTEBOOK ( pwNotebook ),
@@ -669,12 +749,6 @@ GTKShowRace ( const int fActivePage, int anBoard[ 2 ][ 25 ] ) {
 
   /* show dialog */
 
-  gtk_window_set_modal( GTK_WINDOW( pwDialog ), TRUE );
-  gtk_window_set_transient_for( GTK_WINDOW( pwDialog ),
-                                GTK_WINDOW( pwMain ) );
-  gtk_signal_connect( GTK_OBJECT( pwDialog ), "destroy",
-                      GTK_SIGNAL_FUNC( gtk_main_quit ), NULL );
-
   gtk_widget_show_all( pwDialog );
 
   PerformOSR ( NULL, prw );
@@ -685,5 +759,4 @@ GTKShowRace ( const int fActivePage, int anBoard[ 2 ][ 25 ] ) {
   GTKDisallowStdin();
   gtk_main();
   GTKAllowStdin();
-
 }
