@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtktempmap.c,v 1.16 2004/02/02 09:50:59 uid68519 Exp $
+ * $Id: gtktempmap.c,v 1.21 2005/02/25 11:42:35 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -398,9 +398,11 @@ ExposeDie( GtkWidget *pw, GdkEventExpose *pev,
 
     /* render die */
 
-    memcpy( &rd, &rdAppearance, sizeof rd );
+    CopyAppearance(&rd);
     rd.nSize = ptmw->nSizeDie = nSizeDie;
-
+#if USE_BOARD3D
+    Copy3dDiceColour(&rd);
+#endif
     
     for (  i = 0; i < 2; ++i ) {
       g_free( ptmw->achDice[ i ] );
@@ -430,8 +432,12 @@ static void
 TempMapPlyToggled( GtkWidget *pw, tempmapwidget *ptmw ) {
 
   int *pi = (int *) gtk_object_get_user_data( GTK_OBJECT( pw ) );
+#if defined(REDUCTION_CODE)
   evalcontext ec = { TRUE, 0, 0, TRUE, 0.0 };
-
+#else
+  evalcontext ec = { TRUE, 0, FALSE, TRUE, 0.0 };
+#endif
+  
   if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( pw ) ) ) {
 
     /* recalculate equities */
@@ -502,7 +508,12 @@ extern void
 GTKShowTempMap( const matchstate ams[], const int n,
                 const gchar *aszTitle[], const int fInvert ) {
 
+#if defined(REDUCTION_CODE)
   evalcontext ec = { TRUE, 0, 0, TRUE, 0.0 };
+#else
+  evalcontext ec = { TRUE, 0, FALSE, TRUE, 0.0 };
+#endif
+
   tempmapwidget *ptmw;
   int *pi;
   int i, j;
@@ -521,7 +532,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
 
   pwDialog = GTKCreateDialog( _("Sho Sengoku Temperature Map - "
                              "Distribution of rolls"), DT_INFO, NULL, NULL );
-
+  gtk_window_set_modal( GTK_WINDOW( pwDialog ), TRUE );
+  gtk_window_set_transient_for( GTK_WINDOW( pwDialog ),
+                                  GTK_WINDOW( pwMain ) );
 
   ptmw = (tempmapwidget *) g_malloc( sizeof ( tempmapwidget ) );
   ptmw->fShowBestMove = FALSE;
@@ -591,6 +604,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
           
           ptm->aapwDA[ i ][ j ] = gtk_drawing_area_new();
           ptm->aapwe[ i ][ j ] = gtk_event_box_new();
+#if GTK_CHECK_VERSION(2,4,0)
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ptm->aapwe[i][j]), FALSE);
+#endif
           
           gtk_container_add( GTK_CONTAINER( ptm->aapwe[ i ][ j ] ),
                              ptm->aapwDA[ i ][ j ] );
@@ -660,6 +676,9 @@ GTKShowTempMap( const matchstate ams[], const int n,
       
       ptm->pwAverage = gtk_drawing_area_new();
       ptm->pweAverage = gtk_event_box_new();
+#if GTK_CHECK_VERSION(2,4,0)
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ptm->pweAverage), FALSE);
+#endif
 
       gtk_container_add( GTK_CONTAINER( ptm->pweAverage ), ptm->pwAverage );
 
