@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtktheory.c,v 1.29 2003/12/29 20:19:32 uid65656 Exp $
+ * $Id: gtktheory.c,v 1.31 2006/02/24 17:41:47 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -98,21 +98,16 @@ typedef struct _theorywidget {
 } theorywidget;
 
 
-
-static void
-MWGetSelection( GtkWidget *pw, GtkSelectionData *psd,
-                guint n, guint t, theorywidget *ptw ) {
-
-
-  printf( "sorry not implemented\n" );
-
-}
-
 static void
 ResetTheory ( GtkWidget *pw, theorywidget *ptw ) {
 
   float aarRates[ 2 ][ 2 ];
+#if defined(REDUCTION_CODE)
   evalcontext ec = { FALSE, 0, 0, TRUE, 0.0 };
+#else
+  evalcontext ec = { FALSE, 0, FALSE, TRUE, 0.0 };
+#endif
+  
   float arOutput[ NUM_OUTPUTS ];
 
   int i,j;
@@ -610,7 +605,12 @@ PlyClicked( GtkWidget *pw, theorywidget *ptw ) {
   int f = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( pw ) );
   cubeinfo ci;
   float aarRates[ 2 ][ 2 ];
+#if defined (REDUCTION_CODE)
   evalcontext ec = { FALSE, 0, 0, TRUE, 0.0 };
+#else
+  evalcontext ec = { FALSE, 0, FALSE, TRUE, 0.0 };
+#endif
+
   float arOutput[ NUM_OUTPUTS ];
   int i, j;
 
@@ -971,12 +971,6 @@ GTKShowTheory ( const int fActivePage ) {
       gtk_clist_set_column_auto_resize ( GTK_CLIST( ptw->apwMW[ i ] ),
       j, TRUE );*/
 
-    gtk_selection_add_target( ptw->apwMW[ i ], GDK_SELECTION_PRIMARY,
-                              GDK_SELECTION_TYPE_STRING, 0 );
-
-    gtk_signal_connect( GTK_OBJECT( ptw->apwMW[ i ] ), "selection_get",
-                        GTK_SIGNAL_FUNC( MWGetSelection ), ptw );
-
     gtk_container_add ( GTK_CONTAINER( pwFrame ), ptw->apwMW[ i ] );
 
   }
