@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: gtktoolbar.c,v 1.12 2004/02/25 17:03:38 uid68519 Exp $
+ * $Id: gtktoolbar.c,v 1.24 2006/09/11 22:59:40 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -25,7 +25,6 @@
 
 #include <stdlib.h>
 
-#define GTK_ENABLE_BROKEN /* for GtkText */
 #include <gtk/gtk.h>
 
 #include <stdio.h>
@@ -37,7 +36,7 @@
 #include "gtkboard.h"
 #include "gtkgame.h"
 #include "gtk-multiview.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "drawboard.h"
 #include "renderprefs.h"
 extern void NewDialog( gpointer *p, guint n, GtkWidget *pw ); 
@@ -169,13 +168,36 @@ ToolbarToggleClockwise( GtkWidget *pw, toolbarwidget *ptw ) {
 }
 
 
+int editing = FALSE;
+
 static void
 ToolbarToggleEdit( GtkWidget *pw, toolbarwidget *ptw ) {
 
   BoardData *pbd = BOARD( pwBoard )->board_data;
 
-  board_edit( pbd );
+  if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptw->pwEdit ) ))
+  { /* Undo any partial move that may have been made when entering edit mode */
+    Undo();
+    editing = TRUE;
+  }
+  else
+    editing = FALSE;
 
+  board_edit( pbd );
+}
+
+extern int
+ToolbarIsEditing( GtkWidget *pwToolbar ) {
+
+  return editing;
+}
+
+extern void
+ToolbarActivateEdit( GtkWidget *pwToolbar ) {
+  /* This is only used fot the New->Position option */
+	
+  toolbarwidget *ptw = gtk_object_get_user_data ( GTK_OBJECT ( pwToolbar ) );
+  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( ptw->pwEdit ), TRUE );
 }
 
 
@@ -184,72 +206,17 @@ ToolbarStop( GtkWidget *pw, gpointer unused ) {
 
 	fInterrupt = TRUE;
 #if USE_BOARD3D
-	if (rdAppearance.fDisplayType == DT_3D)
+{
+	BoardData *bd = BOARD( pwBoard )->board_data;
+	if (bd->rd->fDisplayType == DT_3D)
 	{
-		BoardData *bd = BOARD( pwBoard )->board_data;
 		StopIdle3d(bd);
 		RestrictiveRedraw();
 	}
+}
 #endif
 }
 
-static void 
-OpenClicked( GtkWidget *pw, gpointer unused ) {
-
-  char *sz = getDefaultPath ( PATH_SGF );
-
-  GTKFileCommand(_("Open match, session, game or position"), 
-		   sz, "load match", "sgf", FDT_NONE);
-  if ( sz ) 
-    free ( sz );
-}
-
-static void 
-SaveClicked( GtkWidget *pw, gpointer unused ) {
-
-  char *sz = getDefaultFileName ( PATH_SGF );
-  GTKFileCommand(_("Save match, session, game or position"), 
-		   sz, "save", "sgf", FDT_SAVE);
-  if ( sz ) 
-    free ( sz );
-}
-
-static void 
-ImportClicked( GtkWidget *pw, gpointer unused ) {
-
-	/* Order of import types in menu */
-	int impTypes[] = {PATH_BKG, PATH_MAT, PATH_POS, PATH_OLDMOVES, PATH_SGG,
-		PATH_TMG, PATH_MAT, PATH_SNOWIE_TXT};
-	char* sz = NULL;
-
-	if (lastImportType != -1)
-		sz = getDefaultPath(impTypes[lastImportType]);
-
-	GTKFileCommand(_("Import match, session, game or position"), 
-		   sz, "import", "N", FDT_IMPORT);
-
-	if (sz)
-		free(sz);
-}
-
-static void 
-ExportClicked( GtkWidget *pw, gpointer unused ) {
-
-	/* Order of export types in menu */
-	int expTypes[] = {PATH_HTML, PATH_GAM, PATH_POS, PATH_MAT, PATH_GAM, 
-		PATH_LATEX, PATH_PDF, PATH_POSTSCRIPT, PATH_EPS, -1, PATH_TEXT, -1, PATH_SNOWIE_TXT};
-	char* sz = NULL;
-
-	if (lastExportType != -1 && expTypes[lastExportType] != -1)
-		sz = getDefaultPath(expTypes[lastExportType]);
-
-	GTKFileCommand(_("Export match, session, game or position"), 
-		   sz, "export", "N", FDT_EXPORT_FULL);
-
-	if (sz)
-		free(sz);
-}
-
 
 extern GtkWidget *
 ToolbarGetStopParent ( GtkWidget *pwToolbar ) {
@@ -261,22 +228,6 @@ ToolbarGetStopParent ( GtkWidget *pwToolbar ) {
   return ptw->pwStopParent;
 }
 
-extern int
-ToolbarIsEditing( GtkWidget *pwToolbar ) {
-
-  toolbarwidget *ptw = gtk_object_get_user_data ( GTK_OBJECT ( pwToolbar ) );
-
-  return gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptw->pwEdit ) );
-}
-
-extern void
-ToolbarActivateEdit( GtkWidget *pwToolbar ) {
-  /* This is only used fot the New->Position option */
-	
-  toolbarwidget *ptw = gtk_object_get_user_data ( GTK_OBJECT ( pwToolbar ) );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( ptw->pwEdit ), TRUE );
-}
-
 extern toolbarcontrol
 ToolbarUpdate ( GtkWidget *pwToolbar,
                 const matchstate *pms,
@@ -383,22 +334,11 @@ ToolbarNew ( void ) {
   
   gtk_object_set_data_full ( GTK_OBJECT ( vbox_toolbar ), "user_data",
                              ptw, g_free );
-#if USE_GTK2
   pwToolbar = gtk_toolbar_new ();
   gtk_toolbar_set_orientation ( GTK_TOOLBAR ( pwToolbar ),
                                 GTK_ORIENTATION_HORIZONTAL );
   gtk_toolbar_set_style ( GTK_TOOLBAR ( pwToolbar ),
                           GTK_TOOLBAR_ICONS );
-#else
-  pwToolbar = gtk_toolbar_new ( GTK_ORIENTATION_HORIZONTAL,
-                                GTK_TOOLBAR_BOTH );
-  gtk_toolbar_set_space_style ( GTK_TOOLBAR ( pwToolbar ),
-                          GTK_TOOLBAR_SPACE_LINE );
-
-  gtk_toolbar_set_button_relief( GTK_TOOLBAR( pwToolbar ), 
-		  GTK_RELIEF_NONE);
-#endif /* ! USE_GTK2 */
-
   gtk_box_pack_start( GTK_BOX( vbox_toolbar ), pwToolbar, 
                       FALSE, FALSE, 0 );
 
@@ -418,7 +358,7 @@ ToolbarNew ( void ) {
                               _("Open game, match, session or position"),
                                NULL,
 			       image_from_xpm_d ( stock_open_xpm, pwToolbar),
-                               GTK_SIGNAL_FUNC( OpenClicked ), NULL );
+                               GTK_SIGNAL_FUNC( GTKOpen ), NULL );
 
     /* Import button */
   gtk_toolbar_append_item ( GTK_TOOLBAR ( pwToolbar ),
@@ -426,7 +366,7 @@ ToolbarNew ( void ) {
                                _("Import game, match or position"), 
                                NULL,
 			       image_from_xpm_d ( stock_import_xpm, pwToolbar),
-                               GTK_SIGNAL_FUNC( ImportClicked ), NULL );
+                               GTK_SIGNAL_FUNC( GTKImport ), NULL );
 
 #define TB_BUTTON_ADD(pointer,icon,label,cb,arg,tooltip,tooltip2) \
   pointer = gtk_button_new(); \
@@ -459,25 +399,21 @@ ToolbarNew ( void ) {
 		  GTK_RELIEF_NONE);
   
   /* Save button */
-  TB_BUTTON_ADD(ptw->pwSave, stock_save_xpm, _("Save"), SaveClicked,
+  TB_BUTTON_ADD(ptw->pwSave, stock_save_xpm, _("Save"), GTKSave,
 		  NULL, 
                   _("Save match, session, game or position"), 
 		  NULL) ;
   
   /* Export button */
 
-  TB_BUTTON_ADD(ptw->pwExport, stock_export_xpm, _("Export"), ExportClicked,
+  TB_BUTTON_ADD(ptw->pwExport, stock_export_xpm, _("Export"), GTKExport,
 		  NULL, 
                           _("Export to another format"), 
                                NULL);
 
   
   gtk_toolbar_append_space(GTK_TOOLBAR(pwToolbar));
-#if !USE_GTK2
-  gtk_toolbar_set_button_relief( GTK_TOOLBAR( pwToolbar ), 
-		  GTK_RELIEF_NONE);
-#endif
-  
+
   /* Take/accept button */
   TB_BUTTON_ADD(ptw->pwTake, stock_ok_xpm, _("Accept"), ButtonClickedYesNo,
 		  "yes", 
@@ -546,6 +482,7 @@ ToolbarNew ( void ) {
   /* stop button */
   
   ptw->pwStopParent = gtk_event_box_new();
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ptw->pwStopParent), FALSE);
   ptw->pwStop = gtk_button_new(),
   gtk_container_add( GTK_CONTAINER( ptw->pwStopParent ), ptw->pwStop );
 
