@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: html.c,v 1.156 2004/02/16 09:52:30 uid68519 Exp $
+ * $Id: html.c,v 1.167 2006/04/07 12:20:23 c_anthon Exp $
  */
 
 #include "config.h"
@@ -172,8 +172,8 @@ WriteStyleSheet ( FILE *pf, const htmlexportcss hecss ) {
     /* write come comments in the file */
 
     fputs( "\n"
-           "/* CSS Stylesheet for GNU Backgammon " VERSION " */\n"
-           "/* $Id: html.c,v 1.156 2004/02/16 09:52:30 uid68519 Exp $ */\n",
+           "/* CSS Stylesheet for " VERSION_STRING " */\n"
+           "/* $Id: html.c,v 1.167 2006/04/07 12:20:23 c_anthon Exp $ */\n",
            pf );
 
     fputs( _("/* This file is distributed as a part of the "
@@ -590,6 +590,11 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
       acOff[ i ] -= anBoard[ i ][ j ];
   }
 
+  PipCount ( anBoard, anPips );
+
+  /* Begin table  and print for player 0*/
+  fprintf ( pf, "<table><tr><th align=\"left\">%s<th align=\"right\">%d<tr><th align=\"center\" colspan=\"2\">", ap[ 0 ].szName,  anPips[ 1 ]);
+
   /* avoid page break when printing */
   fputs( "<p style=\"page-break-inside: avoid\">", pf );
     
@@ -732,27 +737,22 @@ printHTMLBoardBBS ( FILE *pf, matchstate *pms, int fTurn,
 
   fputs( "</p>\n", pf );
 
+  fprintf ( pf, "<tr><th align=\"left\">%s<th align=\"right\">%d<tr><th align=\"center\" colspan=\"2\">", ap[ 1 ].szName, anPips[ 0 ] );
+
   /* pip counts */
 
   fputs ( "<p>", pf );
 
-  PipCount ( anBoard, anPips );
-  fprintf ( pf, _("Pip counts: %s %d, %s %d<br />\n"),
-            ap[ 0 ].szName, anPips[ 1 ], 
-            ap[ 1 ].szName, anPips[ 0 ] );
 
-  /* position ID */
+  /* position ID Player 1 and end of table*/
 
   fprintf( pf, "<span %s>", 
            GetStyle ( CLASS_POSITIONID, hecss ) );
 
-  fprintf ( pf, _("Position ID: <tt>%s</tt> Match ID: <tt>%s</tt><br />\n"),
+  fprintf ( pf, _("Position ID: <tt>%s</tt> Match ID: <tt>%s</tt><br /></span></table>\n"),
             PositionID ( pms->anBoard ),
             MatchIDFromMatchState ( pms ) );
 
-  fprintf( pf, "</span>" );
-
-  fputs ( "</p>\n", pf );
 
 }
 
@@ -1684,7 +1684,9 @@ HTMLBoardHeader ( FILE *pf, const matchstate *pms,
     /* resignation */
 
     fprintf ( pf,
-              _(" %s resigns %d points"), 
+              ngettext( " %s resigns %d point",
+                        " %s resigns %d points",
+                        pms->fResigned * pms->nCube ), 
               ap[ pms->fTurn ].szName,
               pms->fResigned * pms->nCube
             );
@@ -1733,13 +1735,13 @@ HTMLBoardHeader ( FILE *pf, const matchstate *pms,
  *
  */
 
-static void 
+static void
 HTMLPrologue ( FILE *pf, const matchstate *pms,
-               const int iGame,
-               char *aszLinks[ 4 ],
-               const htmlexporttype het,
-               const htmlexportcss hecss ) {
-
+	       const int iGame,
+	       char *aszLinks[ 4 ],
+	       const htmlexporttype het,
+	       const htmlexportcss hecss )
+{
   char szTitle[ 100 ];
 
   int i;
@@ -1747,21 +1749,22 @@ HTMLPrologue ( FILE *pf, const matchstate *pms,
 
   /* DTD */
 
-  sprintf( szTitle, pms->cGames == 1 ? 
-           _("The score (after %d game) is: %s %d, %s %d") :
-           _("The score (after %d games) is: %s %d, %s %d"),
-           pms->cGames, 
-           ap[ 0 ].szName, pms->anScore[ 0 ],
-           ap[ 1 ].szName, pms->anScore[ 1 ] );
+  sprintf( szTitle,
+	   ngettext( "The score (after %d game) is: %s %d, %s %d",
+		     "The score (after %d games) is: %s %d, %s %d",
+		     pms->cGames),
+	     pms->cGames,
+	   ap[ 0 ].szName, pms->anScore[ 0 ],
+	   ap[ 1 ].szName, pms->anScore[ 1 ] );
 
-  if ( pms->nMatchTo > 0 ) 
+  if ( pms->nMatchTo > 0 )
     sprintf( strchr( szTitle, 0 ),
-             pms->nMatchTo == 1 ?
-             _(" (match to %d point%s)") :
-             _(" (match to %d points%s)"),
-             pms->nMatchTo,
-             pms->fCrawford ? 
-             _(", Crawford game") : ( pms->fPostCrawford ?
+	     ngettext( " (match to %d point%s)",
+		       " (match to %d points%s)",
+		       pms->nMatchTo ),
+	     pms->nMatchTo,
+	     pms->fCrawford ?
+	     _(", Crawford game") : ( pms->fPostCrawford ?
 					 _(", post-Crawford play") : ""));
 
   fprintf ( pf,
@@ -1770,20 +1773,20 @@ HTMLPrologue ( FILE *pf, const matchstate *pms,
             "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" "
             "lang=\"en\">\n"
             "<head>\n"
-            "<meta name=\"generator\" content=\"GNU Backgammon %s\" />\n"
+            "<meta name=\"generator\" content=\"%s\" />\n"
             "<meta http-equiv=\"Content-Type\" "
             "content=\"text/html; charset=%s\" />\n" 
             "<meta name=\"keywords\" content=\"%s, %s, %s\" />\n"
             "<meta name=\"description\" "
             "content=\"",
-            VERSION,
+            VERSION_STRING,
             GNUBG_CHARSET,
             ap[ 0 ].szName, ap[ 1 ].szName,
             ( pms->nMatchTo ) ? _("match play") : _("money game") );
 
   fprintf ( pf, 
-            _("%s (analysed by GNU Backgammon %s)"),
-            szTitle, VERSION );
+            _("%s (analysed by %s)"),
+            szTitle, VERSION_STRING );
 
   fprintf ( pf,
             "\" />\n"
@@ -1850,7 +1853,7 @@ HTMLEpilogue ( FILE *pf, const matchstate *pms, char *aszLinks[ 4 ],
   int fFirst;
   int i;
 
-  const char szVersion[] = "$Revision: 1.156 $";
+  const char szVersion[] = "$Revision: 1.167 $";
   int iMajor, iMinor;
 
   iMajor = atoi ( strchr ( szVersion, ' ' ) );
@@ -1884,9 +1887,8 @@ HTMLEpilogue ( FILE *pf, const matchstate *pms, char *aszLinks[ 4 ],
 
   fprintf ( pf, 
             _("Output generated %s by "
-              "<a href=\"http://www.gnu.org/software/gnubg/\">GNU Backgammon " 
-              "%s</a>") ,
-            ctime ( &t ), VERSION );
+              "<a href=\"http://www.gnu.org/software/gnubg/\">%s</a>") ,
+            ctime ( &t ), VERSION_STRING );
 
   fputs ( " ", pf );
             
@@ -1931,7 +1933,7 @@ HTMLEpilogueComment ( FILE *pf ) {
 
   time_t t;
 
-  const char szVersion[] = "$Revision: 1.156 $";
+  const char szVersion[] = "$Revision: 1.167 $";
   int iMajor, iMinor;
   char *pc;
 
@@ -1947,9 +1949,9 @@ HTMLEpilogueComment ( FILE *pf ) {
   fputs ( "\n<!-- Epilogue -->\n\n", pf );
 
   fprintf ( pf, 
-            _("<!-- Output generated %s by GNU Backgammon %s "
+            _("<!-- Output generated %s by %s "
               "(http://www.gnu.org/software/gnubg/) "),
-            pc, VERSION );
+            pc, VERSION_STRING );
 
   fputs ( " ", pf );
             
@@ -2005,7 +2007,6 @@ HTMLPrintCubeAnalysisTable ( FILE *pf,
   /* check if cube analysis should be printed */
 
   if ( pes->et == EVAL_NONE ) return; /* no evaluation */
-  if ( ! GetDPEq ( NULL, NULL, pci ) ) return; /* cube not available */
 
   cd = FindCubeDecision ( arDouble, GCCCONSTAHACK aarOutput, pci );
 
@@ -2379,22 +2380,22 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
     /* cube analysis from move */
 
     HTMLPrintCubeAnalysisTable ( pf, 
-                                 pmr->n.aarOutput, pmr->n.aarStdDev,
-                                 pmr->n.fPlayer,
-                                 &pmr->n.esDouble, &ci, FALSE, -1,
-                                 pmr->n.stCube, SKILL_NONE, hecss );
+                                 pmr->CubeDecPtr->aarOutput, pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, &ci, FALSE, -1,
+                                 pmr->stCube, SKILL_NONE, hecss );
 
     break;
 
   case MOVE_DOUBLE:
 
     HTMLPrintCubeAnalysisTable ( pf, 
-                                 pmr->d.CubeDecPtr->aarOutput, 
-				 pmr->d.CubeDecPtr->aarStdDev,
-                                 pmr->d.fPlayer,
-                                 &pmr->d.CubeDecPtr->esDouble, 
+                                 pmr->CubeDecPtr->aarOutput, 
+				 pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, 
 				 &ci, TRUE, -1,
-                                 pmr->d.st, SKILL_NONE, hecss );
+                                 pmr->stCube, SKILL_NONE, hecss );
 
     break;
 
@@ -2404,13 +2405,13 @@ HTMLPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
     /* cube analysis from double, {take, drop, beaver} */
 
     HTMLPrintCubeAnalysisTable ( pf, 
-                                 pmr->d.CubeDecPtr->aarOutput, 
-				 pmr->d.CubeDecPtr->aarStdDev,
-                                 pmr->d.fPlayer,
-                                 &pmr->d.CubeDecPtr->esDouble, &ci, TRUE, 
+                                 pmr->CubeDecPtr->aarOutput, 
+				 pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, &ci, TRUE, 
                                  pmr->mt == MOVE_TAKE,
                                  SKILL_NONE, /* FIXME: skill from prev. cube */
-                                 pmr->d.st, hecss );
+                                 pmr->stCube, hecss );
 
     break;
 
@@ -2472,18 +2473,18 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
       fprintf ( pf, " (%+7.3f)</span></p>\n",
-                pmr->n.ml.amMoves[ pmr->n.iMove ].rScore -
-                pmr->n.ml.amMoves[ 0 ].rScore  );
+                pmr->ml.amMoves[ pmr->n.iMove ].rScore -
+                pmr->ml.amMoves[ 0 ].rScore  );
     else
       fprintf ( pf, " (%+6.3f%%)</span></p>\n",
                 100.0f *
-                eq2mwc ( pmr->n.ml.amMoves[ pmr->n.iMove ].rScore, &ci ) - 
+                eq2mwc ( pmr->ml.amMoves[ pmr->n.iMove ].rScore, &ci ) - 
                 100.0f *
-                eq2mwc ( pmr->n.ml.amMoves[ 0 ].rScore, &ci ) );
+                eq2mwc ( pmr->ml.amMoves[ 0 ].rScore, &ci ) );
 
   }
 
-  if ( pmr->n.lt != LUCK_NONE ) {
+  if ( pmr->lt != LUCK_NONE ) {
 
     /* joker */
 
@@ -2491,13 +2492,13 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
               GetStyle ( CLASS_JOKER, hecss ) );
     fprintf ( pf, 
               _("Alert: %s roll!"),
-              gettext ( aszLuckType[ pmr->n.lt ] ) );
+              gettext ( aszLuckType[ pmr->lt ] ) );
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
-      fprintf ( pf, " (%+7.3f)</span></p>\n", pmr->n.rLuck );
+      fprintf ( pf, " (%+7.3f)</span></p>\n", pmr->rLuck );
     else
       fprintf ( pf, " (%+6.3f%%)</span></p>\n",
-                100.0f * eq2mwc ( pmr->n.rLuck, &ci ) - 
+                100.0f * eq2mwc ( pmr->rLuck, &ci ) - 
                 100.0f * eq2mwc ( 0.0f, &ci ) );
 
   }
@@ -2525,9 +2526,9 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
             _("Equity") : _("MWC") );
 
 
-  if ( pmr->n.ml.cMoves ) {
+  if ( pmr->ml.cMoves ) {
 
-    for ( i = 0; i < pmr->n.ml.cMoves; i++ ) {
+    for ( i = 0; i < pmr->ml.cMoves; i++ ) {
 
       if ( i >= exsExport.nMoves && i != pmr->n.iMove ) 
         continue;
@@ -2550,8 +2551,8 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
       /* move no */
 
-      if ( i != pmr->n.iMove || i != pmr->n.ml.cMoves - 1 || 
-           pmr->n.ml.cMoves == 1 ) 
+      if ( i != pmr->n.iMove || i != pmr->ml.cMoves - 1 || 
+           pmr->ml.cMoves == 1 ) 
         fprintf ( pf, 
                   "<td %s>%d</td>\n", 
                   GetStyle ( CLASS_MOVENUMBER, hecss ), i + 1 );
@@ -2561,7 +2562,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
       /* ply */
 
-      switch ( pmr->n.ml.amMoves[ i ].esMove.et ) {
+      switch ( pmr->ml.amMoves[ i ].esMove.et ) {
       case EVAL_NONE:
         fprintf ( pf,
                   "<td %s>n/a</td>\n",
@@ -2571,7 +2572,7 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
         fprintf ( pf,
                   "<td %s>%d</td>\n",
                   GetStyle ( CLASS_MOVEPLY, hecss ),
-                  pmr->n.ml.amMoves[ i ].esMove.ec.nPlies );
+                  pmr->ml.amMoves[ i ].esMove.ec.nPlies );
         break;
       case EVAL_ROLLOUT:
         fprintf ( pf,
@@ -2586,12 +2587,12 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
                 "<td %s>%s</td>\n",
                 GetStyle ( CLASS_MOVEMOVE, hecss ),
                 FormatMove ( sz, pms->anBoard,
-                             pmr->n.ml.amMoves[ i ].anMove ) );
+                             pmr->ml.amMoves[ i ].anMove ) );
 
       /* equity */
 
-      rEq = pmr->n.ml.amMoves[ i ].rScore;
-      rEqTop = pmr->n.ml.amMoves[ 0 ].rScore;
+      rEq = pmr->ml.amMoves[ i ].rScore;
+      rEqTop = pmr->ml.amMoves[ 0 ].rScore;
 
       if ( i ) 
         fprintf ( pf,
@@ -2615,9 +2616,9 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
 
       if ( exsExport.fMovesDetailProb ) {
 
-        float *ar = pmr->n.ml.amMoves[ i ].arEvalMove;
+        float *ar = pmr->ml.amMoves[ i ].arEvalMove;
 
-        /*percentages */
+	/* percentages */
 
         if ( i == pmr->n.iMove )
           fprintf ( pf, "<tr %s>\n", 
@@ -2634,19 +2635,19 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
         fputs ( "<td>", pf );
 
 
-        switch ( pmr->n.ml.amMoves[ i ].esMove.et ) {
+        switch ( pmr->ml.amMoves[ i ].esMove.et ) {
         case EVAL_EVAL:
           fputs ( OutputPercents ( ar, TRUE ), pf );
           break;
         case EVAL_ROLLOUT:
           printRolloutTable ( pf, NULL,
                               ( float (*)[ NUM_ROLLOUT_OUTPUTS ] ) 
-                              pmr->n.ml.amMoves[ i ].arEvalMove,
+                              pmr->ml.amMoves[ i ].arEvalMove,
                               ( float (*)[ NUM_ROLLOUT_OUTPUTS ] ) 
-                              pmr->n.ml.amMoves[ i ].arEvalStdDev,
+                              pmr->ml.amMoves[ i ].arEvalStdDev,
                               &ci,
                               1,
-                              pmr->n.ml.amMoves[ i ].esMove.rc.fCubeful,
+                              pmr->ml.amMoves[ i ].esMove.rc.fCubeful,
                               FALSE, hecss );
           break;
         default:
@@ -2665,9 +2666,9 @@ HTMLPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
        */
       
       if ( exsExport.afMovesParameters 
-           [ pmr->n.ml.amMoves[ i ].esMove.et - 1 ] ) {
+           [ pmr->ml.amMoves[ i ].esMove.et - 1 ] ) {
         
-        evalsetup *pes = &pmr->n.ml.amMoves[ i ].esMove;
+        evalsetup *pes = &pmr->ml.amMoves[ i ].esMove;
         
         switch ( pes->et ) {
         case EVAL_EVAL: 
@@ -2808,12 +2809,12 @@ HTMLAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
     if ( pmr->n.anMove[ 0 ] >= 0 )
       fprintf ( pf,
                 _("%s%s moves %s"), bullet,
-                ap[ pmr->n.fPlayer ].szName,
+                ap[ pmr->fPlayer ].szName,
                 FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
-    else if ( ! pmr->n.ml.cMoves )
+    else if ( ! pmr->ml.cMoves )
       fprintf ( pf,
                 _("%s%s cannot move"), bullet,
-                ap[ pmr->n.fPlayer ].szName );
+                ap[ pmr->fPlayer ].szName );
 
     fputs ( "</p>\n", pf );
 
@@ -2841,11 +2842,11 @@ HTMLAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr,
     if ( pmr->mt == MOVE_DOUBLE )
       fprintf ( pf,
                 "%s%s doubles</p>\n", bullet,
-                ap[ pmr->d.fPlayer ].szName );
+                ap[ pmr->fPlayer ].szName );
     else
       fprintf ( pf,
                 "%s%s %s</p>\n", bullet,
-                ap[ pmr->d.fPlayer ].szName,
+                ap[ pmr->fPlayer ].szName,
                 ( pmr->mt == MOVE_TAKE ) ? _("accepts") : _("rejects") );
 
     if ( exsExport.fIncludeAnalysis )
@@ -3020,30 +3021,30 @@ HTMLPrintComment ( FILE *pf, const moverecord *pmr,
   switch ( pmr->mt ) {
 
   case MOVE_GAMEINFO:
-    sz = pmr->g.sz;
+    sz = pmr->sz;
     break;
   case MOVE_DOUBLE:
   case MOVE_TAKE:
   case MOVE_DROP:
-    sz = pmr->d.sz;
+    sz = pmr->sz;
     break;
   case MOVE_NORMAL:
-    sz = pmr->n.sz;
+    sz = pmr->sz;
     break;
   case MOVE_RESIGN:
-    sz = pmr->r.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETBOARD:
-    sz = pmr->sb.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETDICE:
-    sz = pmr->sd.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETCUBEVAL:
-    sz = pmr->scv.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETCUBEPOS:
-    sz = pmr->scp.sz;
+    sz = pmr->sz;
     break;
   case MOVE_TIME:
 	  /* ignore */
@@ -3090,11 +3091,27 @@ HTMLPrintComment ( FILE *pf, const moverecord *pmr,
 static void
 HTMLPrintMI( FILE *pf, const char *szTitle, const char *sz ) {
 
+  gchar **ppch;
+  gchar *pchToken;
+  int i;
+
   if ( ! sz || ! *sz )
     return;
 
-  fprintf( pf, "%s: %s<br />\n", szTitle, sz );
+  fprintf( pf, "<tr valign=\"top\">"
+           "<td style=\"padding-right: 2em\">%s</td><td>", szTitle );
 
+  ppch = g_strsplit( sz, "\n", -1 );
+  for ( i = 0; (pchToken = ppch[ i ]) ; ++i ) {
+    if ( i )
+      fputs( "<br />\n", pf );
+    fputs( pchToken, pf );
+  }
+
+  g_strfreev( ppch );
+
+  fputs( "</td></tr>\n", pf );
+  
 }
 
 static void
@@ -3118,15 +3135,17 @@ HTMLMatchInfo ( FILE *pf, const matchinfo *pmi,
 
   fprintf ( pf, "<h2>%s</h2>\n", _("Match Information") );
 
-  fputs ( "<p>\n", pf );
+  fputs ( "<table border=\"0\">\n", pf );
 
   /* ratings */
 
   for ( i = 0; i < 2; ++i )
-    if ( pmi->pchRating[ i ] )
-      fprintf ( pf, _("%s's rating: %s<br />\n"), 
-                ap[ i ].szName, 
-                pmi->pchRating[ i ] ? pmi->pchRating[ i ] : _("n/a") );
+    if ( pmi->pchRating[ i ] ) {
+      gchar *pch = g_strdup_printf( _("%s's rating"), ap[ i ].szName );
+      HTMLPrintMI( pf, pch, 
+                   pmi->pchRating[ i ] ? pmi->pchRating[ i ] : _("n/a") );
+      g_free( pch );
+    }
 
   /* date */
 
@@ -3136,8 +3155,7 @@ HTMLMatchInfo ( FILE *pf, const matchinfo *pmi,
     tmx.tm_mon = pmi->nMonth - 1;
     tmx.tm_mday = pmi->nDay;
     strftime ( sz, sizeof ( sz ), _("%B %d, %Y"), &tmx );
-    fprintf ( pf, _("Date: %s<br />\n"), sz ); 
-
+    HTMLPrintMI( pf, _("Date"), sz );
   }
 
   /* event, round, place and annotator */
@@ -3148,7 +3166,7 @@ HTMLMatchInfo ( FILE *pf, const matchinfo *pmi,
   HTMLPrintMI( pf, _("Annotator"), pmi->pchAnnotator );
   HTMLPrintMI( pf, _("Comment"), pmi->pchComment );
 
-  fputs ( "</p>\n", pf );
+  fputs ( "</table>\n", pf );
 
   fputs ( "\n<!-- End Match Information -->\n\n", pf );
 
@@ -3250,12 +3268,12 @@ HTMLDumpPlayerRecords ( FILE *pf, const htmlexportcss hecss ) {
  */
 
 static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
-                             const char *szExtension, 
-                             const htmlexporttype het,
-                             const htmlexportcss hecss,
-                             const int iGame, const int fLastGame,
-                             char *aszLinks[ 4 ] ) {
-
+			     const char *szExtension,
+			     const htmlexporttype het,
+			     const htmlexportcss hecss,
+			     const int iGame, const int fLastGame,
+			     char *aszLinks[ 4 ] )
+{
     list *pl;
     moverecord *pmr;
     matchstate msExport;
@@ -3263,7 +3281,7 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
     int iMove = 0;
     statcontext *psc = NULL;
     static statcontext scTotal;
-    movegameinfo *pmgi = NULL;
+    xmovegameinfo *pmgi = NULL;
 
     if ( ! iGame )
       IniStatcontext ( &scTotal );
@@ -3299,14 +3317,14 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
 
       case MOVE_NORMAL:
 
-	if( pmr->n.fPlayer != msExport.fMove ) {
+	if( pmr->fPlayer != msExport.fMove ) {
 	    SwapSides( msExport.anBoard );
-	    msExport.fMove = pmr->n.fPlayer;
+	    msExport.fMove = pmr->fPlayer;
 	}
       
-        msExport.fTurn = msExport.fMove = pmr->n.fPlayer;
-        msExport.anDice[ 0 ] = pmr->n.anRoll[ 0 ];
-        msExport.anDice[ 1 ] = pmr->n.anRoll[ 1 ];
+        msExport.fTurn = msExport.fMove = pmr->fPlayer;
+        msExport.anDice[ 0 ] = pmr->anDice[ 0 ];
+        msExport.anDice[ 1 ] = pmr->anDice[ 1 ];
 
         HTMLBoardHeader ( pf, &msExport, het, hecss, iGame, iMove, TRUE );
 
@@ -3352,13 +3370,13 @@ static void ExportGameHTML ( FILE *pf, list *plGame, const char *szImageDir,
 
       /* print game result */
 
-      fprintf ( pf, 
-                ( pmgi->nPoints == 1 ) ?
-                _("<p %s>%s wins %d point</p>\n") :
-                _("<p %s>%s wins %d points</p>\n"),
-                GetStyle ( CLASS_RESULT, hecss ),
-                ap[ pmgi->fWinner ].szName, 
-                pmgi->nPoints );
+      fprintf ( pf,
+		ngettext( "<p %s>%s wins %d point</p>\n",
+			  "<p %s>%s wins %d points</p>\n",
+			  pmgi->nPoints ),
+		GetStyle ( CLASS_RESULT, hecss ),
+		ap[ pmgi->fWinner ].szName, 
+		pmgi->nPoints );
     }
 
     if ( psc ) {
@@ -3489,7 +3507,7 @@ extern void CommandExportGameHtml( char *sz ) {
     }
     
     if( !sz || !*sz ) {
-	outputl( _("You must specify a file to export to (see `help export"
+	outputl( _("You must specify a file to export to (see `help export "
 		 "game html').") );
 	return;
     }
@@ -3539,6 +3557,21 @@ extern void CommandExportGameHtml( char *sz ) {
  *   Caller must free returned pointer if not NULL
  * 
  */
+#ifdef WIN32
+#define DIR_SEPARATOR  '\\'
+#define DIR_SEPARATOR_S  "\\"
+#else
+#define DIR_SEPARATOR  '/'
+#define DIR_SEPARATOR_S  "/"
+#endif
+
+static char *
+get_basename (const char *filename) 
+{ 
+  char *p1 = strrchr (filename, DIR_SEPARATOR); 
+  return p1 ? p1 + 1 : (char *) filename;
+} 
+
 
 extern char *
 HTMLFilename ( const char *szBase, const int iGame ) {
@@ -3601,23 +3634,23 @@ extern void CommandExportMatchHtml( char *sz ) {
 
       szCurrent = HTMLFilename ( sz, i );
 	  filenames[0] = HTMLFilename ( sz, 0 );
-      aszLinks[ 0 ] = basename ( filenames[ 0 ] );
+      aszLinks[ 0 ] = get_basename ( filenames[ 0 ] );
 	  filenames[ 1 ] = aszLinks[ 1 ] = NULL;
 	  if (i > 0) {
 		filenames[ 1 ] = HTMLFilename ( sz, i - 1 );
-		aszLinks[ 1 ]  = basename ( filenames[ 1 ] );
+		aszLinks[ 1 ]  = get_basename ( filenames[ 1 ] );
 	  }
 		
 	  filenames[ 2 ] = aszLinks[ 2 ] = NULL;
 	  if (i < nGames - 1) {
 		filenames[ 2 ] = HTMLFilename ( sz, i + 1 );
-		aszLinks[ 2 ]  = basename ( filenames[ 2 ] );
+		aszLinks[ 2 ]  = get_basename ( filenames[ 2 ] );
 	  }
 
 
 	  
 	  filenames[ 3 ] = HTMLFilename ( sz, nGames - 1 );
-	  aszLinks[ 3 ] = basename ( filenames[ 3 ] );
+	  aszLinks[ 3 ] = get_basename ( filenames[ 3 ] );
       if ( !i ) {
 
         if ( ! confirmOverwrite ( sz, fConfirmSave ) ) {
@@ -3747,8 +3780,8 @@ extern void CommandExportPositionHtml( char *sz ) {
 
 
 static void
-ExportPositionGammOnLine( FILE *pf ) {
-
+ExportPositionGammOnLine( FILE *pf )
+{
     int fHistory;
     moverecord *pmr = getCurrentMoveRecord ( &fHistory );
     int iMove;
@@ -3757,23 +3790,23 @@ ExportPositionGammOnLine( FILE *pf ) {
 
     fputs( "<strong>", pf );
 
-    fprintf( pf, 
-             ms.cGames == 1 ? 
-             _("The score (after %d game) is: %s %d, %s %d") :
-             _("The score (after %d games) is: %s %d, %s %d"),
-             ms.cGames, 
-             ap[ 0 ].szName, ms.anScore[ 0 ],
-             ap[ 1 ].szName, ms.anScore[ 1 ] );
+    fprintf( pf,
+	     ngettext( "The score (after %d game) is: %s %d, %s %d",
+		       "The score (after %d games) is: %s %d, %s %d",
+		       ms.cGames ),
+	     ms.cGames,
+	     ap[ 0 ].szName, ms.anScore[ 0 ],
+	     ap[ 1 ].szName, ms.anScore[ 1 ] );
 
-    if ( ms.nMatchTo > 0 ) 
+    if ( ms.nMatchTo > 0 )
       fprintf( pf,
-               ms.nMatchTo == 1 ?
-               _(" (match to %d point%s)") :
-               _(" (match to %d points%s)"),
-               ms.nMatchTo,
-               ms.fCrawford ? 
-               _(", Crawford game") : ( ms.fPostCrawford ?
-                                        _(", post-Crawford play") : ""));
+	       ngettext( " (match to %d point%s)",
+			 " (match to %d points%s)",
+			 ms.nMatchTo ),
+	       ms.nMatchTo,
+	       ms.fCrawford ?
+	       _(", Crawford game") : ( ms.fPostCrawford ?
+					_(", post-Crawford play") : ""));
     fputs( "</strong>\n", pf );
 
     fputs ( "\n<!-- End Score -->\n\n", pf );
@@ -3785,7 +3818,6 @@ ExportPositionGammOnLine( FILE *pf ) {
       iMove = getMoveNumber ( plGame, plLastMove->p );
     else
       iMove = -1;
-
     HTMLBoardHeader ( pf, &ms, HTML_EXPORT_TYPE_BBS,
                       HTML_EXPORT_CSS_INLINE,
                       getGameNumber ( plGame ), iMove, FALSE );
