@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: htmlimages.c,v 1.24 2004/03/23 10:15:49 Superfly_Jon Exp $
+ * $Id: htmlimages.c,v 1.28 2004/09/16 07:50:17 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -40,6 +40,10 @@
 #include <unistd.h>
 #endif
 
+#if HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 #if HAVE_LIBART
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_affine.h>
@@ -54,6 +58,8 @@
 #include <libart_lgpl/art_rgb_svp.h>
 #endif
 
+#include <glib.h>
+
 #include "backgammon.h"
 #include "export.h"
 #include "i18n.h"
@@ -117,7 +123,9 @@ static void Write(const char* name, unsigned char* img, int cx, int cy)
 
 static void DrawArrow(int side, int player)
 { /* side 0 = left, 1 = right */
+#if HAVE_LIBART
 	int x, y;
+#endif
 	int offset_x = 0;
 
 	memcpy( auchMidlb, auchBoard, BOARD_WIDTH * s * BOARD_HEIGHT * s * 3 );
@@ -624,7 +632,7 @@ static void RenderObjects()
 {
 	int clockwise;
 	renderdata rd;
-	memcpy( &rd, &rdAppearance, sizeof( renderdata ) );
+	CopyAppearance(&rd);
 
 	rd.fLabels = TRUE; 
 	rd.nSize = s;
