@@ -2,6 +2,7 @@
  * list.c
  *
  * by Gary Wong, 1996
+ * $Id: list.c,v 1.6 2006/09/21 22:24:45 Superfly_Jon Exp $
  */
 
 #include <list.h>
@@ -20,7 +21,7 @@ list *ListInsert( list *pl, void *p ) {
 
     list *plNew;
 
-    if( !( plNew = malloc( sizeof( *plNew ) ) ) )
+    if ( (plNew = malloc( sizeof( *plNew ))) == NULL )
 	return NULL;
 
     plNew->p = p;
@@ -46,8 +47,11 @@ int ListDelete( list *pl ) {
 
 int ListDeleteAll( list *pl ) {
 
-    while( pl->plNext->p )
-	ListDelete( pl->plNext );
+	while( pl->plNext->p )
+	{
+		free(pl->plNext->p);
+		ListDelete( pl->plNext );
+	}
 
 	return 0;
 }
