@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: neuralnet.c,v 1.20 2004/02/24 10:15:50 uid68519 Exp $
+ * $Id: neuralnet.c,v 1.28 2006/04/12 23:45:51 c_anthon Exp $
  */
 
 #include "config.h"
@@ -34,21 +34,12 @@
 #include <time.h>
 #include <assert.h>
 
-#define SIGMOID_BAUR 0
-#define SIGMOID_JTH 0
-
 #if HAVE_LIBATLAS
 #warning "LIBATLAS processing..."
 #include <cblas.h>
 #endif /* HAVE_LIBATLAS */
 
-#if SIGMOID_BAUR
-#define sigmoid sigmoid_baur
-#elif SIGMOID_JTH
-#define sigmoid sigmoid_jth
-#else
-#define sigmoid sigmoid_original
-#endif
+#include "sse.h"
 
 
 /* e[k] = exp(k/10) / 10 */
@@ -166,8 +157,8 @@ static float e[100] = {
    The Intel x87's `f2xm1' instruction makes calculating accurate
    exponentials comparatively fast, but still about 30% slower than
    the lookup table used here. */
-//static inline float sigmoid(float const xin) {
-static float sigmoid_original(float const xin) {
+
+extern /*inline*/ float sigmoid_original(float const xin) {
     
     if( !signbit( xin ) ) { /* signbit() can be faster than a compare to 0.0 */
 	/* xin is almost always positive; we place this branch of the `if'
@@ -176,17 +167,17 @@ static float sigmoid_original(float const xin) {
 	if( xin < 10.0f ) {
 	    /* again, predict the branch not to be taken */
 	    const float x1 = 10.0f * xin;
-	    const unsigned int i = (int) x1;
+	    const int i = (int)x1;
 	    
-	    return 1.0f / ( 1.0f + e[ i ] * ( 10.0f - (int) i + x1 ) );
-	} else
+		return 1 / (1 + e[i] * (10 - i + x1));
+ 	} else
 	    return 1.0f / 19931.370438230298f;
     } else {
 	if( xin > -10.0f ) {
 	    const float x1 = -10.0f * xin;
-	    const unsigned int i = (int) x1;
+	    const int i = (int)x1;
 	    
-	    return 1.0f - 1.0f / ( 1.0f + e[ i ] * ( 10.0f - (int) i + x1 ) );
+		return 1 - 1 / (1 + e[i] * (10 - i + x1));
 	} else
 	    return 19930.370438230298f / 19931.370438230298f;	
     }
@@ -250,7 +241,7 @@ ComputeSigTable (void) {
 
 #if SIGMOID_BAUR
 
-static float 
+extern float 
 sigmoid_baur (float const x) {
 
   float x2 = x * SIG_Q;
@@ -275,7 +266,7 @@ sigmoid_baur (float const x) {
 
 #if SIGMOID_JTH
 
-static float
+extern float
 sigmoid_jth( const float x ) {
 
   float x2 = x * SIG_Q;
@@ -342,33 +333,33 @@ extern int NeuralNetCreate( neuralnet *pnn, int cInput, int cHidden,
     pnn->nTrained = 0;
     pnn->fDirect = FALSE;
    
-    if( !( pnn->arHiddenWeight = malloc( cHidden * cInput *
+    if( !( pnn->arHiddenWeight = sse_malloc( cHidden * cInput *
 					 sizeof( float ) ) ) )
 	return -1;
 
-    if( !( pnn->arOutputWeight = malloc( cOutput * cHidden *
+    if( !( pnn->arOutputWeight = sse_malloc( cOutput * cHidden *
 					 sizeof( float ) ) ) ) {
-	free( pnn->arHiddenWeight );
+	sse_free( pnn->arHiddenWeight );
 	return -1;
     }
     
-    if( !( pnn->arHiddenThreshold = malloc( cHidden * sizeof( float ) ) ) ) {
-	free( pnn->arOutputWeight );
-	free( pnn->arHiddenWeight );
+    if( !( pnn->arHiddenThreshold = sse_malloc( cHidden * sizeof( float ) ) ) ) {
+	sse_free( pnn->arOutputWeight );
+	sse_free( pnn->arHiddenWeight );
 	return -1;
     }
 	   
-    if( !( pnn->arOutputThreshold = malloc( cOutput * sizeof( float ) ) ) ) {
-	free( pnn->arHiddenThreshold );
-	free( pnn->arOutputWeight );
-	free( pnn->arHiddenWeight );
+    if( !( pnn->arOutputThreshold = sse_malloc( cOutput * sizeof( float ) ) ) ) {
+	sse_free( pnn->arHiddenThreshold );
+	sse_free( pnn->arOutputWeight );
+	sse_free( pnn->arHiddenWeight );
 	return -1;
     }
 
     CheckRC();
 
-    pnn->savedBase = malloc( cHidden * sizeof( float ) ); 
-    pnn->savedIBase = malloc( cInput * sizeof( float ) ); 
+    pnn->savedBase = sse_malloc( cHidden * sizeof( float ) ); 
+    pnn->savedIBase = sse_malloc( cInput * sizeof( float ) ); 
  
     for( i = cHidden * cInput, pf = pnn->arHiddenWeight; i; i-- )
 	*pf++ = ( (int) ( irand( &rc ) & 0xFFFF ) - 0x8000 ) / 131072.0;
@@ -384,15 +375,25 @@ extern int NeuralNetCreate( neuralnet *pnn, int cInput, int cHidden,
 
     return 0;
 }
+#if HAVE_MMAP
 extern void *NeuralNetCreateDirect( neuralnet *pnn, void *p ) {
  
-   pnn->cInput = *( ( (int *) p )++ );
-   pnn->cHidden = *( ( (int *) p )++ );
-   pnn->cOutput = *( ( (int *) p )++ );
-   pnn->nTrained = *( ( (int *) p )++ );
+   float *fp;
+   int *ip = (int *)p;
+   pnn->cInput = *ip;
+   ip++;
+   pnn->cHidden = *ip;
+   ip++;
+   pnn->cOutput = *ip;
+   ip++;
+   pnn->nTrained = *ip;
+   ip++;
    pnn->fDirect = TRUE;
-   pnn->rBetaHidden = *( ( (float *) p )++ );
-   pnn->rBetaOutput = *( ( (float *) p )++ );
+   fp = (float*)ip;
+   pnn->rBetaHidden = *fp;
+   fp++;
+   pnn->rBetaOutput = *fp;
+   fp++;
  
    if( pnn->cInput < 1 || pnn->cHidden < 1 || pnn->cOutput < 1 ||
      pnn->nTrained < 0 || pnn->rBetaHidden <= 0.0 ||
@@ -402,40 +403,44 @@ extern void *NeuralNetCreateDirect( neuralnet *pnn, void *p ) {
      return NULL;
    }
  
-   pnn->arHiddenWeight = p;
-   ( (float *) p ) += pnn->cInput * pnn->cHidden;
-   pnn->arOutputWeight = p;
-   ( (float *) p ) += pnn->cHidden * pnn->cOutput;
-   pnn->arHiddenThreshold = p;
-   ( (float *) p ) += pnn->cHidden;
-   pnn->arOutputThreshold = p;
-   ( (float *) p ) += pnn->cOutput;
+   pnn->arHiddenWeight = fp;
+   fp += pnn->cInput * pnn->cHidden;
+   pnn->arOutputWeight = fp;
+   fp += pnn->cHidden * pnn->cOutput;
+   pnn->arHiddenThreshold = fp;
+   fp += pnn->cHidden;
+   pnn->arOutputThreshold = fp;
+   fp += pnn->cOutput;
 
-   pnn->savedBase = malloc( pnn->cHidden * sizeof( float ) ); 
-   pnn->savedIBase = malloc( pnn->cInput * sizeof( float ) ); 
+   pnn->savedBase = sse_malloc( pnn->cHidden * sizeof( float ) ); 
+   pnn->savedIBase = sse_malloc( pnn->cInput * sizeof( float ) ); 
 
-   return p;
+   return fp;
 }
+#endif
 
 extern int
 NeuralNetDestroy( neuralnet *pnn )
 {
   if( !pnn->fDirect ) {
-    free( pnn->arHiddenWeight ); pnn->arHiddenWeight = 0;
-    free( pnn->arOutputWeight ); pnn->arOutputWeight = 0;
-    free( pnn->arHiddenThreshold ); pnn->arHiddenThreshold = 0;
-    free( pnn->arOutputThreshold ); pnn->arOutputThreshold = 0;
+    sse_free( pnn->arHiddenWeight ); pnn->arHiddenWeight = 0;
+    sse_free( pnn->arOutputWeight ); pnn->arOutputWeight = 0;
+    sse_free( pnn->arHiddenThreshold ); pnn->arHiddenThreshold = 0;
+    sse_free( pnn->arOutputThreshold ); pnn->arOutputThreshold = 0;
   }
 
-  free(pnn->savedBase); pnn->savedBase = 0;
-  free(pnn->savedIBase); pnn->savedIBase = 0;
+  sse_free(pnn->savedBase); pnn->savedBase = 0;
+  sse_free(pnn->savedIBase); pnn->savedIBase = 0;
   
   return 0;
 }
 
-static int EvaluateOld( neuralnet *pnn, float arInput[], float ar[],
+static int Evaluate( neuralnet *pnn, float arInput[], float ar[],
                         float arOutput[], float *saveAr ) {
 
+#if HAVE_LIBATLAS
+	return EvaluateBlas( pnn, arInput, ar, arOutput, saveAr );
+#else
     int i, j;
     float *prWeight;
 
@@ -482,14 +487,13 @@ static int EvaluateOld( neuralnet *pnn, float arInput[], float ar[],
 
     return 0;
 
+#endif /* HAVE_LIBATLAS */
 }
 
-static int Evaluate( neuralnet *pnn, float arInput[], float ar[],
+#if HAVE_LIBATLAS
+static int EvaluateBlas( neuralnet *pnn, float arInput[], float ar[],
 		     float arOutput[], float *saveAr ) {
 
-#if !HAVE_LIBATLAS
-    return EvaluateOld( pnn, arInput, ar, arOutput, saveAr );
-#else /* HAVE_LIBATLAS */
     int i;
 
     /* BLAS implementation */
@@ -532,14 +536,16 @@ static int Evaluate( neuralnet *pnn, float arInput[], float ar[],
     for( i = 0; i < pnn->cOutput; i++ )
 	arOutput[ i ] = sigmoid( -pnn->rBetaOutput * arOutput[ i ] );
 
-#endif /* HAVE_LIBATLAS */
-
     return 0;
 }
+#endif /* HAVE_LIBATLAS */
 
-static int EvaluateFromBaseOld( neuralnet *pnn, float arInputDif[], float ar[],
+static int EvaluateFromBase( neuralnet *pnn, float arInputDif[], float ar[],
 		     float arOutput[] ) {
 
+#if HAVE_LIBATLAS
+	EvaluateFromBaseBlas( pnn, arInputDif, ar, arOutput );
+#else
     int i, j;
     float *prWeight;
 
@@ -586,15 +592,13 @@ static int EvaluateFromBaseOld( neuralnet *pnn, float arInputDif[], float ar[],
 
     return 0;
 
+#endif /* HAVE_LIBATLAS */
 }
 
-
-static int EvaluateFromBase( neuralnet *pnn, float arInputDif[], float ar[],
+#if HAVE_LIBATLAS
+static int EvaluateFromBaseBlas( neuralnet *pnn, float arInputDif[], float ar[],
 		     float arOutput[] ) {
 
-#if !HAVE_LIBATLAS
-    return EvaluateFromBaseOld( pnn, arInputDif, ar, arOutput );
-#else /* HAVE_LIBATLAS */
     int i;
 
     /* BLAS implementation */
@@ -628,10 +632,9 @@ static int EvaluateFromBase( neuralnet *pnn, float arInputDif[], float ar[],
     for( i = 0; i < pnn->cOutput; i++ )
 	arOutput[ i ] = sigmoid( -pnn->rBetaOutput * arOutput[ i ] );
     
-#endif /* HAVE_LIBATLAS */
-
     return 0;
 }
+#endif /* HAVE_LIBATLAS */
 
 extern int NeuralNetEvaluate( neuralnet *pnn, float arInput[],
 			      float arOutput[], NNEvalType t ) {
@@ -867,7 +870,7 @@ extern int NeuralNetResize( neuralnet *pnn, int cInput, int cHidden,
     }
     
     if( cHidden != pnn->cHidden || cInput != pnn->cInput ) {
-	if( !( pr = malloc( cHidden * cInput * sizeof( float ) ) ) )
+	if( !( pr = sse_malloc( cHidden * cInput * sizeof( float ) ) ) )
 	    return -1;
 
 	prNew = pr;
@@ -883,7 +886,7 @@ extern int NeuralNetResize( neuralnet *pnn, int cInput, int cHidden,
 		else
 		    *prNew++ = pnn->arHiddenWeight[ i * pnn->cHidden + j ];
 		    
-	free( pnn->arHiddenWeight );
+	sse_free( pnn->arHiddenWeight );
 
 	pnn->arHiddenWeight = pr;
     }
@@ -899,7 +902,7 @@ extern int NeuralNetResize( neuralnet *pnn, int cInput, int cHidden,
     }
     
     if( cOutput != pnn->cOutput || cHidden != pnn->cHidden ) {
-	if( !( pr = malloc( cOutput * cHidden * sizeof( float ) ) ) )
+	if( !( pr = sse_malloc( cOutput * cHidden * sizeof( float ) ) ) )
 	    return -1;
 
 	prNew = pr;
@@ -915,7 +918,7 @@ extern int NeuralNetResize( neuralnet *pnn, int cInput, int cHidden,
 		else
 		    *prNew++ = pnn->arOutputWeight[ i * pnn->cOutput + j ];
 
-	free( pnn->arOutputWeight );
+	sse_free( pnn->arOutputWeight );
 
 	pnn->arOutputWeight = pr;
     }
@@ -1064,3 +1067,101 @@ extern int NeuralNetSaveBinary( neuralnet *pnn, FILE *pf ) {
     
     return 0;
 }
+#if USE_SSE_VECTORIZE
+
+#ifdef _MSC_VER
+
+int SSE_Supported()
+{
+	return 1;
+}
+
+#else
+
+int CheckSSE()
+{
+	int result = 0;
+
+	asm (
+		// Check if cpuid is supported (can bit 21 of flags be changed)
+		"mov $1, %%eax\n\t"
+		"shl $21, %%eax\n\t"
+		"mov %%eax, %%edx\n\t"
+
+		"pushfl\n\t"
+		"popl %%eax\n\t"
+		"mov %%eax, %%ecx\n\t"
+		"xor %%edx, %%eax\n\t"
+		"pushl %%eax\n\t"
+		"popfl\n\t"
+
+		"pushfl\n\t"
+		"popl %%eax\n\t"
+		"xor %%ecx, %%eax\n\t"
+		"test %%edx, %%eax\n\t"
+		"jnz 1f\n\t"
+		// Failed (non-pentium compatible machine)
+		"mov $-1, %%ebx\n\t"
+		"jp 4f\n\t"
+
+"1:"
+		// Check feature test is supported
+		"mov $0, %%eax\n\t"
+		"cpuid\n\t"
+		"cmp $1, %%eax\n\t"
+		"jge 2f\n\t"
+		// Unlucky - somehow cpuid 1 isn't supported
+		"mov $-2, %%ebx\n\t"
+		"jp 4f\n\t"
+
+"2:"
+		// Check if sse is supported (bit 25 in edx from cpuid 1)
+		"mov $1, %%eax\n\t"
+		"cpuid\n\t"
+		"mov $1, %%eax\n\t"
+		"shl $25, %%eax\n\t"
+		"test %%eax, %%edx\n\t"
+		"jnz 3f\n\t"
+		// Not supported
+		"mov $0, %%ebx\n\t"
+		"jp 4f\n\t"
+"3:"
+		// Supported
+		"mov $1, %%ebx\n\t"
+"4:"
+
+			: "=b"(result) : : "%eax", "%ecx", "%edx");
+	
+	switch (result)
+	{
+	case -1:
+		printf("Can't check for SSE - non pentium cpu\n");
+		break;
+	case -2:
+		printf("No sse cpuid check available\n");
+		break;
+	case 0:
+		// No SSE support
+		break;
+	case 1:
+		// SSE support
+		return 1;
+	default:
+		printf("Unknown return testing for SSE\n");
+	}
+
+	return 0;
+}
+
+int SSE_Supported()
+{
+	static int state = -1;
+
+	if (state == -1)
+		state = CheckSSE();
+
+	return state;
+}
+
+#endif
+#endif
