@@ -2,6 +2,7 @@
  * neuralnet.h
  *
  * by Gary Wong, 1998
+ * $Id: neuralnet.h,v 1.13 2006/06/22 22:51:00 Superfly_Jon Exp $
  */
 
 #ifndef _NEURALNET_H_
@@ -9,6 +10,18 @@
 
 #include <stdio.h>
 
+#define SIGMOID_BAUR 0
+#define SIGMOID_JTH 0
+
+#if SIGMOID_BAUR
+#define sigmoid sigmoid_baur
+#elif SIGMOID_JTH
+#define sigmoid sigmoid_jth
+#else
+extern /*inline*/ float sigmoid_original(float const xin);
+#define sigmoid sigmoid_original
+#endif
+
 typedef struct _neuralnet {
     int cInput, cHidden, cOutput, nTrained, fDirect;
     float rBetaHidden, rBetaOutput, *arHiddenWeight, *arOutputWeight,
@@ -27,11 +40,16 @@ extern int NeuralNetDestroy( neuralnet *pnn );
 typedef enum  {
   NNEVAL_NONE,
   NNEVAL_SAVE,
-  NNEVAL_FROMBASE,
+  NNEVAL_FROMBASE
 } NNEvalType;
 
+extern int (*NeuralNetEvaluateFn)( neuralnet *pnn, float arInput[],
+			      float arOutput[], NNEvalType t);
+
 extern int NeuralNetEvaluate( neuralnet *pnn, float arInput[],
 			      float arOutput[], NNEvalType t);
+extern int NeuralNetEvaluate128( neuralnet *pnn, float arInput[],
+			      float arOutput[], NNEvalType t);
 extern int NeuralNetDifferentiate( neuralnet *pnn, float arInput[],
 				   float arOutput[], float arDerivative[] );
 extern int NeuralNetTrain( neuralnet *pnn, float arInput[], float arOutput[],
@@ -47,4 +65,6 @@ extern int NeuralNetSaveBinary( neuralnet *pnn, FILE *pf );
 extern void 
 ComputeSigTable (void);
 
+extern int SSE_Supported();
+
 #endif
