@@ -19,7 +19,7 @@
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
-* $Id: matchequity.c,v 1.44 2004/03/25 20:24:42 thyssen Exp $
+* $Id: matchequity.c,v 1.60 2006/09/21 22:24:05 Superfly_Jon Exp $
 */
 
 #include <stdio.h>
@@ -32,6 +32,7 @@
 #include "config.h"
 #include "list.h"
 #include "path.h"
+#include "mec.h"
 
 #if HAVE_LIBXML2
 #include <libxml/tree.h>
@@ -51,18 +52,23 @@ extern double erf( double x );
 
 #include "eval.h"
 #include "matchequity.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+
+
+#if (LIBXML_VERSION > 20412)
+const static xmlChar* XML_PUBLIC_ID = BAD_CAST "-//GNU Backgammon//DTD Match Equity Tables//EN";
+#endif
 
 typedef struct _parameter {
 
-  char *szName, *szValue;
+  xmlChar *szName, *szValue;
 
 } parameter;
 
 
 typedef struct _metparameters {
 
-  char *szName;
+  xmlChar *szName;
   list lParameters;
 
 } metparameters;
@@ -211,6 +217,7 @@ initPostCrawfordMET ( float afMETPostCrawford[ MAXSCORE ],
     assert( afMETPostCrawford[ i ] >= 0.0f &&
             afMETPostCrawford[ i ] <= 1.0f && 
             "insane post crawford equity(2)" );
+
   }
 
 
@@ -264,9 +271,9 @@ initMETZadeh ( float aafMET[ MAXSCORE ][ MAXSCORE ],
   for ( i = 0; i < MAXSCORE; i++ ) {
 
     aafMET[ i ][ 0 ] = 
-      rG2 * 0.5 *
+      rG1 * 0.5 *
       ( (i-2 >=0) ? afMETPostCrawford[ i-2 ] : 1.0 )
-      + (1.0 - rG2) * 0.5 * 
+      + (1.0 - rG1) * 0.5 * 
       ( (i-1 >=0) ? afMETPostCrawford[ i-1 ] : 1.0 );
     aafMET[ 0 ][ i ] = 1.0 - aafMET[ i ][ 0 ];
 
@@ -782,7 +789,7 @@ ExtendMET ( float aarMET[ MAXSCORE ][ MAXSCORE ],
             const int nMaxScore ) {
 
   static const float arStddevTable[] =
-  { 0, 1.24, 1.27, 1.47, 1.50, 1.60, 1.61, 1.66, 1.68, 1.70, 1.72, 1.77 };
+  { 0, 1.24f, 1.27f, 1.47f, 1.50f, 1.60f, 1.61f, 1.66f, 1.68f, 1.70f, 1.72f, 1.77f };
 
   float rStddev0, rStddev1, rGames, rSigma;
   int i,j;
@@ -800,12 +807,12 @@ ExtendMET ( float aarMET[ MAXSCORE ][ MAXSCORE ],
       rGames = ( nScore0 + nScore1 ) / 2.00f;
 
       if ( nScore0 > 10 ) 
-        rStddev0 = 1.77;
+        rStddev0 = 1.77f;
       else
         rStddev0 = arStddevTable[ nScore0 ];
 
       if ( nScore1 > 10 ) 
-        rStddev1 = 1.77;
+        rStddev1 = 1.77f;
       else
         rStddev1 = arStddevTable[ nScore1 ];
 
@@ -816,7 +823,7 @@ ExtendMET ( float aarMET[ MAXSCORE ][ MAXSCORE ],
       assert ( 6.0f * rSigma > nScore1 - nScore0 );
 
       aafMET[ i ][ j ] =
-        1.0 - NormalDistArea ( nScore1 - nScore0, 6.0f * rSigma,
+        1.0f - NormalDistArea ( (float)(nScore1 - nScore0), 6.0f * rSigma,
                                0.0f, rSigma );
 
     }
@@ -897,15 +904,15 @@ getDefaultMET ( metdata *pmd ) {
   parameter *pp;
 
   static parameter apPreCrawford[] = 
-  { { "gammon-rate-leader", "0.25" },
-    { "gammon-rate-trailer", "0.15" },
-    { "delta", "0.08" },
-    { "deltebar", "0.06" } };
+  { { BAD_CAST "gammon-rate-leader", BAD_CAST "0.25" },
+    { BAD_CAST "gammon-rate-trailer", BAD_CAST "0.15" },
+    { BAD_CAST "delta", BAD_CAST "0.08" },
+    { BAD_CAST "deltebar", BAD_CAST "0.06" } };
 
   static parameter apPostCrawford[] = 
-  { { "gammon-rate-trailer", "0.25" },
-    { "free-drop-2-away", "0.015" },
-    { "free-drop-4-away", "0.004" } };
+  { { BAD_CAST "gammon-rate-trailer", BAD_CAST "0.25" },
+    { BAD_CAST "free-drop-2-away", BAD_CAST "0.015" },
+    { BAD_CAST "free-drop-4-away", BAD_CAST "0.004" } };
 
   /* Setup default met */
 
@@ -913,26 +920,26 @@ getDefaultMET ( metdata *pmd ) {
 
   for ( i = 0; i < 4; i++ ) {
     pp = (parameter *) malloc ( sizeof ( parameter ) );
-    pp->szName = strdup ( apPreCrawford[ i ].szName );
-    pp->szValue = strdup ( apPreCrawford[ i ].szValue );
+    pp->szName = BAD_CAST strdup ( (char*) apPreCrawford[ i ].szName );
+    pp->szValue = BAD_CAST strdup ( (char*) apPreCrawford[ i ].szValue );
     ListInsert ( &pmd->mpPreCrawford.lParameters, pp );
   }
 
   for ( j = 0; j < 2; j++ ) {
     for ( i = 0; i < 3; i++ ) {
       pp = (parameter *) malloc ( sizeof ( parameter ) );
-      pp->szName = strdup ( apPostCrawford[ i ].szName );
-      pp->szValue = strdup ( apPostCrawford[ i ].szValue );
+      pp->szName = BAD_CAST strdup ( (char*) apPostCrawford[ i ].szName );
+      pp->szValue = BAD_CAST strdup ( (char*) apPostCrawford[ i ].szValue );
       ListInsert ( &pmd->ampPostCrawford[ j ].lParameters, pp );
     }
-    pmd->ampPostCrawford[ j ].szName = strdup ( "zadeh" );
+    pmd->ampPostCrawford[ j ].szName = BAD_CAST strdup ( "zadeh" );
   }
 
-  pmd->mpPreCrawford.szName = strdup ( "zadeh" );
+  pmd->mpPreCrawford.szName = BAD_CAST strdup ( "zadeh" );
 
-  pmd->mi.szName = strdup ( "N. Zadeh, Management Science 23, 986 (1977)" );
-  pmd->mi.szFileName = strdup ( "met/zadeh.xml" );
-  pmd->mi.szDescription = strdup ( "" );
+  pmd->mi.szName = BAD_CAST strdup ( "N. Zadeh, Management Science 23, 986 (1977)" );
+  pmd->mi.szFileName = BAD_CAST strdup ( "met/zadeh.xml" );
+  pmd->mi.szDescription = BAD_CAST strdup ( "" );
   pmd->mi.nLength = MAXSCORE;
 
 }
@@ -944,21 +951,16 @@ initMETFromParameters ( float aafMET [ MAXSCORE ][ MAXSCORE ],
                         int nLength,
                         metparameters *pmp ) {
 
-  if ( ! strcmp ( pmp->szName, "zadeh" ) ) {
+  if ( ! strcmp ( (char*)pmp->szName, "mec" ) ) {
     
-    float rG1 = 0.25;
-    float rG2 = 0.15;
-    float rDelta = 0.08;
-    float rDeltaBar = 0.06;
+    float rG = 0.15f;
+    float rWR = 0.5f;
     list *pl; 
     parameter *pp;
 
     /* 
-     * Use Zadeh's formulae 
      */
 
-    PushLocale ( "C" );
-
     /* obtain parameters */
 
     for ( pl = pmp->lParameters.plNext; pl != &pmp->lParameters; 
@@ -966,19 +968,50 @@ initMETFromParameters ( float aafMET [ MAXSCORE ][ MAXSCORE ],
 
       pp = pl->p;
 
-      if ( ! strcmp ( pp->szName, "gammon-rate-leader" ) )
-        rG1 = atof ( pp->szValue );
-      else if ( ! strcmp ( pp->szName, "gammon-rate-trailer" ) )
-        rG2 = atof ( pp->szValue );
-      else if ( ! strcmp ( pp->szName, "delta" ) )
-        rDelta = atof ( pp->szValue );
-      else if ( ! strcmp ( pp->szName, "delta-bar" ) )
-        rDeltaBar = atof ( pp->szValue );
+      if ( ! strcmp ( (char*)pp->szName, "gammon-rate" ) )
+        rG = (float) g_ascii_strtod ( (char*)pp->szValue, NULL );
+      else if ( ! strcmp ( (char*)pp->szName, "win-rate" ) )
+        rWR = (float) g_ascii_strtod ( (char*)pp->szValue, NULL );
 
     }
 
-    PopLocale ();
+    /* calculate table */
+
+    mec( rG, rWR, aafMETPostCrawford, aafMET );
+
+    return 0;
+
+  }
+  else if ( ! strcmp ( (char*) pmp->szName, "zadeh" ) ) {
     
+    float rG1 = 0.25f;
+    float rG2 = 0.15f;
+    float rDelta = 0.08f;
+    float rDeltaBar = 0.06f;
+    list *pl; 
+    parameter *pp;
+
+    /* 
+     * Use Zadeh's formulae 
+     */
+
+    /* obtain parameters */
+
+    for ( pl = pmp->lParameters.plNext; pl != &pmp->lParameters; 
+          pl = pl->plNext ) {
+
+      pp = pl->p;
+
+      if ( ! strcmp ( (char*) pp->szName, "gammon-rate-leader" ) )
+        rG1 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "gammon-rate-trailer" ) )
+        rG2 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "delta" ) )
+        rDelta = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "delta-bar" ) )
+        rDeltaBar = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+
+    }
 
     /* calculate table */
 
@@ -1004,11 +1037,11 @@ initPostCrawfordMETFromParameters ( float afMETPostCrawford[ MAXSCORE ],
                                     int nLength,
                                     metparameters *pmp ) {
 
-  if ( ! strcmp ( pmp->szName, "zadeh" ) ) {
+  if ( ! strcmp ( (char*) pmp->szName, "zadeh" ) ) {
     
-    float rG = 0.25;
-    float rFD2 = 0.015;
-    float rFD4 = 0.004;
+    float rG = 0.25f;
+    float rFD2 = 0.015f;
+    float rFD4 = 0.004f;
     list *pl;
     parameter *pp;
 
@@ -1016,8 +1049,6 @@ initPostCrawfordMETFromParameters ( float afMETPostCrawford[ MAXSCORE ],
      * Use Zadeh's formulae 
      */
 
-    PushLocale ( "C" );
-
     /* obtain parameters */
 
     for ( pl = pmp->lParameters.plNext; pl != &pmp->lParameters; 
@@ -1025,22 +1056,54 @@ initPostCrawfordMETFromParameters ( float afMETPostCrawford[ MAXSCORE ],
 
       pp = pl->p;
 
-      if ( ! strcmp ( pp->szName, "gammon-rate-trailer" ) )
-        rG = atof ( pp->szValue );
-      else if ( ! strcmp ( pp->szName, "free-drop-2-away" ) )
-        rFD2 = atof ( pp->szValue );
-      else if ( ! strcmp ( pp->szName, "free-drop-4-away" ) )
-        rFD4 = atof ( pp->szValue );
+      if ( ! strcmp ( (char*) pp->szName, "gammon-rate-trailer" ) )
+        rG = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "free-drop-2-away" ) )
+        rFD2 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "free-drop-4-away" ) )
+        rFD4 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
 
     }
 
-    PopLocale ();
-
     /* calculate table */
 
     initPostCrawfordMET ( afMETPostCrawford, 0, rG, rFD2, rFD4 );
     return 0;
 
+  }
+  else if ( ! strcmp ( (char*) pmp->szName, "mec" ) ) {
+
+    float rG = 0.25f;
+    float rFD2 = 0.015f;
+    float rFD4 = 0.004f;
+    float rWR = 0.5f;
+    list *pl;
+    parameter *pp;
+
+    /* obtain parameters */
+
+    for ( pl = pmp->lParameters.plNext; pl != &pmp->lParameters; 
+          pl = pl->plNext ) {
+
+      pp = pl->p;
+
+      if ( ! strcmp ( (char*) pp->szName, "gammon-rate" ) )
+        rG = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "free-drop-2-away" ) )
+        rFD2 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "free-drop-4-away" ) )
+        rFD4 = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+      else if ( ! strcmp ( (char*) pp->szName, "win-rate" ) )
+        rWR = (float) g_ascii_strtod ( (char*) pp->szValue, NULL );
+
+    }
+
+    /* calculate table */
+
+    mec_pc( rG, rFD2, rFD4, rWR, afMETPostCrawford );
+
+    return 0;
+
   }
   else {
 
@@ -1066,12 +1129,12 @@ parseRow ( float arRow[], xmlDocPtr doc, xmlNodePtr root ) {
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "me" ) ) {
-      arRow[ iCol ]  = 
-        atof ( xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 ) );
+    if ( ! strcmp ( (char*) (char*)cur->name, "me" ) ) {
+      xmlChar *row = xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 );
+      arRow[ iCol ]  = g_ascii_strtod( (char*)row , NULL );
+      xmlFree(row);        
 
       iCol++;
-
     }
 
   }
@@ -1089,7 +1152,7 @@ parsePreCrawfordExplicit ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root ) {
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "row" ) )
+    if ( ! strcmp ( (char*)cur->name, "row" ) )
       parseRow ( pmd->aarMET[ iRow++ ], doc, cur );
 
   }
@@ -1100,20 +1163,22 @@ static void
 parseParameters ( list *plList, xmlDocPtr doc, xmlNodePtr root ) {
 
   xmlNodePtr cur;
-  char *pc;
+  xmlChar *pc;
   parameter *pp;
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "parameter" ) ) {
+    if ( ! strcmp ( (char*)cur->name, "parameter" ) ) {
 
       pp = (parameter *) malloc ( sizeof ( parameter ) );
 
-      pc = xmlGetProp ( cur, "name" );
-      pp->szName = strdup ( pc ? pc : "" );
+      pc = xmlGetProp ( cur, BAD_CAST "name" );
+      pp->szName = BAD_CAST strdup ( (char*) (pc ? pc : BAD_CAST "" ));
+      xmlFree(pc);
       
       pc = xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 );
-      pp->szValue = strdup ( pc ? pc : "" );
+      pp->szValue = BAD_CAST strdup ( (char*) (pc ? pc : BAD_CAST ""));
+      xmlFree(pc);
 
       ListInsert ( plList, pp );
 
@@ -1132,7 +1197,7 @@ parsePreCrawfordFormula ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root ) {
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "parameters" ) )
+    if ( ! strcmp ( (char*) cur->name, "parameters" ) )
       parseParameters ( &pmd->mpPreCrawford.lParameters, 
                         doc, cur );
 
@@ -1149,7 +1214,7 @@ parsePostCrawfordFormula ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root,
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "parameters" ) ) {
+    if ( ! strcmp ( (char*) cur->name, "parameters" ) ) {
 
       if ( fPlayer < 0 ) {
         parseParameters ( &pmd->ampPostCrawford[ 0 ].lParameters, 
@@ -1176,7 +1241,7 @@ parsePostCrawfordExplicit ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root,
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "row" ) ) {
+    if ( ! strcmp ( (char*) cur->name, "row" ) ) {
 
       if ( fPlayer < 0 ) {
         parseRow ( pmd->aarMETPostCrawford[ 0 ], doc, cur );
@@ -1196,7 +1261,7 @@ parsePostCrawfordExplicit ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root,
 
 static void parsePreCrawford ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root ) {
 
-  if ( ! strcmp ( pmd->mpPreCrawford.szName, "explicit" ) )
+  if ( ! strcmp ( (char*) pmd->mpPreCrawford.szName, "explicit" ) )
     parsePreCrawfordExplicit ( pmd, doc, root );
   else
     parsePreCrawfordFormula ( pmd, doc, root );
@@ -1209,7 +1274,7 @@ static void parsePostCrawford ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root,
 
   int i = ( fPlayer < 0 ) ? 0 : fPlayer;
 
-  if ( ! strcmp ( pmd->ampPostCrawford[ i ].szName, "explicit" ) )
+  if ( ! strcmp ( (char*) pmd->ampPostCrawford[ i ].szName, "explicit" ) )
     parsePostCrawfordExplicit ( pmd, doc, root, fPlayer );
   else
     parsePostCrawfordFormula ( pmd, doc, root, fPlayer );
@@ -1220,26 +1285,32 @@ static void parsePostCrawford ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root,
 static void parseInfo ( metdata *pmd, xmlDocPtr doc, xmlNodePtr root ) {
 
   xmlNodePtr cur;
-  char *pc;
+  xmlChar *pc;
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "name" ) ) {
+    if ( ! strcmp ( (char*) cur->name, "name" ) ) {
       pc = xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 );
-      pmd->mi.szName = ( pc ) ? strdup ( pc ) : NULL;
+      pmd->mi.szName = ( pc ) ? BAD_CAST strdup ( (char*) pc ) : NULL;
+      xmlFree(pc);
     }
-    else if ( ! strcmp ( cur->name, "description" ) ) {
+    else if ( ! strcmp ( (char*) cur->name, "description" ) ) {
       pc = xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 );
-      pmd->mi.szDescription = ( pc ) ? strdup ( pc ) : NULL;
+      pmd->mi.szDescription = ( pc ) ? BAD_CAST strdup ( (char*) pc ) : NULL;
+      xmlFree(pc);
+    }
+    else if ( ! strcmp ( (char*) cur->name, "length" ) )
+    {
+      xmlChar* len = xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 );
+      pmd->mi.nLength = atoi((char*)len);
+      xmlFree(len);
     }
-    else if ( ! strcmp ( cur->name, "length" ) )
-      pmd->mi.nLength =
-        atoi ( xmlNodeListGetString ( doc, cur->xmlChildrenNode, 1 ) );
-
   }
 
 }
 
+#if (LIBXML_VERSION > 20412)
+
 static void validateWarning ( void *ctx,
                         const char *msg, 
                         ... ) {
@@ -1264,6 +1335,7 @@ static void validateError ( void *ctx,
 
 }
 
+#endif
 
 static int readMET ( metdata *pmd, const char *szFileName,
 		     const char *szDir ) {
@@ -1271,9 +1343,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
   xmlDocPtr doc;
   xmlNodePtr root, cur;
 
-  xmlValidCtxt ctxt;
-
-  char *pc, *pch;
+  xmlChar *pc, *pch;
 
   int fError;
 
@@ -1281,7 +1351,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
 
   /* parse document */
 
-  doc = xmlParseFile( pch = PathSearch( szFileName, szDir ) );
+  doc = xmlParseFile( (char*) (pch = BAD_CAST PathSearch( szFileName, szDir )) );
   free( pch );
   
   /* check root */
@@ -1290,18 +1360,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
 
   if ( ! root ) {
 
-    printf ( _("Error reading XML file (%s): no document root!\n"), szFileName );
-    fError = 1;
-    goto finish;
-
-  }
-
-  /* check if the dtd has the correct name */
-
-  if ( xmlStrcmp ( root->name, (const xmlChar *) "met" ) ) {
-
-    printf ( _("Error reading XML file (%s): wrong DTD (%s)\n"), 
-             szFileName, root->name );
+    printf ( _("Error reading XML file (%s): not well-formed!\n"), szFileName );
     fError = 1;
     goto finish;
 
@@ -1311,79 +1370,100 @@ static int readMET ( metdata *pmd, const char *szFileName,
    * validate document 
    */
 
-  /* load catalogs */
+/* libxml2 version 2.4.3 introduced xml catalogs, it dates 25th august 2001 ... */
+/* older versions used SGML format catalogs, but it's not clear when the default behaviour changed */
+#if (LIBXML_VERSION > 20412)
+{
+  xmlValidCtxtPtr ctxt;
+  xmlDtdPtr dtd;
 
-  /* FIXME: any tweaks needed for win32? */
+  /* load catalog */
 
-  xmlLoadCatalog( pch = PathSearch( "met/catalog", szDir ) );
-  free( pch );
-  xmlLoadCatalog ( PKGDATADIR "/met/catalog " );
-  if ( ( pc = getenv ( "SGML_CATALOG_FILES" ) ) )
-    xmlLoadCatalog ( pc );
-  xmlLoadCatalog ( "./met/catalog" );
-  xmlLoadCatalog ( "./catalog" );
+  	xmlInitializeCatalog();
+	pch = NULL;
+	
+	if (0 == xmlLoadCatalog((char*) (pch = BAD_CAST PathSearch( "met/catalog.xml", szDir )))) {}
+	else if ( 0 == xmlLoadCatalog(PKGDATADIR "/met/catalog.xml")) {}
+	else if ( 0 == xmlLoadCatalog ( "./met/catalog.xml" )) {}
+	else if ( 0 == xmlLoadCatalog ( "./catalog.xml" )) {}
+    else {
+      printf ( _("Error reading \"catalog.xml\". File not found or parse error.") );
+	      fError = 1;
+		  if (pch)  free( pch );
+		  goto finish;
+  }
+  if (pch) free( pch );
 
-  pc = xmlCatalogResolve ( doc->intSubset->ExternalID, 
-                           doc->intSubset->SystemID );
-  fError = ! pc;
-  free ( pc );
+  /* load dtd */
 
-  if ( fError ) {
-
-    printf ( _("met DTD not found\n") );
+  dtd = xmlParseDTD(XML_PUBLIC_ID, NULL);
+  if (!dtd) {
+	  pch = xmlCatalogResolvePublic(XML_PUBLIC_ID);
+	  dtd = xmlParseDTD(NULL, pch);
+	  if (pch) free(pch);
+  }
+  if (!dtd) {
+    printf ( _("Error resolving DTD for public ID %s"), XML_PUBLIC_ID );
     fError = 1;
     goto finish;
 
   }
 
-  ctxt.error = validateError;
-  ctxt.warning = validateWarning;
+#if (LIBXML_VERSION > 20507)
+  /* validate against the DTD */
+  ctxt = xmlNewValidCtxt();
+  ctxt->error = validateError;
+  ctxt->warning = validateWarning;
 
-  if ( ! xmlValidateDocument ( &ctxt, doc ) ) {
+  if ( !(xmlValidateDtd(ctxt, doc, dtd) && xmlValidateDtdFinal(ctxt, doc)) ) {
 
-    printf ( _("XML does not validate!\n") );
+    printf ( _("Error reading XML file (%s): not valid!\n"), szFileName );
     fError = 1;
     goto finish;
 
   }
 
+  if (ctxt) xmlFreeValidCtxt(ctxt);
+  if (dtd) xmlFreeDtd(dtd);
+#endif /* XMLVERSION > 20507 */
+}
+#endif /* XMLVERSION */
+
   /* initialise data */
 
-  PushLocale ( "C" );
-
   initMD ( pmd );
 
   /* fetch information from xml doc */
 
   for ( cur = root->xmlChildrenNode; cur; cur = cur->next ) {
 
-    if ( ! strcmp ( cur->name, "info" ) )
+    if ( ! strcmp ( (char*) cur->name, "info" ) )
       parseInfo ( pmd, doc, cur );
-    else if ( ! strcmp ( cur->name, "pre-crawford-table" ) ) {
+    else if ( ! strcmp ( (char*) cur->name, "pre-crawford-table" ) ) {
 
-      pc = xmlGetProp ( cur, "type" );
+      pc = xmlGetProp ( cur, BAD_CAST "type" );
       if ( pc )
         pmd->mpPreCrawford.szName = pc;
       else
-        pmd->mpPreCrawford.szName = strdup ( "explicit" );
+        pmd->mpPreCrawford.szName = BAD_CAST strdup ( "explicit" );
 
       parsePreCrawford ( pmd, doc, cur );
 
     }
-    else if ( ! strcmp ( cur->name, "post-crawford-table" ) ) {
+    else if ( ! strcmp ( (char*) cur->name, "post-crawford-table" ) ) {
 
       int fPlayer;
       int i;
 
-      pc = xmlGetProp ( cur, "player" );
+      pc = xmlGetProp ( cur, BAD_CAST "player" );
       if ( pc ) {
         
         /* attribute "player" was specified */
 
-        if ( ! strcmp ( pc, "both" ) || ! strcmp ( pc, "all" ) )
+        if ( ! strcmp ( (char*) pc, "both" ) || ! strcmp ( (char*) pc, "all" ) )
           fPlayer = -1;
         else
-          fPlayer = atoi ( pc );
+          fPlayer = atoi ( (char*)pc );
 
         if ( fPlayer < -1 || fPlayer > 1 ) {
           printf ( _("Illegal value for attribute player: '%s'\n"), pc );
@@ -1391,6 +1471,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
           goto finish;
         }
 
+        xmlFree(pc);
       }
       else
         /* assume "both" */
@@ -1401,12 +1482,12 @@ static int readMET ( metdata *pmd, const char *szFileName,
 
         if ( fPlayer < 0 || i == fPlayer ) {
 
-          pc = xmlGetProp ( cur, "type" );
+          pc = xmlGetProp ( cur, BAD_CAST "type" );
 
           if ( pc )
-            pmd->ampPostCrawford[ i ].szName = pc;
+            pmd->ampPostCrawford[ i ].szName = BAD_CAST pc;
           else
-            pmd->ampPostCrawford[ i ].szName = strdup ( "explicit" );
+            pmd->ampPostCrawford[ i ].szName = BAD_CAST strdup ( "explicit" );
         }
 
       }
@@ -1421,6 +1502,8 @@ static int readMET ( metdata *pmd, const char *szFileName,
 
 #ifdef UNDEF
 
+  int i, j;
+
   /* debug dump */
 
   printf ( "Name          : '%s'\n", pmd->mi.szName );
@@ -1428,7 +1511,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
   printf ( "Length        : %d\n", pmd->mi.nLength );
   printf ( "pre-Crawford:\n" );
 
-  if ( ! strcmp ( pmd->mpPreCrawford.szName, "explicit" ) ) {
+  if ( ! strcmp ( (char*) pmd->mpPreCrawford.szName, "explicit" ) ) {
     for ( j = 0; j < pmd->mi.nLength; j++ ) {
       printf ( "row %2d: ", j );
       for ( i = 0; i < pmd->mi.nLength; i++ ) {
@@ -1438,9 +1521,7 @@ static int readMET ( metdata *pmd, const char *szFileName,
     }
   }
 
-  printf ( "post-Crawford:\n" );
-
-  if ( ! strcmp ( pmd->mpPostCrawford.szName, "explicit" ) ) {
+  if ( ! strcmp ( (char*) pmd->mpPostCrawford.szName, "explicit" ) ) {
     printf ( "row %2d: ", 0 );
     for ( i = 0; i < pmd->mi.nLength; i++ ) {
       printf ( "%5.3f ", pmd->arMETPostCrawford[ i ] );
@@ -1452,21 +1533,17 @@ static int readMET ( metdata *pmd, const char *szFileName,
 
  finish:
 
-  PopLocale ();
-
   if ( doc )
     xmlFreeDoc ( doc );
 
   if ( ! fError )
-    pmd->mi.szFileName = strdup ( szFileName );
+    pmd->mi.szFileName = BAD_CAST strdup ( (char*) szFileName );
 
   return fError;
 
-  
-  
 }
 
-#endif
+#endif /* HAVE_LIBXML */
 
 
 /*
@@ -1491,7 +1568,7 @@ getGammonPrice ( float arGammonPrice[ 4 ],
                  float aafMET[ MAXSCORE ][ MAXSCORE ],
                  float aafMETPostCrawford[ 2 ][ MAXSCORE ] ) {
 
-  const float epsilon = 1.0E-7;
+  const float epsilon = 1.0E-7f;
 
   float rWin = 
     getME ( nScore0, nScore1, nMatchTo,
@@ -1622,8 +1699,6 @@ calcGammonPrices ( float aafMET[ MAXSCORE ][ MAXSCORE ],
 
 }
 
-
-
 extern void
 InitMatchEquity ( const char *szFileName, const char *szDir ) {
 
@@ -1676,7 +1751,7 @@ InitMatchEquity ( const char *szFileName, const char *szDir ) {
 
   for ( j = 0; j < 2; j++ ) {
 
-    if ( ! strcmp ( md.ampPostCrawford[ j ].szName, "explicit" ) ) {
+    if ( ! strcmp ( (char*) md.ampPostCrawford[ j ].szName, "explicit" ) ) {
 
       /* copy and extend table */
 
@@ -1691,7 +1766,7 @@ InitMatchEquity ( const char *szFileName, const char *szDir ) {
         aafMETPostCrawford[ j ][ i ] = md.aarMETPostCrawford[ j ][ i ];
 
       initPostCrawfordMET ( aafMETPostCrawford[ j ], md.mi.nLength - 1, 
-                            GAMMONRATE, 0.015, 0.004 );
+                            GAMMONRATE, 0.015f, 0.004f );
 
     }
     else {
@@ -1713,7 +1788,7 @@ InitMatchEquity ( const char *szFileName, const char *szDir ) {
 
   /* pre-Crawford table */
 
-  if ( ! strcmp ( md.mpPreCrawford.szName, "explicit" ) ) {
+  if ( ! strcmp ( (char*) md.mpPreCrawford.szName, "explicit" ) ) {
 
     /* copy table */
 
@@ -1906,9 +1981,7 @@ invertMET ( void ) {
       aafMET[ j ][ i ] = 1.0 - r;
 
     }
-    
   }
-
 }
 
 /* given a match score, return a pair of arrays with the METs for
