@@ -18,25 +18,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: osr.c,v 1.12 2004/01/30 09:33:50 uid68519 Exp $
+ * $Id: osr.c,v 1.16 2006/04/12 06:53:35 Superfly_Jon Exp $
  */
 
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
 
+#if HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 
 #include "config.h"
 
+#include "backgammon.h"
 #include "eval.h"
 #include "positionid.h"
 #include "osr.h"
-#include "mt19937int.h"
+#include "mt19937ar.h"
 #include "bearoff.h"
 
 #define MAX_PROBS        32
 #define MAX_GAMMON_PROBS 15
-#define min(x,y)   (((x) > (y)) ? (y) : (x))
+
+static unsigned long mt[ N ];
+static int mti = N + 1;
 
 static int 
 OSRQuasiRandomDice( const int iTurn, const int iGame, const int cGames,
@@ -51,8 +57,8 @@ OSRQuasiRandomDice( const int iTurn, const int iGame, const int cGames,
     anDice[ 1 ] = ( ( iGame / 216 ) % 6 ) + 1;
     return 0;
   } else {
-    anDice[ 0 ] = ( genrand() % 6 ) + 1;
-    anDice[ 1 ] = ( genrand() % 6 ) + 1;
+    anDice[ 0 ] = ( genrand_int32(&mti,mt) % 6 ) + 1;
+    anDice[ 1 ] = ( genrand_int32(&mti,mt) % 6 ) + 1;
     return ( anDice[ 0 ] > 0 && anDice[ 1 ] > 0 );
   }
 }
@@ -661,14 +667,14 @@ rollOSR ( const int nGames, const int anBoard[ 25 ], const int nOut,
 
     /* update counts */
 
-    ++anCounts[ min ( m == 15 ? n + 1 : n, nMaxGammonProbs - 1 ) ];
+    ++anCounts[ MIN( m == 15 ? n + 1 : n, nMaxGammonProbs - 1 ) ];
 
     /* get prob. from bearoff1 */
 
     getBearoffProbs ( PositionBearoff ( an, 6, 15 ), anProb );
 
     for ( i = 0; i < 32; ++i )
-      arProbs[ min ( n + i, nMaxProbs - 1 ) ] += anProb[ i ] / 65535.0f;
+      arProbs[ MIN( n + i, nMaxProbs - 1 ) ] += anProb[ i ] / 65535.0f;
 
   }
 
@@ -755,7 +761,7 @@ osp ( const int anBoard[ 25 ], const int nGames,
     getBearoffProbs ( PositionBearoff ( anBoard, 6, 15 ), anProb );
 
     for ( i = 0; i < 32; ++i ) {
-      n = min ( i, MAX_PROBS - 1 );
+      n = MIN( i, MAX_PROBS - 1 );
       arProbs[ n ] += anProb[ i ] / 65535.0f;
       /* printf ( "arProbs[%d]=%f\n", n, arProbs[n] ); */
     }
@@ -855,7 +861,9 @@ raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
 
   float w, s;
 
-  sgenrand ( 0 );
+  /* Seed set to ensure that OSR are reproducable */
+
+  init_genrand ( 0, &mti, mt );
 
   for ( i = 0; i < 5; ++i )
     arOutput[ i ] = 0.0f;
@@ -885,7 +893,7 @@ raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
 
   }
 
-  arOutput[ OUTPUT_WIN ] = min ( w, 1.0f );
+  arOutput[ OUTPUT_WIN ] = MIN( w, 1.0f );
 
   /* calculate gammon and backgammon probs */
 
@@ -922,10 +930,10 @@ raceProbs ( int anBoard[ 2 ][ 25 ], const int nGames,
 
   }
 
-  arOutput[ OUTPUT_WINGAMMON ] = min ( arG[ 1 ], 1.0f );
-  arOutput[ OUTPUT_LOSEGAMMON ] = min ( arG[ 0 ], 1.0f );
-  arOutput[ OUTPUT_WINBACKGAMMON ] = min ( arBG[ 1 ], 1.0f );
-  arOutput[ OUTPUT_LOSEBACKGAMMON ] = min ( arBG[ 0 ], 1.0f );
+  arOutput[ OUTPUT_WINGAMMON ] = MIN( arG[ 1 ], 1.0f );
+  arOutput[ OUTPUT_LOSEGAMMON ] = MIN( arG[ 0 ], 1.0f );
+  arOutput[ OUTPUT_WINBACKGAMMON ] = MIN( arBG[ 1 ], 1.0f );
+  arOutput[ OUTPUT_LOSEBACKGAMMON ] = MIN( arBG[ 0 ], 1.0f );
 
   /* calculate average number of rolls to bear off */
 
