@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: progress.c,v 1.16 2004/03/23 10:17:58 Superfly_Jon Exp $
+ * $Id: progress.c,v 1.20 2005/05/08 12:34:56 Superfly_Jon Exp $
  */
 
 #include "config.h"
@@ -66,13 +66,52 @@ typedef struct _rolloutprogress {
   GtkWidget *pwElapsed;
   GtkWidget *pwLeft;
   GtkWidget *pwSE;
+  int nGamesDone;
+  char ***pListText;
 #endif
 
 } rolloutprogress;
 
+#if USE_GTK
+void AllocTextList(rolloutprogress *prp)
+{	// 2d array to cache displayed widget text
+	int i;
+	int lines = prp->n;
+	prp->pListText = malloc(sizeof(char*) * lines * 2);
 
+	for (i = 0; i < lines; i++)
+	{
+		prp->pListText[i * 2] = malloc(sizeof(char*) * (NUM_ROLLOUT_OUTPUTS + 2));
+		memset(prp->pListText[i * 2], 0, sizeof(char*) * (NUM_ROLLOUT_OUTPUTS + 2));
+		prp->pListText[i * 2 + 1] = malloc(sizeof(char*) * (NUM_ROLLOUT_OUTPUTS + 2));
+		memset(prp->pListText[i * 2 + 1], 0, sizeof(char*) * (NUM_ROLLOUT_OUTPUTS + 2));
+	}
+}
 
+void FreeTextList(rolloutprogress *prp)
+{	// destroy list
+	int i;
+	int lines = prp->n;
 
+	for (i = 0; i < lines; i++)
+	{
+		free(prp->pListText[i * 2]);
+		free(prp->pListText[i * 2 + 1]);
+	}
+	free(prp->pListText);
+}
+
+void SetRolloutText(rolloutprogress *prp, int x, int y, char* sz)
+{	// Cache set text to reduce flicker (and speed things up a bit)
+	if (!prp->pListText[x][y] || strcmp(prp->pListText[x][y - 1], sz))
+	{
+		gtk_clist_set_text(GTK_CLIST(prp->pwRolloutResult), x, y, sz);
+		free(prp->pListText[x][y - 1]);
+		prp->pListText[x][y - 1] = malloc(strlen(sz) + 1);
+		strcpy(prp->pListText[x][y - 1], sz);
+	}
+}
+#endif
 
 
 /*
@@ -158,7 +197,7 @@ GTKStatPageWin ( const rolloutstat *prs, const int cGames ) {
 
   /* table with results */
 
-  aszColumnTitle[ 0 ] = strdup ( _("Cube") );
+  aszColumnTitle[ 0 ] = g_strdup ( _("Cube") );
   for ( i = 0; i < 2; i++ ) {
     aszColumnTitle[ 3 * i + 1 ] = 
       g_strdup_printf ( _("Win Single\n%s"), ap[ i ].szName );
@@ -247,7 +286,7 @@ GTKStatPageCube ( const rolloutstat *prs, const int cGames ) {
   char sz[ 100 ];
   int anTotal[ 4 ];
 
-  aszColumnTitle[ 0 ] = strdup ( _("Cube") );
+  aszColumnTitle[ 0 ] = g_strdup ( _("Cube") );
   for ( i = 0; i < 2; i++ ) {
     aszColumnTitle[ 2 * i + 1 ] = 
       g_strdup_printf ( _("#Double, take\n%s"), ap[ i ].szName );
@@ -660,7 +699,8 @@ GTKViewRolloutStatistics(GtkWidget *widget, gpointer data){
   
   rolloutprogress *prp = (rolloutprogress *) data;
   rolloutstat *prs = prp->prs;
-  int cGames = gtk_progress_get_value ( GTK_PROGRESS( prp->pwRolloutProgress ) );
+  // int cGames = gtk_progress_get_value ( GTK_PROGRESS( prp->pwRolloutProgress ) );
+  int cGames = prp->nGamesDone;
   int nRollouts = GTK_CLIST( prp->pwRolloutResult )->rows / 2;
   int i;
 
@@ -770,6 +810,8 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
   prp->n = n;
   fInterrupt = FALSE;
 
+  AllocTextList(prp);
+
   prp->pwRolloutDialog = 
     GTKCreateDialog( _("GNU Backgammon - Rollout"), DT_INFO,
                      NULL, NULL );
@@ -814,8 +856,10 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
   gtk_clist_column_titles_passive( GTK_CLIST( prp->pwRolloutResult ) );
     
   prp->pwRolloutProgress = gtk_progress_bar_new();
+#if !USE_GTK2
   gtk_progress_set_show_text ( GTK_PROGRESS ( prp->pwRolloutProgress ), TRUE );
-
+#endif
+  
   gtk_box_pack_start( GTK_BOX( pwVbox ), prp->pwRolloutResult, TRUE, TRUE, 0 );
   gtk_box_pack_start( GTK_BOX( pwVbox ), prp->pwRolloutProgress, FALSE, FALSE,
                       0 );
@@ -837,11 +881,13 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
                         _("Standard error") );
   }
 
+#if !USE_GTK2
   gtk_progress_configure( GTK_PROGRESS( prp->pwRolloutProgress ), 0, 0, 
                           prc->nTrials );
   gtk_progress_set_format_string( GTK_PROGRESS( prp->pwRolloutProgress ),
                                   "%v/%u (%p%%)" );
-
+#endif
+  
   /* time elapsed and left */
 
   pwhbox = gtk_hbox_new( FALSE, 4 );
@@ -897,8 +943,6 @@ GTKRolloutProgressStart( const cubeinfo *pci, const int n,
 
 }
 
-
-
 static void
 GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
                     float aarStdDev[][ NUM_ROLLOUT_OUTPUTS ],
@@ -914,6 +958,10 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 
     char sz[ 32 ];
     int i;
+#if USE_GTK2
+    gchar *gsz;
+    double frac;
+#endif
 
     if( !prp ||  !prp->pwRolloutResult )
       return;
@@ -933,8 +981,7 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
                 prc->fCubeful ? OutputMWC( aarOutput[ iAlternative ][ i ],
                                            &aci[ 0 ], TRUE ) : "n/a" );
 
-      gtk_clist_set_text( GTK_CLIST( prp->pwRolloutResult ), 
-                          iAlternative * 2, i + 1, sz );
+		SetRolloutText(prp, iAlternative * 2, i + 1, sz);
 
       /* standard errors */
 
@@ -949,30 +996,39 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
                 prc->fCubeful ? OutputMWC( aarStdDev[ iAlternative ][ i ],
                                            &aci[ 0 ], FALSE ) : "n/a" );
 
-      gtk_clist_set_text( GTK_CLIST( prp->pwRolloutResult ), 
-                          iAlternative * 2 + 1, i + 1, sz );
+		SetRolloutText(prp, iAlternative * 2 + 1, i + 1, sz);
 
     }
 
     if (fShowRanks && iGame > 1) {
 	  sprintf (sz, "%d %s", nRank, fStopped ? "s" : "r");
-	  gtk_clist_set_text( GTK_CLIST( prp->pwRolloutResult ),
-			      iAlternative * 2, i + 1, sz);
+	  SetRolloutText(prp, iAlternative * 2, i + 1, sz);
 	  if (nRank != 1)
 	    sprintf( sz,  "%5.3f", rJsd);
 	  else
 	    strcpy (sz, " ");
 
-	  gtk_clist_set_text( GTK_CLIST( prp->pwRolloutResult ),
-			      iAlternative * 2 + 1, i + 1, sz);
+	  SetRolloutText(prp, iAlternative * 2 + 1, i + 1, sz);
 	} else {
-	  gtk_clist_set_text( GTK_CLIST( prp->pwRolloutResult ),
-						  iAlternative * 2, i + 1, "n/a");
+	  SetRolloutText(prp, iAlternative * 2, i + 1, "n/a");
 	}
 	  
+#if !USE_GTK2
     gtk_progress_configure( GTK_PROGRESS( prp->pwRolloutProgress ),
                             iGame + 1, 0, prc->nTrials );
-
+#else
+    frac = (iGame + 1) * 1.0 / (prc->nTrials * 1.0); 
+    /* why doesn't type casting work? */
+    gsz = g_strdup_printf( "%d/%d (%.0f%%)" ,
+		    iGame + 1 , prc->nTrials, 100 * frac );
+    prp->nGamesDone = iGame + 1;
+     
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR( prp->pwRolloutProgress),
+		    frac );
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR( prp->pwRolloutProgress), gsz );
+    g_free( gsz );
+#endif
+    
     /* calculate estimate time left */
 
     if ( iAlternative == ( prp->n - 1 ) ) {
@@ -1022,6 +1078,10 @@ GTKRolloutProgress( float aarOutput[][ NUM_ROLLOUT_OUTPUTS ],
 }
 
 static void GTKRolloutProgressEnd( void **pp ) {
+    
+#if USE_GTK2
+    gchar *gsz;
+#endif
 
     rolloutprogress *prp = *pp;
 
@@ -1029,6 +1089,8 @@ static void GTKRolloutProgressEnd( void **pp ) {
 
     pwGrab = pwOldGrab;
 
+    FreeTextList(prp);
+
     /* if they cancelled the rollout early, 
        prp->pwRolloutDialog has already been destroyed */
     if( !prp->pwRolloutDialog ) {
@@ -1040,8 +1102,15 @@ static void GTKRolloutProgressEnd( void **pp ) {
     gtk_widget_set_sensitive( prp->pwRolloutStop, FALSE );
     gtk_widget_set_sensitive( prp->pwRolloutViewStat, TRUE );
 
+#if !USE_GTK2
     gtk_progress_set_format_string( GTK_PROGRESS( prp->pwRolloutProgress ),
                                     _("Finished (%v trials)") );
+#else
+    gsz = g_strdup_printf( _("Finished (%d trials)") , prp->nGamesDone);
+    gtk_progress_bar_set_text( GTK_PROGRESS_BAR( prp->pwRolloutProgress ),
+                                    gsz );
+    g_free( gsz );
+#endif
 
     gtk_signal_disconnect( GTK_OBJECT( prp->pwRolloutDialog ), 
                            prp->nRolloutSignal );
