@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: renderprefs.h,v 1.3 2003/09/01 09:21:02 Superfly_Jon Exp $
+ * $Id: renderprefs.h,v 1.4 2004/03/31 09:51:56 Superfly_Jon Exp $
  */
 
 #ifndef _RENDERPREFS_H_
@@ -27,7 +27,8 @@
 #endif
 
 extern char *aszWoodName[];
-extern renderdata rdAppearance;
+extern renderdata* GetMainAppearance();
+extern void CopyAppearance(renderdata* prd);
 
 extern void RenderPreferencesParam( renderdata *prd, char *szParam,
 				   char *szValue );
