@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: rollout.c,v 1.139 2004/01/30 09:33:50 uid68519 Exp $
+ * $Id: rollout.c,v 1.150 2006/09/21 22:24:06 Superfly_Jon Exp $
  */
 
 #if HAVE_CONFIG_H
@@ -51,7 +51,7 @@
 #include "matchid.h"
 #include "positionid.h"
 #include "rollout.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "format.h"
 
 int log_rollouts = 0;
@@ -225,6 +225,7 @@ static int RolloutDice( int iTurn, int iGame, int cGames,
                             int fInitial,
                             int anDice[ 2 ],
                             const rng rngx,
+                            void *rngctx,
                             const int fRotate ) {
 
   if ( fInitial && !iTurn ) {
@@ -250,7 +251,7 @@ static int RolloutDice( int iTurn, int iGame, int cGames,
 	  int n;
      
       reroll:
-	  if( ( n = RollDice( anDice, rngx ) ) )
+	  if( ( n = RollDice( anDice, rngx, rngctx ) ) )
 	      return n;
 	  
 	  if ( fInitial && ! iTurn && anDice[ 0 ] == anDice[ 1 ] )
@@ -273,7 +274,7 @@ static int RolloutDice( int iTurn, int iGame, int cGames,
 #endif
       return 0;
   } else 
-      return RollDice( anDice, rngx );
+      return RollDice( anDice, rngx, rngctx );
 }
 
 
@@ -375,9 +376,13 @@ BasicCubefulRollout ( int aanBoard[][ 2 ][ 25 ],
   int aanMoves[ 6 ][ 6 ][ 8 ];
   float aaar[ 6 ][ 6 ][ NUM_ROLLOUT_OUTPUTS ];
 
+#if defined(REDUCTION_CODE)
   evalcontext ecCubeless0ply = { FALSE, 0, 0, TRUE, 0.0 };
   evalcontext ecCubeful0ply = { TRUE, 0, 0, TRUE, 0.0 };
-
+#else
+  evalcontext ecCubeless0ply = { FALSE, 0, FALSE, TRUE, 0.0 };
+  evalcontext ecCubeful0ply = { TRUE, 0, FALSE, TRUE, 0.0 };
+#endif
   /* local pointers to the eval contexts to use */
   evalcontext *pecCube[2], *pecChequer[2];
 
@@ -565,7 +570,7 @@ BasicCubefulRollout ( int aanBoard[][ 2 ][ 25 ],
     /* Chequer play */
 
     if( RolloutDice( iTurn, iGame, cGames, prc->fInitial, anDice,
-                         prc->rngRollout, prc->fRotate ) < 0 )
+                         prc->rngRollout, rngctxRollout, prc->fRotate ) < 0 )
       return -1;
 
     if( anDice[ 0 ] < anDice[ 1 ] )
@@ -634,7 +639,7 @@ BasicCubefulRollout ( int aanBoard[][ 2 ][ 25 ],
 
               for ( k = 0; k < NUM_ROLLOUT_OUTPUTS; k++ )
                 arMean[ k ] += ( ( i == j ) ? aaar[ i ][ j ][ k ] :
-                                 ( aaar[ i ][ j ][ k ] * 2.0 ) );
+                                 ( aaar[ i ][ j ][ k ] * 2.0f ) );
 
             }
 
@@ -955,6 +960,22 @@ comp_jsdinfo_order (const void *a, const void *b) {
   return 0;
 }
 
+extern double get_time();
+
+int UpdateTimePassed()
+{
+	static double lasttime = -1;
+	double current = get_time();
+
+	if (fabs(current - lasttime) > 1000)
+	{
+		lasttime = current;
+		return TRUE;
+	}
+	else
+		return FALSE;
+}
+
 extern int
 RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ], 
                 float (* apOutput[])[ NUM_ROLLOUT_OUTPUTS ],
@@ -981,7 +1002,7 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
 	alloca( alternatives * NUM_ROLLOUT_OUTPUTS *  sizeof ( double ) );
   cubeinfo *aciLocal = alloca ( alternatives * sizeof ( cubeinfo ) );
   jsdinfo * ajiJSD = alloca ( alternatives * sizeof ( jsdinfo ));
-  int *nGamesDone = alloca ( alternatives * sizeof ( int ));
+  unsigned int *nGamesDone = alloca ( alternatives * sizeof ( unsigned int ));
   int *fNoMore =  alloca ( alternatives * sizeof ( int ));
 
 #else
@@ -993,19 +1014,20 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
   double aarVariance[ MAX_ROLLOUT_CUBEINFO ][ NUM_ROLLOUT_OUTPUTS ];
   cubeinfo aciLocal[ MAX_ROLLOUT_CUBEINFO ];
   jsdinfo ajiJSD[MAX_ROLLOUT_CUBEINFO];
-  int nGamesDone[MAX_ROLLOUT_CUBEINFO];
+  unsigned int nGamesDone[MAX_ROLLOUT_CUBEINFO];
   int fNoMore[MAX_ROLLOUT_CUBEINFO];
 
 #endif
   
-  int i, j, alt;
+  int j, alt;
+  unsigned int i;
   int anBoardOrig[ 2 ][ 25 ];
   int     err_too_big;
   double    v, s;
   unsigned int nFirstTrial;
   evalsetup *pes;
   rolloutcontext *prc = NULL, rcRolloutSave;
-  int   cGames;
+  unsigned int   cGames;
   int nIsCubeless = 0;
   int nIsCubeful = 0;
   int fOutputMWCSave = fOutputMWC;
@@ -1113,7 +1135,7 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
 	  prc->aecCubeLate[ i ].fCubeful = 
 	  prc->aecChequerLate[ i] .fCubeful = (prc->fCubeful || fCubeRollout);
 
-      if ((nGamesDone[ alt ] = nGames) < nFirstTrial) 
+      if ((nGamesDone[ alt ] = nGames) < (int)nFirstTrial) 
         nFirstTrial = nGames;
       /* restore internal variables from input values */
       for ( j = 0; j < NUM_ROLLOUT_OUTPUTS; ++j) {
@@ -1154,6 +1176,8 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
   /* ============ begin rollout loop ============= */
 
   for( i = nFirstTrial; i < cGames; i++ ) {
+	int updateProgress = fShowProgress && pfProgress && UpdateTimePassed();
+
     active_alternatives = alternatives;
     err_too_big = 1;
 
@@ -1176,7 +1200,7 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
 
       /* ... and the RNG */
       if( prc->rngRollout != RNG_MANUAL )
-        InitRNGSeed( prc->nSeed + ( i << 8 ), prc->rngRollout );
+        InitRNGSeed( prc->nSeed + ( i << 8 ), prc->rngRollout, rngctxRollout );
 
       memcpy ( &aanBoardEval[alt][0][0], apBoard[ alt ], 
                sizeof( anBoardOrig ));
@@ -1221,7 +1245,7 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
         float rMuNew, rDelta;
       
         aarResult[ alt ][ j ] += aar[ alt ][ j ];
-        rMuNew = aarResult[ alt ][ j ] / ( i + 1 );
+        rMuNew = (float)aarResult[ alt ][ j ] / ( i + 1 );
       
         if ( i ) {
           /* for i == 0 aarVariance is not defined */
@@ -1242,10 +1266,10 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
             aarMu[ alt ][ j ] = 1.0f;
         }
       
-        aarSigma[ alt ][ j ] = sqrt( aarVariance[ alt ][ j ] / ( i + 1 ) );
+        aarSigma[ alt ][ j ] = (float)sqrt( aarVariance[ alt ][ j ] / ( i + 1 ) );
       } /* for (j = 0; j < NUM_ROLLOUT_OUTPUTS; j++ ) */
 
-      if( fShowProgress && pfProgress ) {
+      if( updateProgress) {
         (*pfProgress)( aarMu, aarSigma, prc, aciLocal,
                        i, alt, ajiJSD[ alt ].nRank + 1,
 		       ajiJSD[ alt ].rJSD, fNoMore[ alt ], show_jsds,
@@ -1267,7 +1291,7 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
     */
     if (show_jsds) {
       float v, s, denominator;
-      int  reset_to = i;
+      unsigned int  reset_to = i;
 
       for (alt = 0; alt < alternatives; ++alt) {
 
@@ -1326,10 +1350,10 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
         ajiJSD[ alt ].nRank = alt;
         ajiJSD[ alt ].rEquity = v - ajiJSD[ alt ].rEquity;
         
-	denominator = sqrt (s + ajiJSD[ alt ].rJSD * ajiJSD[ alt ].rJSD);
+	denominator = (float)sqrt (s + ajiJSD[ alt ].rJSD * ajiJSD[ alt ].rJSD);
 
-	if (denominator < 1e-8)
-	  denominator = 1e-8;
+	if (denominator < 1e-8f)
+	  denominator = 1e-8f;
 
         ajiJSD[ alt ].rJSD = ajiJSD[ alt ].rEquity / denominator;
         
@@ -1381,8 +1405,8 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
              comp_jsdinfo_order);
 
     }
- 
-    if( fShowProgress && pfProgress ) 
+
+    if( updateProgress) 
       for (alt = 0; alt < alternatives; ++alt) {
         (*pfProgress)( aarMu, aarSigma, prc, aciLocal,
                        i, alt, ajiJSD[ alt ].nRank + 1,
@@ -1452,12 +1476,25 @@ RolloutGeneral( int (* apBoard[])[ 2 ][ 25 ],
       break;
   } /* for( i = nFirstTrial; i < cGames; i++ ) */
 
+	/* Make sure final output is upto date */
+	for (alt = 0; alt < alternatives; ++alt) {
+	 if (nGamesDone[ alt ] > 0)
+	(*pfProgress)( aarMu, aarSigma, prc, aciLocal,
+					MIN(i, cGames - 1), alt, ajiJSD[ alt ].nRank + 1,
+					ajiJSD[ alt ].rJSD, fNoMore[ alt ], show_jsds,
+					pUserData );
+	}
+
   if (log_rollouts && log_name) {
     free (log_name);
     log_name = 0;
   }
 
-  cGames = i;
+  for (alt = 0, cGames = 0; alt < alternatives; ++alt)
+  {
+    if ( nGamesDone[alt] > cGames )
+      cGames = nGamesDone[alt];
+  }
 
   memcpy (&rcRollout, &rcRolloutSave, sizeof (rcRollout));
   fOutputMWC = fOutputMWCSave;
