@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: sgf.c,v 1.89 2003/10/18 12:46:59 Superfly_Jon Exp $
+ * $Id: sgf.c,v 1.97 2004/10/26 19:11:35 oysteijo Exp $
  */
 
 #include "config.h"
@@ -42,6 +42,7 @@
 static char *szFile;
 static int fError;
 
+static int CheckSGFVersion ( const char **sz );
 static void ErrorHandler( char *sz, int fParseError ) {
 
     if( !fError ) {
@@ -143,11 +144,9 @@ static list *LoadCollection( char *sz ) {
 	}
 	    
 	if( ListEmpty( plCollection ) ) {
-	    fError = FALSE; /* we always want to see this one */
 	    ErrorHandler( _("warning: no backgammon games in SGF file"), TRUE );
 	    free( plCollection );
-            PopLocale ();
-	    return NULL;
+	    plCollection = NULL;
 	}
     }
     
@@ -172,15 +171,16 @@ static void CopyName( int i, char *sz, char *szCharset ) {
 
 }
 
-static void SetScore( movegameinfo *pmgi, int fBlack, int n ) {
+static void SetScore( xmovegameinfo *pmgi, int fBlack, int n ) {
 
     if( n >= 0 && ( !pmgi->nMatch || n < pmgi->nMatch ) )
 	pmgi->anScore[ fBlack ] = n;
 }
 
-static void RestoreMI( list *pl, movegameinfo *pmgi ) {
+static void RestoreMI( list *pl, moverecord *pmr ) {
 
     char *pch;
+    xmovegameinfo *pmgi = &pmr->g;
 
     for( pl = pl->plNext; ( pch = pl->p ); pl = pl->plNext )
 	if( !strncmp( pch, "length:", 7 ) ) {
@@ -197,9 +197,9 @@ static void RestoreMI( list *pl, movegameinfo *pmgi ) {
 	    SetScore( pmgi, *pch == 'b', atoi( pch + 3 ) );
 #if USE_TIMECONTROL
 	else if( !strncmp( pch, "wtime:", 6 ) ) {
-	    pmgi->tl[0].tv_sec= atoi( pch + 6 );
+	    pmr->tl[0].tv_sec= atoi( pch + 6 );
 	} else if( !strncmp( pch, "btime:", 6 ) ) {
-	    pmgi->tl[1].tv_sec= atoi( pch + 6 );
+	    pmr->tl[1].tv_sec= atoi( pch + 6 );
 	} else if( !strncmp( pch, "wtimeouts:", 10 ) ) {
 	    pmgi->nTimeouts[0] = atoi( pch + 10 );
 	} else if( !strncmp( pch, "btimeouts:", 10 ) ) {
@@ -357,7 +357,7 @@ static void RestoreText( char *sz, char **ppch, const char *szCharset ) {
 }
 
 static void
-RestoreRules( movegameinfo *pmgi, const char *sz ) {
+RestoreRules( xmovegameinfo *pmgi, const char *sz ) {
 
   char *pch;
   char *pchx;
@@ -399,36 +399,38 @@ RestoreRules( movegameinfo *pmgi, const char *sz ) {
 static void RestoreRootNode( list *pl, char *szCharset ) {
 
     property *pp;
-    movegameinfo *pmgi = malloc( sizeof( *pmgi ) );
+    moverecord *pmr;
     char *pch;
     int i;
+
+    pmr = NewMoveRecord();
     
-    pmgi->mt = MOVE_GAMEINFO;
-    pmgi->sz = NULL;
-    pmgi->i = 0;
-    pmgi->nMatch = 0;
-    pmgi->anScore[ 0 ] = 0;
-    pmgi->anScore[ 1 ] = 0;
-    pmgi->fCrawford = FALSE;
-    pmgi->fCrawfordGame = FALSE;
-    pmgi->fJacoby = FALSE;
-    pmgi->fWinner = -1;
-    pmgi->nPoints = 0;
-    pmgi->fResigned = FALSE;
-    pmgi->nAutoDoubles = 0;
-    pmgi->bgv = VARIATION_STANDARD;
-    pmgi->fCubeUse = TRUE;
+    pmr->mt = MOVE_GAMEINFO;
+
+    pmr->g.i = 0;
+    pmr->g.nMatch = 0;
+    pmr->g.anScore[ 0 ] = 0;
+    pmr->g.anScore[ 1 ] = 0;
+    pmr->g.fCrawford = FALSE;
+    pmr->g.fCrawfordGame = FALSE;
+    pmr->g.fJacoby = FALSE;
+    pmr->g.fWinner = -1;
+    pmr->g.nPoints = 0;
+    pmr->g.fResigned = FALSE;
+    pmr->g.nAutoDoubles = 0;
+    pmr->g.bgv = VARIATION_STANDARD;
+    pmr->g.fCubeUse = TRUE;
 #if USE_TIMECONTROL
-    pmgi->tl[0].tv_sec = pmgi->tl[1].tv_sec = 0;
-    pmgi->tl[0].tv_usec = pmgi->tl[1].tv_usec = 0;
-    pmgi->nTimeouts[0] = pmgi->nTimeouts[1] = 0; 
+    pmr->tl[0].tv_sec = pmr->tl[1].tv_sec = 0;
+    pmr->tl[0].tv_usec = pmr->tl[1].tv_usec = 0;
+    pmr->g.nTimeouts[0] = pmr->g.nTimeouts[1] = 0; 
 #endif
-    IniStatcontext( &pmgi->sc );
+    IniStatcontext( &pmr->g.sc );
     
     for( pl = pl->plNext; ( pp = pl->p ); pl = pl->plNext ) 
 	if( pp->ach[ 0 ] == 'M' && pp->ach[ 1 ] == 'I' )
 	    /* MI - Match info property */
-	    RestoreMI( pp->pl, pmgi );
+	    RestoreMI( pp->pl, pmr );
         else if ( pp->ach[ 0 ] == 'C' && pp->ach[ 1 ] == 'A' )
           /* CA - charset property */
           strcpy ( szCharset, (const char *) pp->pl->plNext->p );
@@ -442,37 +444,37 @@ static void RestoreRootNode( list *pl, char *szCharset ) {
 	    /* RE - Result property */
 	    pch = pp->pl->plNext->p;
 
-	    pmgi->fWinner = -1;
+	    pmr->g.fWinner = -1;
 	    
 	    if( toupper( *pch ) == 'B' )
-		pmgi->fWinner = 1;
+		pmr->g.fWinner = 1;
 	    else if( toupper( *pch ) == 'W' )
-		pmgi->fWinner = 0;
+		pmr->g.fWinner = 0;
 
-	    if( pmgi->fWinner == -1 )
+	    if( pmr->g.fWinner == -1 )
 		continue;
 	    
 	    if( *++pch != '+' )
 		continue;
 
-	    pmgi->nPoints = strtol( pch, &pch, 10 );
-	    if( pmgi->nPoints < 1 )
-		pmgi->nPoints = 1;
+	    pmr->g.nPoints = strtol( pch, &pch, 10 );
+	    if( pmr->g.nPoints < 1 )
+		pmr->g.nPoints = 1;
 	    
-	    pmgi->fResigned = toupper( *pch ) == 'R';
+	    pmr->g.fResigned = toupper( *pch ) == 'R';
 	} else if( pp->ach[ 0 ] == 'R' && pp->ach[ 1 ] == 'U' ) { 
 	    /* RU - Rules property */
-            RestoreRules( pmgi, (const char *) pp->pl->plNext->p );
+            RestoreRules( &pmr->g, (const char *) pp->pl->plNext->p );
 	} else if( pp->ach[ 0 ] == 'C' && pp->ach[ 1 ] == 'V' ) {
 	    /* CV - Cube value (i.e. automatic doubles) */
 	    for( i = 0; ( 1 << i ) <= MAX_CUBE; i++ )
 		if( atoi( pp->pl->plNext->p ) == 1 << i ) {
-		    pmgi->nAutoDoubles = i;
+		    pmr->g.nAutoDoubles = i;
 		    break;
 		}
 	} else if( pp->ach[ 0 ] == 'G' && pp->ach[ 1 ] == 'S' )
 	    /* GS - Game statistics */
-	    RestoreGS( pp->pl, &pmgi->sc );
+	    RestoreGS( pp->pl, &pmr->g.sc );
 	else if( pp->ach[ 0 ] == 'W' && pp->ach[ 1 ] == 'R' )
 	    /* WR - White rank */
 	    RestoreText( pp->pl->plNext->p, &mi.pchRating[ 0 ], szCharset );
@@ -506,7 +508,7 @@ static void RestoreRootNode( list *pl, char *szCharset ) {
 	    /* GC - Game comment */
 	    RestoreText( pp->pl->plNext->p, &mi.pchComment, szCharset );
     
-    AddMoveRecord( pmgi );
+    AddMoveRecord( pmr );
 }
 
 static int Point( char ch, int f ) {
@@ -573,7 +575,11 @@ InitEvalContext ( evalcontext *pec ) {
 
   pec->nPlies = 0;
   pec->fCubeful = FALSE;
+#if defined( REDUCTION_CODE )
   pec->nReduced = 0;
+#else
+  pec->fUsePrune = FALSE;
+#endif
   pec->fDeterministic = FALSE;
   pec->rNoise = 0.0;
 
@@ -583,17 +589,29 @@ InitEvalContext ( evalcontext *pec ) {
 static void
 RestoreEvalContext ( evalcontext *pec, const char *sz ) {
 
-  int nPlies, nReduced, fDeterministic;
+  int nPlies, fUsePrune = 0, fDeterministic;
   char ch;
+  int ver = CheckSGFVersion( &sz );
+  int red = 0;
 
   InitEvalContext ( pec );
+  /* Still need to read reduced */
+  if( ver < 3 ) {
+    sscanf ( sz, "%d%c %d %d %f",
+          &nPlies, &ch, &red, &fDeterministic, &pec->rNoise );
+  }
+  else {
+    sscanf ( sz, "%d%c %d %f %d",
+	     &nPlies, &ch, &fDeterministic, &pec->rNoise, &fUsePrune ); 
+  }
 
-  sscanf ( sz, "%d%c %d %d %f",
-           &nPlies, &ch, &nReduced, &fDeterministic, &pec->rNoise );
-
+#if defined( REDUCTION_CODE )
+    pec->nReduced = red;
+#else
+  pec->fUsePrune = fUsePrune;
+#endif
   pec->nPlies = nPlies;
   pec->fCubeful = ch == 'C';
-  pec->nReduced = nReduced;
   pec->fDeterministic = fDeterministic;
 
 }
@@ -735,14 +753,17 @@ RestoreRolloutInternals ( evalsetup *pes, const char *sz) {
 }
 
 static int 
-CheckExtendedRolloutVersion ( const char *sz ) {
+CheckSGFVersion ( const char **sz ) {
 
   int	n;
-  char  *pch = strstr (sz, "ver");
+  char  *pch = strstr (*sz, "ver");
   if ((pch == 0) || (sscanf (pch, "ver %d", &n) != 1))
     return 0;
-
-  return (n == SGF_ROLLOUT_VER);
+  /* skip over version info */
+  pch += 4;
+  while( *pch != ' ' ) { ++pch; }
+  *sz = pch;
+  return n;
 }
 
 static void
@@ -841,7 +862,7 @@ RestoreExtendedCubeRollout ( const char *sz,
   /* we assume new versions will still begin with Eq 4 floats
      Trials int 
      */
-  is_current = CheckExtendedRolloutVersion (sz);
+  is_current = CheckSGFVersion( &sz );
 
   RestoreRolloutTrials ( &pes->rc.nGamesDone, sz );
   RestoreCubeRolloutOutput ( aarOutput[ 0 ], aarStdDev[ 0 ], sz, "NoDouble" );
@@ -862,7 +883,7 @@ RestoreExtendedRollout ( move *pm, const char *sz) {
   /* we assume new versions will still begin with Score 2 floats
      Trials int 
      */
-  is_current = CheckExtendedRolloutVersion (sz);
+  is_current = CheckSGFVersion( &sz );
   pes->et = EVAL_ROLLOUT;
   RestoreRolloutScore ( pm, sz );
   RestoreRolloutTrials ( &pes->rc.nGamesDone, sz );
@@ -880,14 +901,17 @@ static void RestoreDoubleAnalysis( property *pp,
                                    evalsetup *pes ) {
     
     char *pch = pp->pl->plNext->p, ch;
-    int nPlies, nReduced, fDeterministic;
-    /* FIXME: removing arUnued will break existing SGF files */
+    int nPlies, nReduced, fUsePrune = 0, fDeterministic;
+    /* leftovers from earlier formats */
     float arUnused[ 4 ];
+    int  ver;
     
     switch( *pch ) {
     case 'E':
 	/* EVAL_EVAL */
+        ++pch;
 	pes->et = EVAL_EVAL;
+	ver = CheckSGFVersion( (const char **) &pch );
 	nReduced = 0;
         pes->ec.rNoise = 0.0f;
         fDeterministic = TRUE;
@@ -896,7 +920,8 @@ static void RestoreDoubleAnalysis( property *pp,
         aarOutput[ 0 ][ OUTPUT_CUBEFUL_EQUITY ] = -20000.0;
         aarOutput[ 1 ][ OUTPUT_CUBEFUL_EQUITY ] = -20000.0;
 	
-	sscanf( pch + 1, "%f %f %f %f %d%c %d %d %f"
+	if( ver < 2 ) {
+	  sscanf( pch, "%f %f %f %f %d%c %d %d %f"
                 "%f %f %f %f %f %f %f" 
                 "%f %f %f %f %f %f %f", 
                 &arUnused[ 0 ], &arUnused[ 1 ], 
@@ -913,9 +938,46 @@ static void RestoreDoubleAnalysis( property *pp,
                 &aarOutput[ 1 ][ 2 ], &aarOutput[ 1 ][ 3 ], 
                 &aarOutput[ 1 ][ 4 ], &aarOutput[ 1 ][ 5 ], 
                 &aarOutput[ 1 ][ 6 ] );
-
+	} else if( ver == 2 ) {
+	  sscanf( pch, "%d%c %d %d %f %d"
+                "%f %f %f %f %f %f %f" 
+                "%f %f %f %f %f %f %f", 
+                &nPlies, &ch,
+                &nReduced, 
+                &fDeterministic,
+                &pes->ec.rNoise,
+                &fUsePrune, 
+                &aarOutput[ 0 ][ 0 ], &aarOutput[ 0 ][ 1 ], 
+                &aarOutput[ 0 ][ 2 ], &aarOutput[ 0 ][ 3 ], 
+                &aarOutput[ 0 ][ 4 ], &aarOutput[ 0 ][ 5 ], 
+                &aarOutput[ 0 ][ 6 ],
+                &aarOutput[ 1 ][ 0 ], &aarOutput[ 1 ][ 1 ], 
+                &aarOutput[ 1 ][ 2 ], &aarOutput[ 1 ][ 3 ], 
+                &aarOutput[ 1 ][ 4 ], &aarOutput[ 1 ][ 5 ], 
+                &aarOutput[ 1 ][ 6 ] );
+	} else {
+	  sscanf( pch, "%d%c %d %f %d"
+                "%f %f %f %f %f %f %f" 
+                "%f %f %f %f %f %f %f", 
+                &nPlies, &ch,
+                &fDeterministic,
+                &pes->ec.rNoise,
+                &fUsePrune, 
+                &aarOutput[ 0 ][ 0 ], &aarOutput[ 0 ][ 1 ], 
+                &aarOutput[ 0 ][ 2 ], &aarOutput[ 0 ][ 3 ], 
+                &aarOutput[ 0 ][ 4 ], &aarOutput[ 0 ][ 5 ], 
+                &aarOutput[ 0 ][ 6 ],
+                &aarOutput[ 1 ][ 0 ], &aarOutput[ 1 ][ 1 ], 
+                &aarOutput[ 1 ][ 2 ], &aarOutput[ 1 ][ 3 ], 
+                &aarOutput[ 1 ][ 4 ], &aarOutput[ 1 ][ 5 ], 
+                &aarOutput[ 1 ][ 6 ] );
+	}
 	pes->ec.nPlies = nPlies;
+#if defined( REDUCTION_CODE )
         pes->ec.nReduced = nReduced;
+#else
+        pes->ec.fUsePrune = fUsePrune;
+#endif
         pes->ec.fDeterministic = fDeterministic;
 	pes->ec.fCubeful = ch == 'C';
 
@@ -945,11 +1007,12 @@ static void RestoreMoveAnalysis( property *pp, int fPlayer,
                                  evalsetup *pesChequer,
                                  const matchstate *pms ) {
     list *pl = pp->pl->plNext;
-    char *pch, ch;
+    const char *pch;
+    char ch;
     move *pm;
-    int i, nPlies, fDeterministic, nReduced;
+    int i, nPlies, fDeterministic, nReduced, fUsePrune = 0;
     int anBoardMove[ 2 ][ 25 ];
-    
+    int ver;
     *piMove = atoi( pl->p );
 
     for( pml->cMoves = 0, pl = pl->plNext; pl->p; pl = pl->plNext )
@@ -990,24 +1053,50 @@ static void RestoreMoveAnalysis( property *pp, int fPlayer,
 	switch( ch ) {
 	case 'E':
 	    /* EVAL_EVAL */
+	  ver = CheckSGFVersion( &pch );
 	    pm->esMove.et = EVAL_EVAL;
 	    nReduced = 0;
             fDeterministic = 0;
-
-	    sscanf( pch, " %*c %f %f %f %f %f %f %d%c %d %d %f",
-		    &pm->arEvalMove[ 0 ], &pm->arEvalMove[ 1 ],
-		    &pm->arEvalMove[ 2 ], &pm->arEvalMove[ 3 ],
-		    &pm->arEvalMove[ 4 ], &pm->rScore,
-		    &nPlies, 
-                    &ch, 
-                    &nReduced, 
-                    &fDeterministic, 
-                    &pm->esMove.ec.rNoise );
-
+	    if( ver <  2 ) {
+	      sscanf( pch, " %*c %f %f %f %f %f %f %d%c %d %d %f",
+		      &pm->arEvalMove[ 0 ], &pm->arEvalMove[ 1 ],
+		      &pm->arEvalMove[ 2 ], &pm->arEvalMove[ 3 ],
+		      &pm->arEvalMove[ 4 ], &pm->rScore,
+		      &nPlies, 
+		      &ch, 
+		      &nReduced,
+		      &fDeterministic, 
+		      &pm->esMove.ec.rNoise);
+	    } else if( ver == 2 ) {
+	      sscanf( pch, " %*c %f %f %f %f %f %f %d%c %d %d %f %d",
+		      &pm->arEvalMove[ 0 ], &pm->arEvalMove[ 1 ],
+		      &pm->arEvalMove[ 2 ], &pm->arEvalMove[ 3 ],
+		      &pm->arEvalMove[ 4 ], &pm->rScore,
+		      &nPlies, 
+		      &ch, 
+		      &nReduced,
+		      &fDeterministic, 
+		      &pm->esMove.ec.rNoise,
+		      &fUsePrune);
+	    } else {
+	      sscanf( pch, " %*c %f %f %f %f %f %f %d%c %d %f %d",
+		      &pm->arEvalMove[ 0 ], &pm->arEvalMove[ 1 ],
+		      &pm->arEvalMove[ 2 ], &pm->arEvalMove[ 3 ],
+		      &pm->arEvalMove[ 4 ], &pm->rScore,
+		      &nPlies, 
+		      &ch, 
+		      &fDeterministic, 
+		      &pm->esMove.ec.rNoise,
+		      &fUsePrune);
+	    }
 
 	    pm->esMove.ec.fCubeful = ch == 'C';
 	    pm->esMove.ec.nPlies = nPlies;
+#if defined( REDUCTION_CODE )
 	    pm->esMove.ec.nReduced = nReduced;
+#else
+	    pm->esMove.ec.fUsePrune = fUsePrune;
+#endif
 	    pm->esMove.ec.fDeterministic = fDeterministic;
 	    break;
 
@@ -1084,62 +1173,56 @@ static void RestoreNode( list *pl, char *szCharset ) {
 	    
 	    if( !strcmp( pch, "double" ) ) {
 
-		pmr = calloc( 1, sizeof( pmr->d ) );
+                pmr = NewMoveRecord();
+
 		pmr->mt = MOVE_DOUBLE;
-		pmr->d.sz = NULL;
-		pmr->d.fPlayer = fPlayer;
-		if( ! LinkToDouble( pmr ) ) {
-		  pmr->d.nAnimals = 0;
-		pmr->d.CubeDecPtr = &pmr->d.CubeDec;
-		pmr->d.CubeDecPtr->esDouble.et = EVAL_NONE;
-		}
+		pmr->fPlayer = fPlayer;
+		LinkToDouble( pmr );
+
 	    } else if( !strcmp( pch, "take" ) ) {
-		pmr = calloc( 1, sizeof( pmr->d ) );
+
+                pmr = NewMoveRecord();
+
 		pmr->mt = MOVE_TAKE;
-		pmr->d.sz = NULL;
-		pmr->d.fPlayer = fPlayer;
-		pmr->d.st = SKILL_NONE;
+		pmr->fPlayer = fPlayer;
 		if (!LinkToDouble( pmr ) ) {
 		  free (pmr);
 		  continue;
 		}
 	    } else if( !strcmp( pch, "drop" ) ) {
-		pmr = calloc( 1, sizeof( pmr->d ) );
+
+                pmr = NewMoveRecord();
+
 		pmr->mt = MOVE_DROP;
-		pmr->d.sz = NULL;
-		pmr->d.fPlayer = fPlayer;
-		pmr->d.st = SKILL_NONE;
+		pmr->fPlayer = fPlayer;
 		if (!LinkToDouble( pmr ) ) {
 		  free (pmr);
 		  continue;
 		}
 	    } else {
-		pmr = calloc( 1, sizeof( pmr->n ) );
+
+                pmr = NewMoveRecord();
+
 		pmr->mt = MOVE_NORMAL;
-		pmr->n.sz = NULL;
-		pmr->n.fPlayer = fPlayer;
-		pmr->n.ml.cMoves = 0;
-		pmr->n.ml.amMoves = NULL;
-                pmr->n.esChequer.et = EVAL_NONE;
-		pmr->n.esDouble.et = EVAL_NONE;
+		pmr->fPlayer = fPlayer;
 		
-		pmr->n.anRoll[ 1 ] = 0;
+		pmr->anDice[ 1 ] = 0;
 		
 		for( i = 0; i < 2 && *pch; pch++, i++ )
-		    pmr->n.anRoll[ i ] = *pch - '0';
+		    pmr->anDice[ i ] = *pch - '0';
 
 		for( i = 0; i < 4 && pch[ 0 ] && pch[ 1 ]; pch += 2, i++ ) {
 		    pmr->n.anMove[ i << 1 ] = Point( pch[ 0 ],
-						     pmr->n.fPlayer );
+						     pmr->fPlayer );
 		    pmr->n.anMove[ ( i << 1 ) | 1 ] = Point( pch[ 1 ],
-							     pmr->n.fPlayer );
+							     pmr->fPlayer );
 		}
 
 		if( i < 4 )
 		    pmr->n.anMove[ i << 1 ] = -1;
 			
-		if( pmr->n.anRoll[ 0 ] < 1 || pmr->n.anRoll[ 0 ] > 6 ||
-		    pmr->n.anRoll[ 1 ] < 1 || pmr->n.anRoll[ 1 ] > 6 ) {
+		if( pmr->anDice[ 0 ] < 1 || pmr->anDice[ 0 ] > 6 ||
+		    pmr->anDice[ 1 ] < 1 || pmr->anDice[ 1 ] > 6 ) {
 		    /* illegal roll -- ignore */
 		    free( pmr );
 		    pmr = NULL;
@@ -1171,18 +1254,18 @@ static void RestoreNode( list *pl, char *szCharset ) {
 	} else if( pp->ach[ 0 ] == 'C' && pp->ach[ 1 ] == 'V' ) {
 	    for( i = 1; i <= MAX_CUBE; i <<= 1 )
 		if( atoi( pp->pl->plNext->p ) == i ) {
-		    pmr = calloc( 1, sizeof( pmr->scv ) );
+                    pmr = NewMoveRecord();
+
 		    pmr->mt = MOVE_SETCUBEVAL;
-		    pmr->scv.sz = NULL;
 		    pmr->scv.nCube = i;
 		    AddMoveRecord( pmr );
 		    pmr = NULL;
 		    break;
 		}
 	} else if( pp->ach[ 0 ] == 'C' && pp->ach[ 1 ] == 'P' ) {
-	    pmr = calloc( 1, sizeof( pmr->scp ) );
+            pmr = NewMoveRecord();
+
 	    pmr->mt = MOVE_SETCUBEPOS;
-	    pmr->scp.sz = NULL;
 	    switch( *( (char *) pp->pl->plNext->p ) ) {
 	    case 'c':
 		pmr->scp.fCubeOwner = -1;
@@ -1209,12 +1292,12 @@ static void RestoreNode( list *pl, char *szCharset ) {
 
 	    if( ach[ 0 ] >= '1' && ach[ 0 ] <= '6' && ach[ 1 ] >= '1' &&
 		ach[ 1 ] <= '6' ) {
-		pmr = calloc( 1, sizeof( pmr->sd ) );
+                pmr = NewMoveRecord();
+
 		pmr->mt = MOVE_SETDICE;
-		pmr->sd.sz = NULL;
-		pmr->sd.fPlayer = ms.fMove;
-		pmr->sd.anDice[ 0 ] = ach[ 0 ] - '0';
-		pmr->sd.anDice[ 1 ] = ach[ 1 ] - '0';
+		pmr->fPlayer = ms.fMove;
+		pmr->anDice[ 0 ] = ach[ 0 ] - '0';
+		pmr->anDice[ 1 ] = ach[ 1 ] - '0';
 	    }
 	} else if( pp->ach[ 0 ] == 'D' && pp->ach[ 1 ] == 'A' )
 	    /* double analysis */
@@ -1261,27 +1344,28 @@ static void RestoreNode( list *pl, char *szCharset ) {
 	    LUCK_BAD;
 #if USE_TIMECONTROL
 	else if( pp->ach[ 0 ] == 'W' && pp->ach[ 1 ] == 'L' ) {
-	    pmr->a.tl[0].tv_sec = atoi ( (char *) pp->pl->plNext->p );
+            pmr->tl[0].tv_sec = atoi ( (char *) pp->pl->plNext->p );
 	    fTimeset |= 1;
 	}
 	else if( pp->ach[ 0 ] == 'B' && pp->ach[ 1 ] == 'L' ) {
-	    pmr->a.tl[1].tv_sec = atoi ( (char *) pp->pl->plNext->p );
+	    pmr->tl[1].tv_sec = atoi ( (char *) pp->pl->plNext->p );
 	    fTimeset |= 2;
 	}
 	else if( ( pp->ach[ 0 ] == 'W' || pp->ach[ 0 ] == 'B') && pp->ach[ 1 ] == 'X' ) {
-	    pmr = malloc( sizeof( pmr->t ) );
-	    pmr->t.mt = MOVE_TIME;
-	    pmr->t.sz = 0;
+            pmr = NewMoveRecord();
+
+	    pmr->mt = MOVE_TIME;
+	    pmr->fPlayer = (pp->ach[ 0 ] == 'W');
+
 	    pmr->t.nPoints = atoi ( (char *) pp->pl->plNext->p );
-	    pmr->t.fPlayer = (pp->ach[ 0 ] == 'W');
 	}
 #endif
     }
     
     if( fSetBoard && !pmr ) {
-	pmr = calloc( 1, sizeof( pmr->sb ) );
+        pmr = NewMoveRecord();
+
 	pmr->mt = MOVE_SETBOARD;
-	pmr->sb.sz = NULL;
 	ClosestLegalPosition( ms.anBoard );
 	PositionKey( ms.anBoard, pmr->sb.auchKey );
     }
@@ -1289,14 +1373,14 @@ static void RestoreNode( list *pl, char *szCharset ) {
 #if USE_TIMECONTROL
     /* Compensate for mising timestamps */
     if (pmr && !(fTimeset&1))
-	pmr->a.tl[0]= ((moverecord *) (plLastMove->p))->a.tl[0];
+	pmr->tl[0]= ((moverecord *) (plLastMove->p))->tl[0];
 
     if (pmr && !(fTimeset&2))
-	pmr->a.tl[1]= ((moverecord *) (plLastMove->p))->a.tl[1];
+	pmr->tl[1]= ((moverecord *) (plLastMove->p))->tl[1];
 #endif
 
     if( pmr && ppC )
-	pmr->a.sz = CopyEscapedString( ppC->pl->plNext->p, szCharset );
+	pmr->sz = CopyEscapedString( ppC->pl->plNext->p, szCharset );
 
     if( pmr ) {
       
@@ -1306,18 +1390,18 @@ static void RestoreNode( list *pl, char *szCharset ) {
 	case MOVE_NORMAL:
 	    if( ppDA )
 		RestoreDoubleAnalysis( ppDA, 
-                                       pmr->n.aarOutput,
-                                       pmr->n.aarStdDev,
-                                       &pmr->n.esDouble );
+                                       pmr->CubeDecPtr->aarOutput,
+                                       pmr->CubeDecPtr->aarStdDev,
+                                       &pmr->CubeDecPtr->esDouble );
 	    if( ppA )
-		RestoreMoveAnalysis( ppA, pmr->n.fPlayer, &pmr->n.ml,
-				     &pmr->n.iMove, &pmr->n.esChequer,
+                RestoreMoveAnalysis( ppA, pmr->fPlayer, &pmr->ml,
+				     &pmr->n.iMove, &pmr->esChequer,
                                      &ms );
             /* FIXME: separate st's */
 	    pmr->n.stMove = ast[ 0 ];
-	    pmr->n.stCube = ast[ 1 ];
-	    pmr->n.lt = fPlayer ? lt : LUCK_VERYGOOD - lt;
-	    pmr->n.rLuck = rLuck;
+	    pmr->stCube = ast[ 1 ];
+	    pmr->lt = fPlayer ? lt : LUCK_VERYGOOD - lt;
+	    pmr->rLuck = rLuck;
 	    break;
 	    
 	case MOVE_DOUBLE:
@@ -1325,15 +1409,15 @@ static void RestoreNode( list *pl, char *szCharset ) {
 	case MOVE_DROP:
 	    if( ppDA )
 		RestoreDoubleAnalysis( ppDA, 
-							   pmr->d.CubeDecPtr->aarOutput,
-							   pmr->d.CubeDecPtr->aarStdDev,
-							   &pmr->d.CubeDecPtr->esDouble );
-	    pmr->d.st = ast[ 0 ];
+                                       pmr->CubeDecPtr->aarOutput,
+                                       pmr->CubeDecPtr->aarStdDev,
+                                       &pmr->CubeDecPtr->esDouble );
+	    pmr->stCube = ast[ 0 ];
 	    break;
 	    
 	case MOVE_SETDICE:
-	    pmr->sd.lt = lt;
-	    pmr->sd.rLuck = rLuck;
+	    pmr->lt = lt;
+	    pmr->rLuck = rLuck;
 	    break;
 	    
 	default:
@@ -1401,7 +1485,7 @@ static void RestoreGame( list *pl, char *szCharset ) {
     AddGame( pmr );
     
 #if USE_TIMECONTROL
-    if( pmr->g.tl[0].tv_sec && pmr->g.tl[1].tv_sec) 
+    if( pmr->tl[0].tv_sec && pmr->tl[1].tv_sec) 
 	ms.gc.pc[0].tc.timing = ms.gc.pc[1].tc.timing =  TC_UNKNOWN;
 #endif
     if( pmr->g.fResigned ) {
@@ -1409,11 +1493,11 @@ static void RestoreGame( list *pl, char *szCharset ) {
          inverted when shown. /jth 2003-10-12 
          ms.fTurn = ms.fMove = -1; */
 	
-	pmrResign = calloc( 1, sizeof( pmrResign ->r ) );
+        pmrResign = NewMoveRecord();
+
 	pmrResign->mt = MOVE_RESIGN;
-	pmrResign->r.sz = NULL;
-        pmrResign->r.esResign.et = EVAL_NONE;
-	pmrResign->r.fPlayer = !pmr->g.fWinner;
+
+	pmrResign->fPlayer = !pmr->g.fWinner;
 	pmrResign->r.nResigned = pmr->g.nPoints / ms.nCube;
 
 	if( pmrResign->r.nResigned < 1 )
@@ -1453,7 +1537,10 @@ extern void CommandLoadGame( char *sz ) {
 
 #if USE_GTK
 	if( fX )
-	    GTKFreeze();
+	{	/* Clear record to avoid ugly updates */
+		GTKClearMoveRecord();
+		GTKFreeze();
+	}
 #endif
 	
 	FreeMatch();
@@ -1510,7 +1597,10 @@ CommandLoadPosition( char *sz ) {
 
 #if USE_GTK
 	if( fX )
-	    GTKFreeze();
+	{	/* Clear record to avoid ugly updates */
+		GTKClearMoveRecord();
+		GTKFreeze();
+	}
 #endif
 	
 	FreeMatch();
@@ -1565,7 +1655,10 @@ extern void CommandLoadMatch( char *sz ) {
 
 #if USE_GTK
 	if( fX )
+	{	/* Clear record to avoid ugly updates */
+		GTKClearMoveRecord();
 		GTKFreeze();
+	}
 #endif
 	
 	FreeMatch();
@@ -1640,12 +1733,26 @@ static void WriteEscapedString( FILE *pf, char *pch, int fEscapeColons ) {
 static void
 WriteEvalContext ( FILE *pf, const evalcontext *pec ) {
 
-  fprintf( pf, "%d%s %d %d %.4f",
-           pec->nPlies,
-           pec->fCubeful ? "C" : "",
-           pec->nReduced, 
-           pec->fDeterministic, 
-           pec->rNoise );
+#if defined( REDUCTION_CODE )
+  fprintf( pf, "ver %d %d%s %d %d %.4f %d",
+	   SGF_FORMAT_VER,
+	   pec->nPlies,
+	   pec->fCubeful ? "C" : "",
+	   pec->nReduced,
+	   pec->fDeterministic, 
+	   pec->rNoise,
+/*         , pec->fUsePrune */
+	   0
+	   );
+#else
+  fprintf( pf, "ver %d %d%s %d %.4f %d",
+	   SGF_FORMAT_VER,
+	   pec->nPlies,
+	   pec->fCubeful ? "C" : "",
+	   pec->fDeterministic, 
+	   pec->rNoise,
+	   pec->fUsePrune);
+#endif
 
 }
 
@@ -1667,6 +1774,8 @@ WriteRolloutContext ( FILE *pf, const rolloutcontext *prc ) {
   
   int i;
 
+  printf( "rng %d\n", prc->rngRollout );
+
   fprintf ( pf, "RC %d %d %d %d %d %d %d %d %d %d \"%s\" %d ",
             prc->fCubeful,
             prc->fVarRedn,
@@ -1730,10 +1839,10 @@ static void WriteRolloutAnalysis( FILE *pf, int fIsMove,
      the version is defined in eval.h
      */
   if (fIsMove) {
-    fprintf ( pf, "X ver %d Score %.10g %.10g ", SGF_ROLLOUT_VER,
+    fprintf ( pf, "X ver %d Score %.10g %.10g ", SGF_FORMAT_VER,
 	      rScore, rScore2);
   } else {
-    fprintf ( pf, "X ver %d Eq ", SGF_ROLLOUT_VER);
+    fprintf ( pf, "X ver %d Eq ", SGF_FORMAT_VER);
   }
 
   fprintf ( pf, "Trials %d ", pes->rc.nGamesDone);
@@ -1773,23 +1882,46 @@ static void WriteDoubleAnalysis( FILE *pf,
 
   switch( pes->et ) {
   case EVAL_EVAL:
-    fprintf( pf, "E %.4f %.4f %.4f %.4f %d%s %d %d %.4f "
-             "%.4f %.4f %.4f %.4f %.4f %.4f %.4f "
-             "%.4f %.4f %.4f %.4f %.4f %.4f %.4f", 
-             0.0f, 0.0f, 0.0f, 0.0f,
-             pes->ec.nPlies,
-             pes->ec.fCubeful ? "C" : "",
-             pes->ec.nReduced,
-             pes->ec.fDeterministic,
-             pes->ec.rNoise,
-             aarOutput[ 0 ][ 0 ], aarOutput[ 0 ][ 1 ], 
-             aarOutput[ 0 ][ 2 ], aarOutput[ 0 ][ 3 ], 
-             aarOutput[ 0 ][ 4 ], aarOutput[ 0 ][ 5 ], 
-             aarOutput[ 0 ][ 6 ],
-             aarOutput[ 1 ][ 0 ], aarOutput[ 1 ][ 1 ], 
-             aarOutput[ 1 ][ 2 ], aarOutput[ 1 ][ 3 ], 
-             aarOutput[ 1 ][ 4 ], aarOutput[ 1 ][ 5 ], 
-             aarOutput[ 1 ][ 6 ] );
+#if defined( REDUCTION_CODE )
+    fprintf( pf, "E ver %d %d%s %d %d %.4f %d "
+	     "%.4f %.4f %.4f %.4f %.4f %.4f %.4f "
+	     "%.4f %.4f %.4f %.4f %.4f %.4f %.4f", 
+	     SGF_FORMAT_VER,
+	     pes->ec.nPlies,
+	     pes->ec.fCubeful ? "C" : "",
+	     pes->ec.nReduced,
+	     pes->ec.fDeterministic,
+	     pes->ec.rNoise,
+/*	     pes->ec.fUsePrune, */
+	     0,
+	     aarOutput[ 0 ][ 0 ], aarOutput[ 0 ][ 1 ], 
+	     aarOutput[ 0 ][ 2 ], aarOutput[ 0 ][ 3 ], 
+	     aarOutput[ 0 ][ 4 ], aarOutput[ 0 ][ 5 ], 
+	     aarOutput[ 0 ][ 6 ],
+	     aarOutput[ 1 ][ 0 ], aarOutput[ 1 ][ 1 ], 
+	     aarOutput[ 1 ][ 2 ], aarOutput[ 1 ][ 3 ], 
+	     aarOutput[ 1 ][ 4 ], aarOutput[ 1 ][ 5 ], 
+	     aarOutput[ 1 ][ 6 ] );
+#else
+    fprintf( pf, "E ver %d %d%s %d %.4f %d "
+	     "%.4f %.4f %.4f %.4f %.4f %.4f %.4f "
+	     "%.4f %.4f %.4f %.4f %.4f %.4f %.4f", 
+	     SGF_FORMAT_VER,
+	     pes->ec.nPlies,
+	     pes->ec.fCubeful ? "C" : "",
+	     pes->ec.fDeterministic,
+	     pes->ec.rNoise,
+	     pes->ec.fUsePrune,
+	     aarOutput[ 0 ][ 0 ], aarOutput[ 0 ][ 1 ], 
+	     aarOutput[ 0 ][ 2 ], aarOutput[ 0 ][ 3 ], 
+	     aarOutput[ 0 ][ 4 ], aarOutput[ 0 ][ 5 ], 
+	     aarOutput[ 0 ][ 6 ],
+	     aarOutput[ 1 ][ 0 ], aarOutput[ 1 ][ 1 ], 
+	     aarOutput[ 1 ][ 2 ], aarOutput[ 1 ][ 3 ], 
+	     aarOutput[ 1 ][ 4 ], aarOutput[ 1 ][ 5 ], 
+	     aarOutput[ 1 ][ 6 ] );
+#endif
+
     break;
     
   case EVAL_ROLLOUT:
@@ -1844,7 +1976,7 @@ static void WriteMoveAnalysis( FILE *pf, int fPlayer, movelist *pml,
 	    
 	case EVAL_EVAL:
 	    fprintf( pf, "E %.4f %.4f %.4f %.4f %.4f %.4f %d%s "
-                     "%d %d %.4f",
+                     "%d %d %.4f %d",
 		     pml->amMoves[ i ].arEvalMove[ 0 ],
 		     pml->amMoves[ i ].arEvalMove[ 1 ],
 		     pml->amMoves[ i ].arEvalMove[ 2 ],
@@ -1853,9 +1985,19 @@ static void WriteMoveAnalysis( FILE *pf, int fPlayer, movelist *pml,
 		     pml->amMoves[ i ].rScore,
 		     pml->amMoves[ i ].esMove.ec.nPlies,
 		     pml->amMoves[ i ].esMove.ec.fCubeful ? "C" : "",
-                     pml->amMoves[ i ].esMove.ec.nReduced, 
+#if defined( REDUCTION_CODE )
+                     pml->amMoves[ i ].esMove.ec.nReduced,
+#else
+		     0,
+#endif
                      pml->amMoves[ i ].esMove.ec.fDeterministic, 
-                     pml->amMoves[ i ].esMove.ec.rNoise );
+                     pml->amMoves[ i ].esMove.ec.rNoise ,
+#if !defined( REDUCTION_CODE )
+		     pml->amMoves[ i ].esMove.ec.fUsePrune
+#else
+	             0 
+#endif
+		     );
 	    break;
 	    
 	case EVAL_ROLLOUT:
@@ -2088,12 +2230,16 @@ static void SaveGame( FILE *pf, list *plGame ) {
     /* If not, money games will be loaded without score and game number */
     /* if( pmr->g.nMatch ) */
 #if USE_TIMECONTROL
-    fprintf( pf, "MI[length:%d][game:%d][ws:%d][bs:%d][wtime:%d][btime:%d][wtimeouts:%d][btimeouts:%d]", pmr->g.nMatch,
-             pmr->g.i, pmr->g.anScore[ 0 ], pmr->g.anScore[ 1 ],
-	     (int)pmr->g.tl[0].tv_sec, (int)pmr->g.tl[1].tv_sec, pmr->g.nTimeouts[0], pmr->g.nTimeouts[1]);
+    fprintf( pf, "MI[length:%d][game:%d][ws:%d][bs:%d]"
+             "[wtime:%d][btime:%d][wtimeouts:%d][btimeouts:%d]", 
+             pmr->g.nMatch, pmr->g.i, 
+             pmr->g.anScore[ 0 ], pmr->g.anScore[ 1 ],
+	     (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec, 
+             pmr->g.nTimeouts[0], pmr->g.nTimeouts[1]);
 #else
-    fprintf( pf, "MI[length:%d][game:%d][ws:%d][bs:%d]", pmr->g.nMatch,
-             pmr->g.i, pmr->g.anScore[ 0 ], pmr->g.anScore[ 1 ] );
+    fprintf( pf, "MI[length:%d][game:%d][ws:%d][bs:%d]", 
+             pmr->g.nMatch, pmr->g.i, 
+             pmr->g.anScore[ 0 ], pmr->g.anScore[ 1 ] );
 #endif
     
     /* Names */
@@ -2156,76 +2302,80 @@ static void SaveGame( FILE *pf, list *plGame ) {
 	pmr = pl->p;
 	switch( pmr->mt ) {
 	case MOVE_NORMAL:
-	    fprintf( pf, "\n;%c[%d%d", pmr->n.fPlayer ? 'B' : 'W',
-		     pmr->n.anRoll[ 0 ], pmr->n.anRoll[ 1 ] );
-	    WriteMove( pf, pmr->n.fPlayer, pmr->n.anMove );
+	    fprintf( pf, "\n;%c[%d%d", pmr->fPlayer ? 'B' : 'W',
+		     pmr->anDice[ 0 ], pmr->anDice[ 1 ] );
+	    WriteMove( pf, pmr->fPlayer, pmr->n.anMove );
 	    putc( ']', pf );
 
 #if USE_TIMECONTROL
-	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->n.tl[0].tv_sec, (int)pmr->n.tl[1].tv_sec);
+	    fprintf( pf, "WL[%d]BL[%d]", 
+                     (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec);
 #endif
-	    if( pmr->n.esDouble.et != EVAL_NONE )
+	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE )
 		WriteDoubleAnalysis( pf, 
-                                     pmr->n.aarOutput, pmr->n.aarStdDev,
-				     &pmr->n.esDouble );
+                                     pmr->CubeDecPtr->aarOutput, 
+                                     pmr->CubeDecPtr->aarStdDev,
+				     &pmr->CubeDecPtr->esDouble );
 
-	    if( pmr->n.ml.cMoves )
-		WriteMoveAnalysis( pf, pmr->n.fPlayer, &pmr->n.ml,
+	    if( pmr->ml.cMoves )
+		WriteMoveAnalysis( pf, pmr->fPlayer, &pmr->ml,
 				   pmr->n.iMove );
 
-	    WriteLuck( pf, pmr->n.fPlayer, pmr->n.rLuck, pmr->n.lt );
+	    WriteLuck( pf, pmr->fPlayer, pmr->rLuck, pmr->lt );
             /* FIXME: separate skill for cube and move */
 	    WriteSkill( pf, pmr->n.stMove );
-	    WriteSkillCube( pf, pmr->n.stCube );
+	    WriteSkillCube( pf, pmr->stCube );
 	    
 	    break;
 	    
 	case MOVE_DOUBLE:
-	    fprintf( pf, "\n;%c[double]", pmr->d.fPlayer ? 'B' : 'W' );
+	    fprintf( pf, "\n;%c[double]", pmr->fPlayer ? 'B' : 'W' );
 
 #if USE_TIMECONTROL
-	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->d.tl[0].tv_sec, (int)pmr->d.tl[1].tv_sec);
+	    fprintf( pf, "WL[%d]BL[%d]", 
+                     (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec);
 #endif
-	    if( pmr->d.CubeDecPtr->esDouble.et != EVAL_NONE )
+	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE )
 		WriteDoubleAnalysis( pf, 
-				     pmr->d.CubeDecPtr->aarOutput, 
-				     pmr->d.CubeDecPtr->aarStdDev,
-				     &pmr->d.CubeDecPtr->esDouble );
+				     pmr->CubeDecPtr->aarOutput, 
+				     pmr->CubeDecPtr->aarStdDev,
+				     &pmr->CubeDecPtr->esDouble );
 	    
-	    WriteSkill( pf, pmr->d.st );
+	    WriteSkill( pf, pmr->stCube );
 	    
 	    break;
 	    
 	case MOVE_TAKE:
-	    fprintf( pf, "\n;%c[take]", pmr->d.fPlayer ? 'B' : 'W' );
+	    fprintf( pf, "\n;%c[take]", pmr->fPlayer ? 'B' : 'W' );
 
 #if USE_TIMECONTROL
-	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->d.tl[0].tv_sec, (int)pmr->d.tl[1].tv_sec);
+	    fprintf( pf, "WL[%d]BL[%d]", 
+                     (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec);
 #endif
-	    if( pmr->d.CubeDecPtr->esDouble.et != EVAL_NONE )
+	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE )
 		WriteDoubleAnalysis( pf, 
-				     pmr->d.CubeDecPtr->aarOutput, 
-				     pmr->d.CubeDecPtr->aarStdDev,
-				     &pmr->d.CubeDecPtr->esDouble );
+				     pmr->CubeDecPtr->aarOutput, 
+				     pmr->CubeDecPtr->aarStdDev,
+				     &pmr->CubeDecPtr->esDouble );
 	    
-	    WriteSkill( pf, pmr->d.st );
+	    WriteSkill( pf, pmr->stCube );
 	    
 	    break;
 	    
 	case MOVE_DROP:
-	    fprintf( pf, "\n;%c[drop]", pmr->d.fPlayer ? 'B' : 'W' );
+	    fprintf( pf, "\n;%c[drop]", pmr->fPlayer ? 'B' : 'W' );
 
 #if USE_TIMECONTROL
-	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->d.tl[0].tv_sec, (int)pmr->d.tl[1].tv_sec);
+	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec);
 #endif
-	    if( pmr->d.CubeDecPtr->esDouble.et != EVAL_NONE )
+	    if( pmr->CubeDecPtr->esDouble.et != EVAL_NONE )
 		WriteDoubleAnalysis( pf, 
-				     pmr->d.CubeDecPtr->aarOutput,
-				     pmr->d.CubeDecPtr->aarStdDev,
-				     &pmr->d.CubeDecPtr->esDouble );
+				     pmr->CubeDecPtr->aarOutput,
+				     pmr->CubeDecPtr->aarStdDev,
+				     &pmr->CubeDecPtr->esDouble );
 
 	    
-	    WriteSkill( pf, pmr->d.st );
+	    WriteSkill( pf, pmr->stCube );
 	    
 	    break;
 	    	    
@@ -2260,10 +2410,10 @@ static void SaveGame( FILE *pf, list *plGame ) {
 	    break;
 	    
 	case MOVE_SETDICE:
-	    fprintf( pf, "\n;PL[%c]DI[%d%d]", pmr->sd.fPlayer ? 'B' : 'W',
-		     pmr->sd.anDice[ 0 ], pmr->sd.anDice[ 1 ] );
+	    fprintf( pf, "\n;PL[%c]DI[%d%d]", pmr->fPlayer ? 'B' : 'W',
+		     pmr->anDice[ 0 ], pmr->anDice[ 1 ] );
 
-	    WriteLuck( pf, pmr->sd.fPlayer, pmr->sd.rLuck, pmr->sd.lt );
+	    WriteLuck( pf, pmr->fPlayer, pmr->rLuck, pmr->lt );
 	    
 	    break;
 	    
@@ -2277,9 +2427,10 @@ static void SaveGame( FILE *pf, list *plGame ) {
 	    
 #if USE_TIMECONTROL
 	case MOVE_TIME:
-	    fprintf( pf, "\n;%cX[%d]",  ! pmr->sd.fPlayer ? 'B' : 'W',
+	    fprintf( pf, "\n;%cX[%d]",  ! pmr->fPlayer ? 'B' : 'W',
 		pmr->t.nPoints);
-	    fprintf( pf, "WL[%d]BL[%d]", (int)pmr->t.tl[0].tv_sec, (int)pmr->t.tl[1].tv_sec);
+	    fprintf( pf, "WL[%d]BL[%d]", 
+                     (int)pmr->tl[0].tv_sec, (int)pmr->tl[1].tv_sec);
 	    break;
 #endif
 
@@ -2287,9 +2438,9 @@ static void SaveGame( FILE *pf, list *plGame ) {
 	    assert( FALSE );
 	}
 
-	if( pmr->a.sz ) {
+	if( pmr->sz ) {
 	    fputs( "C[", pf );
-	    WriteEscapedString( pf, pmr->a.sz, FALSE );
+	    WriteEscapedString( pf, pmr->sz, FALSE );
 	    putc( ']', pf );
 	}
     }
@@ -2382,11 +2533,11 @@ extern void CommandSavePosition( char *sz ) {
 
     FILE *pf;
     list l;
-    movegameinfo *pmgi;
-    movesetboard *pmsb;
-    movesetdice *pmsd;
-    movesetcubeval *pmscv;
-    movesetcubepos *pmscp;
+    moverecord *pmgi;
+    moverecord *pmsb;
+    moverecord *pmsd;
+    moverecord *pmscv;
+    moverecord *pmscp;
     
     sz = NextToken( &sz );
     
@@ -2412,53 +2563,65 @@ extern void CommandSavePosition( char *sz ) {
     }
 
     ListCreate( &l );
+
+    /* gameinfo record */
     
-    pmgi = calloc( 1, sizeof( movegameinfo ) );
+    pmgi = NewMoveRecord();
+
     pmgi->mt = MOVE_GAMEINFO;
     pmgi->sz = NULL;
-    pmgi->i = 0;
-    pmgi->nMatch = ms.nMatchTo;
-    pmgi->anScore[ 0 ] = ms.anScore[ 0 ];
-    pmgi->anScore[ 1 ] = ms.anScore[ 1 ];
-    pmgi->fCrawford = fAutoCrawford && ms.nMatchTo > 1;
-    pmgi->fCrawfordGame = ms.fCrawford;
-    pmgi->fJacoby = ms.fJacoby && !ms.nMatchTo;
-    pmgi->fWinner = -1;
-    pmgi->nPoints = 0;
-    pmgi->fResigned = FALSE;
-    pmgi->nAutoDoubles = 0;
-    pmgi->bgv = ms.bgv;
-    pmgi->fCubeUse = ms.fCubeUse;
-    IniStatcontext( &pmgi->sc );
+    pmgi->g.i = 0;
+    pmgi->g.nMatch = ms.nMatchTo;
+    pmgi->g.anScore[ 0 ] = ms.anScore[ 0 ];
+    pmgi->g.anScore[ 1 ] = ms.anScore[ 1 ];
+    pmgi->g.fCrawford = fAutoCrawford && ms.nMatchTo > 1;
+    pmgi->g.fCrawfordGame = ms.fCrawford;
+    pmgi->g.fJacoby = ms.fJacoby && !ms.nMatchTo;
+    pmgi->g.fWinner = -1;
+    pmgi->g.nPoints = 0;
+    pmgi->g.fResigned = FALSE;
+    pmgi->g.nAutoDoubles = 0;
+    pmgi->g.bgv = ms.bgv;
+    pmgi->g.fCubeUse = ms.fCubeUse;
+    IniStatcontext( &pmgi->g.sc );
     ListInsert( &l, pmgi );
 
-    pmsb = calloc( 1, sizeof( movesetboard ) );
+    /* setboard record */
+
+    pmsb = NewMoveRecord();
+
     pmsb->mt = MOVE_SETBOARD;
-    pmsb->sz = NULL;
     if( ms.fMove )
 	SwapSides( ms.anBoard );
-    PositionKey( ms.anBoard, pmsb->auchKey );
+    PositionKey( ms.anBoard, pmsb->sb.auchKey );
     if( ms.fMove )
 	SwapSides( ms.anBoard );
     ListInsert( &l, pmsb );
 
-    pmscv = calloc( 1, sizeof( movesetcubeval ) );
+    /* set cube value */
+
+    pmscv = NewMoveRecord();
+
     pmscv->mt = MOVE_SETCUBEVAL;
-    pmscv->sz = NULL;
-    pmscv->nCube = ms.nCube;
+    pmscv->scv.nCube = ms.nCube;
     ListInsert( &l, pmscv );
+
+    /* cube position */
     
-    pmscp = calloc( 1, sizeof( movesetcubepos ) );
+    pmscp = NewMoveRecord();
+
     pmscp->mt = MOVE_SETCUBEPOS;
-    pmscp->sz = NULL;
-    pmscp->fCubeOwner = ms.fCubeOwner;
+    pmscp->scp.fCubeOwner = ms.fCubeOwner;
     ListInsert( &l, pmscp );
+
+    /* set dice */
     
     /* FIXME if the dice are not rolled, this should be done with a PL
        property (which is SaveGame()'s job) */
-    pmsd = calloc( 1, sizeof( movesetdice ) );
+    
+    pmsd = NewMoveRecord();
+
     pmsd->mt = MOVE_SETDICE;
-    pmsd->sz = NULL;
     pmsd->fPlayer = ms.fMove;
     pmsd->anDice[ 0 ] = ms.anDice[ 0 ];
     pmsd->anDice[ 1 ] = ms.anDice[ 1 ];
