@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: show.c,v 1.180 2004/03/25 20:10:50 thyssen Exp $
+ * $Id: show.c,v 1.208 2006/09/21 22:24:06 Superfly_Jon Exp $
  */
 
 #include "config.h"
@@ -41,7 +41,7 @@
 #include "dice.h"
 #include "matchequity.h"
 #include "matchid.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "sound.h"
 #include "onechequer.h"
 #include "osr.h"
@@ -60,11 +60,10 @@
 #include "gtkrolls.h"
 #include "gtktempmap.h"
 #include "gtkbearoff.h"
-#elif USE_EXT
-#include "xgame.h"
 #else
 #include <glib.h>
 #endif
+#include "openurl.h"
 
 extern char *aszCopying[], *aszWarranty[]; /* from copying.c */
 
@@ -112,11 +111,22 @@ ShowMoveFilters ( const movefilter aamf[ MAX_FILTER_PLIES ][ MAX_FILTER_PLIES ]
 static void 
 ShowEvaluation( const evalcontext *pec ) {
   
+#if defined( REDUCTION_CODE )
   outputf( _("        %d-ply evaluation.\n"
              "        %d%% speed.\n"
              "        %s evaluations.\n"),
+#else
+  outputf( _("        %d-ply evaluation.\n"
+             "        %s"
+             "        %s evaluations.\n"),
+#endif
            pec->nPlies, 
+#if defined( REDUCTION_CODE )
            (pec->nReduced) ? 100 / pec->nReduced : 100,
+#else
+           (pec->fUsePrune) ? _("Using pruning neural nets.") :
+	   _("Not using pruning neural nets."),
+#endif
            pec->fCubeful ? _("Cubeful") : _("Cubeless") );
 
   if( pec->rNoise )
@@ -356,7 +366,7 @@ static void ShowPaged( char **ppch ) {
 	if( !( ioctl( STDIN_FILENO, TIOCGWINSZ, &ws ) ) )
 	    nRows = ws.ws_row;
 #endif
-	if( !nRows && ( pchLines = getenv( "LINES" ) ) )
+	if( !nRows && ((pchLines = getenv( "LINES" )) != NULL) )
 	    nRows = atoi( pchLines );
 
 	/* FIXME we could try termcap-style tgetnum( "li" ) here, but it
@@ -491,14 +501,10 @@ extern void CommandShowBoard( char *sz ) {
     if( ParsePosition( an, &sz, NULL ) < 0 )
 	return;
 
-#if USE_GUI
-    if( fX )
 #if USE_GTK
+    if( fX )
       game_set( BOARD( pwBoard ), an, TRUE, "", "", 0, 0, 0, -1, -1, FALSE,
                 anChequers[ ms.bgv ] );
-#else
-        GameSet( &ewnd, an, TRUE, "", "", 0, 0, 0, -1, -1 );    
-#endif
     else
 #endif
         outputl( DrawBoard( szOut, an, TRUE, ap, 
@@ -526,17 +532,13 @@ void CommandShowFullBoard( char *sz ) {
     if( ParsePosition( an, &sz, NULL ) < 0 )
 	return;
 
-#if USE_GUI
-    if( fX )
 #if USE_GTK
+    if( fX )
 	game_set( BOARD( pwBoard ), an, ms.fTurn, 
                   ap[ 1 ].szName, ap[ 0 ].szName , ms.nMatchTo, 
                   ms.anScore[ 1 ], ms.anScore[ 0 ], 
                   ms.anDice[ 0 ], ms.anDice[ 1 ], FALSE,
                   anChequers[ ms.bgv ] );
-#else
-        GameSet( &ewnd, an, TRUE, "", "", 0, 0, 0, -1, -1 );    
-#endif
     else
 #endif
         outputl( DrawBoard( szOut, an, TRUE, apch, 
@@ -546,7 +548,7 @@ void CommandShowFullBoard( char *sz ) {
 
 
 extern void CommandShowDelay( char *sz ) {
-#if USE_GUI
+#if USE_GTK
     if( nDelay )
 	outputf( _("The delay is set to %d ms.\n"),nDelay);
     else
@@ -651,7 +653,7 @@ extern void CommandShowCopying( char *sz ) {
 
 #if USE_GTK
     if( fX )
-	ShowList( aszCopying, _("Copying") );
+	ShowList( aszCopying, _("Copying"), NULL );
     else
 #endif
 	ShowPaged( aszCopying );
@@ -784,7 +786,7 @@ extern void CommandShowMatchInfo( char *sz ) {
 	     _("unknown rating") );
 
     if( mi.nYear )
-	outputf( ", %04d-%02d-%02d\n", mi.nYear, mi.nMonth, mi.nDay );
+	outputf( _(", %04d-%02d-%02d\n"), mi.nYear, mi.nMonth, mi.nDay );
     else
 	outputc( '\n' );
 
@@ -826,6 +828,14 @@ extern void CommandShowPipCount( char *sz ) {
     
     outputf( _("The pip counts are: %s %d, %s %d.\n"), ap[ ms.fMove ].szName,
 	    anPips[ 1 ], ap[ !ms.fMove ].szName, anPips[ 0 ] );
+
+#if USE_GTK
+	if (fX && fFullScreen)
+	{	/* Display in dialog box in full screen mode (urgh) */
+		output(" ");
+		outputx();
+	}
+#endif
 }
 
 extern void CommandShowPlayer( char *sz ) {
@@ -921,7 +931,8 @@ extern void CommandShowScore( char *sz ) {
 
 extern void CommandShowSeed( char *sz ) {
 
-    PrintRNGSeed( rngCurrent );
+    PrintRNGSeed( rngCurrent, rngctxCurrent );
+    PrintRNGCounter( rngCurrent, rngctxCurrent );
 }
 
 extern void CommandShowTurn( char *sz ) {
@@ -944,14 +955,6 @@ extern void CommandShowTurn( char *sz ) {
 		gettext ( aszGameResult[ ms.fResigned - 1 ] ) );
 }
 
-extern void CommandShowBuildInfo( char *sz )
-{
-#if USE_GTK
-	if( fX )
-		GTKShowBuildInfo();
-#endif
-}
-
 static void
 ShowAuthors( const credEntry ace[], const char *title ) {
 
@@ -971,12 +974,119 @@ ShowAuthors( const credEntry ace[], const char *title ) {
 
 }
 
+extern void CommandShowBuildInfo( char *sz )
+{
+    char *pch;
+
+#if USE_GTK
+	if( fX )
+		GTKShowBuildInfo(pwMain, NULL);
+#endif
+
+    while((pch = GetBuildInfoString()) != 0)
+		outputl( gettext(pch) );
+
+    outputc( '\n' );
+}
+
+extern void CommandShowScoreSheet( char *sz )
+{
+	int i, width1, width2;
+	char *data[2];
+	list *pl;
+
+	if( ms.gs == GAME_NONE ) {
+		outputl( _("No game in progress (type `new game' to start one).") );
+		return;
+	}
+
+#if USE_GTK
+	if( fX )
+	{
+		GTKShowScoreSheet();
+		return;
+	}
+#endif
+
+	output(_("Score Sheet - "));
+	if ( ms.nMatchTo > 0 )
+		outputf(ms.nMatchTo == 1 ? 
+	         _("Match to %d point") :
+	         _("Match to %d points"),
+                 ms.nMatchTo);
+	else
+		output(_("Money Session"));
+
+	output("\n\n");
+
+	width1 = strlen(ap[0].szName);
+	width2 = strlen(ap[1].szName);
+
+	outputf("%s | %s\n", ap[0].szName, ap[1].szName);
+	for (i = 0; i < width1 + width2 + 3; i++)
+		outputc('-');
+	output("\n");
+
+	data[0] = malloc(50);
+	data[1] = malloc(50);
+
+	for (pl = lMatch.plNext; pl->p; pl = pl->plNext )
+	{
+		int score[2];
+		list *plGame = pl->plNext->p;
+
+		if (plGame)
+		{
+			moverecord *pmr = plGame->plNext->p;
+			score[0] = pmr->g.anScore[0];
+			score[1] = pmr->g.anScore[1];
+		}
+		else
+		{
+			moverecord *pmr;
+			list *plGame = pl->p;
+			if (!plGame)
+			{
+				continue;
+			}
+			else
+			{
+				pmr = plGame->plNext->p;
+				score[0] = pmr->g.anScore[0];
+				score[1] = pmr->g.anScore[1];
+				if (pmr->g.fWinner == -1)
+				{
+					if (pl == lMatch.plNext)
+					{	/* First game */
+						score[0] = score[1] = 0;
+					}
+					else
+						continue;
+				}
+				else
+					score[pmr->g.fWinner] += pmr->g.nPoints;
+			}
+		}
+		sprintf(data[0], "%d", score[0]);
+		sprintf(data[1], "%d", score[1]);
+		outputf("%*s | %s\n", width1, data[0], data[1]);
+	}
+
+	free(data[0]);
+	free(data[1]);
+
+	output("\n");
+	outputx();
+}
 
 extern void CommandShowCredits( char *sz )
 {
 #if USE_GTK
   if( fX )
-    GTKCommandShowCredits();
+  {
+    GTKCommandShowCredits(NULL, NULL);
+    return;
+  }
 #endif
 
   outputl( aszAUTHORS );
@@ -987,7 +1097,7 @@ extern void CommandShowWarranty( char *sz ) {
 
 #if USE_GTK
     if( fX )
-	ShowList( aszWarranty, _("Warranty") );
+	ShowList( aszWarranty, _("Warranty"), NULL );
     else
 #endif
 	ShowPaged( aszWarranty );
@@ -1071,6 +1181,44 @@ extern void CommandShowThorp( char *sz ) {
 
 }
 
+extern void CommandShowKeith( char *sz ) {
+
+    int an[ 2 ][ 25 ];
+    int nLeader, nTrailer;
+    float fL;
+
+    if( !*sz && ms.gs == GAME_NONE ) {
+        outputl( _("No position specified and no game in progress.") );
+        return;
+    }
+
+    if( ParsePosition( an, &sz, NULL ) < 0 )
+	return;
+
+#if USE_GTK
+    if ( fX ) {
+      GTKShowRace ( 3, an );
+      return;
+    }
+#endif
+
+    KeithCount ( an, &nLeader, &nTrailer );
+    fL = (float) nLeader*8.0f /7.0f;
+    outputf("L = %d(%.1f)  T = %d  -> ", nLeader, fL, nTrailer);
+    if (nTrailer >= (fL - 3))
+      output(_("Redouble, "));
+    else if (nTrailer >= (fL - 4))
+      output(_("Double, "));
+    else
+      output(_("No double, "));
+    
+    if (nTrailer >= (fL - 2))
+      outputl(_("Drop"));
+    else
+      outputl(_("Take"));
+    
+}
+
 extern void CommandShowBeavers( char *sz ) {
 
     if( nBeavers > 1 )
@@ -1145,7 +1293,7 @@ EffectivePipCount( const float arMu[ 2 ], const int anPips[ 2 ] ) {
   int i;
   const float x = ( 2 * 3 + 3 * 4 + 4 * 5 + 4 * 6 + 6 * 7 +
               5* 8  + 4 * 9 + 2 * 10 + 2 * 11 + 1 * 12 + 
-              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0;
+              1 * 16 + 1 * 20 + 1 * 24 ) / 36.0f;
 
   outputl ( "" );
   outputl ( _("Effective pip count:") );
@@ -1290,9 +1438,9 @@ extern void CommandShowMatchEquityTable ( char *sz ) {
 #endif
 
   output ( _("Match equity table: ") );
-  outputl( miCurrent.szName );
+  outputl( (char*)miCurrent.szName );
   outputf( "(%s)\n", miCurrent.szFileName );
-  outputl( miCurrent.szDescription );
+  outputl( (char*)miCurrent.szDescription );
   outputl( "" );
   
   /* write tables */
@@ -1325,7 +1473,7 @@ extern void CommandShowOutput( char *sz ) {
              _("Game winning chances will be shown as percentages.\n") :
              _("Game winning chances will be shown as probabilities.\n") );
 
-#if USE_GUI
+#if USE_GTK
     if( !fX )
 #endif
       outputf( fOutputRawboard ? 
@@ -1348,26 +1496,19 @@ extern void CommandShowTraining( char *sz ) {
 	outputl( _("Error threshold disabled.") );
 }
 
-extern void CommandShowVersion( char *sz ) {
-
-    char **ppch = aszVersion;
-    extern char *aszCredits[];
-    int i = 0, cch, cCol = 0;
-    
+extern void CommandShowVersion( char *sz )
+{
 #if USE_GTK
-    if( fX ) {
-	GTKShowVersion();
-	return;
-    }
+	if (fX)
+	{
+		GTKShowVersion();
+		return;
+	}
 #endif
 
-    while( *ppch )
-	outputl( gettext ( *ppch++ ) );
-
-    outputc( '\n' );
-
-    ShowAuthors( ceAuthors, _("GNU Backgammon was written by:") );
-  
+	outputl( gettext ( VERSION_STRING ) );
+	outputc( '\n' );
+	ShowAuthors( ceAuthors, _("Written by:") );
 }
 
 extern void CommandShowMarketWindow ( char * sz ) {
@@ -1403,15 +1544,15 @@ extern void CommandShowMarketWindow ( char * sz ) {
 
   /* see if ratios are given on command line */
 
-  aarRates[ 0 ][ 0 ] = ParseReal ( &sz );
+  aarRates[ 0 ][ 0 ] = (float)ParseReal ( &sz );
 
   if ( aarRates[ 0 ][ 0 ] >= 0 ) {
 
     /* read the others */
 
-    aarRates[ 1 ][ 0 ]  = ( (r = ParseReal ( &sz ) ) > 0.0) ? r : 0.0;
-    aarRates[ 0 ][ 1 ] = ( (r = ParseReal ( &sz ) ) > 0.0) ? r : 0.0;
-    aarRates[ 1 ][ 1 ] = ( (r = ParseReal ( &sz ) ) > 0.0) ? r : 0.0;
+    aarRates[ 1 ][ 0 ]  = ( (r = (float)ParseReal ( &sz ) ) > 0.0f) ? r : 0.0f;
+    aarRates[ 0 ][ 1 ] = ( (r = (float)ParseReal ( &sz ) ) > 0.0f) ? r : 0.0f;
+    aarRates[ 1 ][ 1 ] = ( (r = (float)ParseReal ( &sz ) ) > 0.0f) ? r : 0.0f;
 
     /* If one of the ratios are larger than 1 we assume the user
        has entered 25.1 instead of 0.251 */
@@ -1444,11 +1585,11 @@ extern void CommandShowMarketWindow ( char * sz ) {
 
     arOutput[ OUTPUT_WIN ] = 0.5;
     arOutput[ OUTPUT_WINGAMMON ] =
-      ( aarRates[ ms.fMove ][ 0 ] + aarRates[ ms.fMove ][ 1 ] ) * 0.5;
+      ( aarRates[ ms.fMove ][ 0 ] + aarRates[ ms.fMove ][ 1 ] ) * 0.5f;
     arOutput[ OUTPUT_LOSEGAMMON ] =
-      ( aarRates[ !ms.fMove ][ 0 ] + aarRates[ !ms.fMove ][ 1 ] ) * 0.5;
-    arOutput[ OUTPUT_WINBACKGAMMON ] = aarRates[ ms.fMove ][ 1 ] * 0.5;
-    arOutput[ OUTPUT_LOSEBACKGAMMON ] = aarRates[ !ms.fMove ][ 1 ] * 0.5;
+      ( aarRates[ !ms.fMove ][ 0 ] + aarRates[ !ms.fMove ][ 1 ] ) * 0.5f;
+    arOutput[ OUTPUT_WINBACKGAMMON ] = aarRates[ ms.fMove ][ 1 ] * 0.5f;
+    arOutput[ OUTPUT_LOSEBACKGAMMON ] = aarRates[ !ms.fMove ][ 1 ] * 0.5f;
 
   } else {
 
@@ -1487,7 +1628,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
       /* MWC for "double, take; win" */
 
       rDTW =
-        (1.0 - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
+        (1.0f - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
         getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                 2 * ms.nCube, i, ms.fCrawford,
                 aafMET, aafMETPostCrawford )
@@ -1503,7 +1644,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
       /* MWC for "no double, take; win" */
 
       rNDW =
-        (1.0 - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
+        (1.0f - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
         getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                 ms.nCube, i, ms.fCrawford,
                 aafMET, aafMETPostCrawford )
@@ -1519,7 +1660,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
       /* MWC for "Double, take; lose" */
 
       rDTL =
-        (1.0 - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
+        (1.0f - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
         getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                 2 * ms.nCube, ! i, ms.fCrawford,
                 aafMET, aafMETPostCrawford )
@@ -1535,7 +1676,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
       /* MWC for "No double; lose" */
 
       rNDL =
-        (1.0 - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
+        (1.0f - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
         getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                 1 * ms.nCube, ! i, ms.fCrawford,
                 aafMET, aafMETPostCrawford )
@@ -1568,14 +1709,14 @@ extern void CommandShowMarketWindow ( char * sz ) {
       rRisk = rDTW - rDP;
       rGain = rDP - rDTL;
 
-      arCP1 [ i ] = 1.0 - rRisk / ( rRisk + rGain );
+      arCP1 [ i ] = 1.0f - rRisk / ( rRisk + rGain );
 
       if ( fAutoRedouble[ i ] ) {
 
         /* With redouble */
 
         rDTW =
-          (1.0 - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
+          (1.0f - aarRates[ i ][ 0 ] - aarRates[ i ][ 1 ]) *
           getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                   4 * ms.nCube, i, ms.fCrawford,
                   aafMET, aafMETPostCrawford )
@@ -1589,7 +1730,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
                   aafMET, aafMETPostCrawford );
 
         rDTL =
-          (1.0 - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
+          (1.0f - aarRates[ ! i ][ 0 ] - aarRates[ ! i ][ 1 ]) *
           getME ( ms.anScore[ 0 ], ms.anScore[ 1 ], ms.nMatchTo, i,
                   4 * ms.nCube, ! i, ms.fCrawford,
                   aafMET, aafMETPostCrawford )
@@ -1605,7 +1746,7 @@ extern void CommandShowMarketWindow ( char * sz ) {
         rRisk = rDTW - rDP;
         rGain = rDP - rDTL;
 
-        arCP2 [ i ] = 1.0 - rRisk / ( rRisk + rGain );
+        arCP2 [ i ] = 1.0f - rRisk / ( rRisk + rGain );
 
         /* Double point */
 
@@ -1694,7 +1835,7 @@ CommandShowExport ( char *sz ) {
 
   int i;
 
-#if USE_GTK 
+#if USE_GTK
   if( fX ) {
     GTKShowExport( &exsExport ); 
     return;
@@ -1872,7 +2013,7 @@ CommandShowPath ( char *sz ) {
   outputl ( _("Default and current paths "
             "for load, save, import, and export: \n") );
 
-  for ( i = 0; i < NUM_PATHS; ++i ) {
+  for ( i = 0; i < sizeof(aszPathNames)/sizeof(aszPathNames[0]); ++i ) {
 
     outputf ( "%s:\n", gettext ( aszPathNames[ i ] ) );
     if ( ! strcmp ( aaszPaths[ i ][ 0 ], "" ) )
@@ -1945,39 +2086,6 @@ outputl(_("Tutor mode evaluates moves using the same settings as Analysis.") );
 
 }
 
-static void GetGeometryString(char* buf, windowobject* pwo)
-{
-	char dispName[50];
-	sprintf(dispName, "%c%s %s", toupper(pwo->winName[0]), &pwo->winName[1], _("window"));
-
-	sprintf(buf, "%-17s : size %dx%d, position (%d,%d)\n",
-		dispName, pwo->wg.nWidth, pwo->wg.nHeight, pwo->wg.nPosX, pwo->wg.nPosY);
-}
-
-extern void
-CommandShowGeometry ( char *sz )
-{
-	int i;
-	char szBuf[1024];
-	output(_("Default geometries:\n\n"));
-
-	for (i = 0; i < NUM_WINDOWS; i++)
-	{
-		GetGeometryString(szBuf, &woPanel[i]);
-		output(szBuf);
-	}
-}
-
-extern void CommandShowHighlightColour ( char *sz ) {
-
-  outputf ( _("Moves will be highlighted in %s %s.\n"),
-              HighlightIntensity == 2 ? "dark" :
-              HighlightIntensity == 1 ? "medium" : "",
-			  HighlightColour->colourname);
-}
-
-
-
 #if USE_SOUND
 
 extern void
@@ -2017,7 +2125,7 @@ CommandShowSound ( char *sz ) {
 extern void
 CommandShowRolls ( char *sz ) {
 
-#if USE_GTK2
+#if USE_GTK
   int nDepth = ParseNumber ( &sz );
 #endif
 
@@ -2027,10 +2135,14 @@ CommandShowRolls ( char *sz ) {
     return;
   }
 
-#if USE_GTK2
+#if USE_GTK
 
   if ( fX ) {
+#if defined (REDUCTION_CODE)
     static evalcontext ec0ply = { TRUE, 0, 0, TRUE, 0.0 };
+#else
+    static evalcontext ec0ply = { TRUE, 0, FALSE, TRUE, 0.0 };
+#endif
     GTKShowRolls( nDepth, &ec0ply, &ms );
     return;
   }
@@ -2140,7 +2252,13 @@ CommandShowCubeEfficiency( char *sz ) {
 extern void
 CommandShowBearoff( char *sz ) {
 
-  char szTemp[ 2048 ];
+  char szTemp[ 4096 ];
+
+	if( ms.gs != GAME_PLAYING )
+	{
+		outputl( _("No game is being played.") );
+		return;
+	}
 
 #if USE_GTK
   if ( fX ) {
@@ -2297,7 +2415,8 @@ CommandShowMatchResult( char *sz ) {
   float arSum[ 2 ] = { 0.0f, 0.0f };
   float arSumSquared[ 2 ] = { 0.0f, 0.0f };
   int n = 0;
-  movegameinfo *pmgi;
+  moverecord *pmr;
+  xmovegameinfo *pmgi;
   statcontext *psc;
   list *pl;
   float r;
@@ -2310,8 +2429,9 @@ CommandShowMatchResult( char *sz ) {
 
   for( pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext, ++n ) {
   
-      pmgi = ( (list *) pl->p )->plNext->p;
-      assert( pmgi->mt == MOVE_GAMEINFO );
+      pmr = (moverecord *) ( (list *) pl->p )->plNext->p;
+      pmgi = &pmr->g;
+      assert( pmr->mt == MOVE_GAMEINFO );
       
       psc = &pmgi->sc;
       
@@ -2372,8 +2492,8 @@ CommandShowMatchResult( char *sz ) {
 
 extern void CommandShowTCTutorial( char *sz ) {
 #if USE_GTK
-    if ( fX )
-	ShowList( aszTcTutorial, _("Time Control Tutorial") );
+    if ( fX )	/* This should be converted to texi and added to the manual */
+	ShowList( aszTcTutorial, _("Time Control Tutorial"), NULL );
     else
 #endif
 	ShowPaged(aszTcTutorial);
@@ -2381,17 +2501,6 @@ extern void CommandShowTCTutorial( char *sz ) {
 #endif
 
 
-extern void CommandShowDisplayPanels( char *sz ) {
-	if (fDisplayPanels)
-	  outputf( _("Game list, Annotation and Message panels/windows "
-		        "will be displayed."));
-	else
-	  outputf( _("Game list, Annotation and Message panels/windows "
-		        "will not be displayed."));
-}
-
-
-
 extern void
 CommandShowEPC( char *sz ) {
 
@@ -2418,3 +2527,25 @@ CommandShowEPC( char *sz ) {
     }
 
 }
+
+
+extern void
+CommandShowManualWeb( char *sz ) {
+
+  OpenURL( "http://www.gnubg.org/win32/gnubg/gnubg.html" );
+
+}
+
+#if USE_GTK
+extern void
+CommandShowManualGUI( char *sz ) {
+
+  if ( fX ) {
+    GTKShowManual();
+  }
+  else
+    outputl( _("`show manual gui' does not work in command line mode.\n"
+               "Please use `show manual web' instead.") );
+
+}
+#endif /* USE_GTK */
