@@ -20,7 +20,7 @@
  * File modified by Joern Thyssen <jthyssen@dk.ibm.com> for use with
  * GNU Backgammon.
  *
- * $Id: sound.c,v 1.27 2004/03/15 17:45:54 Superfly_Jon Exp $
+ * $Id: sound.c,v 1.35 2004/04/16 16:00:37 Superfly_Jon Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,12 @@
 #endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef SIGIO
+  #ifdef HAVE_STROPTS_H
+  #include <stropts.h>
+#endif
+
 #endif
 
 #if HAVE_ESD
@@ -64,6 +70,10 @@
 #include <audio/audiolib.h>
 #endif
 
+#if HAVE_AUDIOFILE
+#include <audiofile.h>
+#endif
+
 #ifdef WIN32
 /* for PlaySound */
 #include "windows.h"
@@ -655,7 +665,7 @@ static char *can_play_audio( void ) {
 
 static int _af_ulaw2linear(unsigned char ulawbyte);
 
-#ifndef HAVE_AUDIOFILE
+#ifndef HAVE_AF_ULAW2LINEAR
 
 /*
 ** This routine converts from ulaw to 16 bit linear.
@@ -690,7 +700,8 @@ static int _af_ulaw2linear(unsigned char ulawbyte)
 	return (sample);
 }
 
-#endif /* ! HAVE_AUDIOFILE */
+#endif /* !HAVE_AF_ULAW2LINEAR */
+
 
 static int play_artsc(unsigned char *data, int size)
 {
@@ -1003,6 +1014,21 @@ void PlaySound_QuickTime (const char *cSoundFilename)
 
 #endif /* __APPLE__ */
 
+#ifdef WIN32
+void PrintWinError()
+{
+	LPVOID lpMsgBuf;
+	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+		FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS ,
+		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+		(LPTSTR) &lpMsgBuf, 0, NULL);
+
+	g_print("Windows error: ");
+	g_print((LPCTSTR)lpMsgBuf);
+
+	LocalFree(lpMsgBuf);
+}
+#endif
 
 static void 
 play_file_child(soundcache *psc, const char *filename) {
@@ -1089,8 +1115,36 @@ play_file_child(soundcache *psc, const char *filename) {
     case SOUND_SYSTEM_WINDOWS:
 
 #ifdef WIN32
-      while (!PlaySound(filename, NULL, SND_FILENAME | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT))
-        Sleep(1);	/* Wait (1ms) for previous sound to finish */
+	if (access(filename, F_OK))
+	{
+		g_print("Missing sound file: %s\n", filename);
+		return;
+	}
+	SetLastError(0);
+	while (!PlaySound(filename, NULL, SND_FILENAME | SND_ASYNC | SND_NOSTOP | SND_NODEFAULT))
+	{
+		static int soundDeviceAttached = -1;
+		if (soundDeviceAttached == -1)
+		{	/* Check for sound card */
+			soundDeviceAttached = waveOutGetNumDevs();
+		}
+		if (!soundDeviceAttached)
+		{	/* No sound card found - disable sound */
+			g_print("No soundcard found - sounds disabled\n");
+			fSound = FALSE;
+			return;
+		}
+		/* Check for errors */
+		if (GetLastError())
+		{
+			g_print("Error playing sound file: %s\n", filename);
+			PrintWinError();
+			SetLastError(0);
+			return;
+		}
+		Sleep(1);	/* Wait (1ms) for current sound to finish */
+	}
+
 #else
       assert ( FALSE );
 #endif
@@ -1209,6 +1263,9 @@ extern void SoundFlushCache( const gnubgsound gs ) {
 
 extern void SoundWait( void ) {
 
+    if (!fSound)
+        return;
+
     switch( ssSoundSystem ) {
 #ifdef SIGIO
     case SOUND_SYSTEM_NORMAL:
