@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: text.c,v 1.64 2004/02/04 18:02:18 uid68519 Exp $
+ * $Id: text.c,v 1.70 2005/10/11 07:56:38 Superfly_Jon Exp $
  */
 
 #include "config.h"
@@ -233,15 +233,17 @@ TextPrologue ( FILE *pf, const matchstate *pms, const int iGame ) {
            ap[ 0 ].szName, pms->anScore[ 0 ],
            ap[ 1 ].szName, pms->anScore[ 1 ] );
 
-  if ( pms->nMatchTo > 0 ) 
+  if ( pms->nMatchTo > 0 ) {
     fprintf( pf,
-             pms->nMatchTo == 1 ?
-             _(" (match to %d point%s)") :
-             _(" (match to %d points%s)"),
-             pms->nMatchTo,
-             pms->fCrawford ? 
-             _(", Crawford game") : ( pms->fPostCrawford ?
-					 _(", post-Crawford play") : ""));
+             ngettext( " (match to %d point)", 
+                       " (match to %d points)",
+                       pms->nMatchTo ),
+             pms->nMatchTo );
+    if ( pms->fCrawford )
+       fputs( _(", Crawford game"), pf );
+    if ( pms->fPostCrawford )
+       fputs( _(", post-Crawford play"), pf );
+  }
 
   fputs( "\n", pf );
 
@@ -262,7 +264,7 @@ TextEpilogue ( FILE *pf, const matchstate *pms ) {
 
   time_t t;
 
-  const char szVersion[] = "$Revision: 1.64 $";
+  const char szVersion[] = "$Revision: 1.70 $";
   int iMajor, iMinor;
 
   iMajor = atoi ( strchr ( szVersion, ' ' ) );
@@ -272,8 +274,8 @@ TextEpilogue ( FILE *pf, const matchstate *pms ) {
 
   fprintf ( pf, 
             _("Output generated %s"
-              "by GNU Backgammon %s ") ,
-            ctime ( &t ), VERSION );
+              "by %s ") ,
+            ctime ( &t ), VERSION_STRING );
             
   fprintf ( pf,
             _("(Text Export version %d.%d)\n\n"),
@@ -312,7 +314,6 @@ TextPrintCubeAnalysisTable ( FILE *pf,
   /* check if cube analysis should be printed */
 
   if ( pes->et == EVAL_NONE ) return; /* no evaluation */
-  if ( ! GetDPEq ( NULL, NULL, pci ) ) return; /* cube not available */
 
   FindCubeDecision ( arDouble, GCCCONSTAHACK aarOutput, pci );
 
@@ -369,23 +370,23 @@ TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
     /* cube analysis from move */
 
     TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->n.aarOutput,
-				 GCCCONSTAHACK pmr->n.aarStdDev,
-                                 pmr->n.fPlayer,
-                                 &pmr->n.esDouble, &ci, FALSE, -1,
-                                 pmr->n.stCube, SKILL_NONE );
+                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput,
+				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, &ci, FALSE, -1,
+                                 pmr->stCube, SKILL_NONE );
 
     break;
 
   case MOVE_DOUBLE:
 
     TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->d.CubeDecPtr->aarOutput, 
-				 GCCCONSTAHACK pmr->d.CubeDecPtr->aarStdDev,
-                                 pmr->d.fPlayer,
-                                 &pmr->d.CubeDecPtr->esDouble, 
+                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
+				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, 
 								 &ci, TRUE, -1,
-                                 pmr->d.st, SKILL_NONE );
+                                 pmr->stCube, SKILL_NONE );
 
     break;
 
@@ -395,14 +396,14 @@ TextPrintCubeAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
     /* cube analysis from double, {take, drop, beaver} */
 
     TextPrintCubeAnalysisTable ( pf, 
-                                 GCCCONSTAHACK pmr->d.CubeDecPtr->aarOutput, 
-				 GCCCONSTAHACK pmr->d.CubeDecPtr->aarStdDev,
-                                 pmr->d.fPlayer,
-                                 &pmr->d.CubeDecPtr->esDouble, 
+                                 GCCCONSTAHACK pmr->CubeDecPtr->aarOutput, 
+				 GCCCONSTAHACK pmr->CubeDecPtr->aarStdDev,
+                                 pmr->fPlayer,
+                                 &pmr->CubeDecPtr->esDouble, 
 								 &ci, TRUE, 
                                  pmr->mt == MOVE_TAKE,
                                  SKILL_NONE, /* FIXME: skill from prev. cube */
-                                 pmr->d.st );
+                                 pmr->stCube );
 
     break;
 
@@ -458,57 +459,57 @@ TextPrintMoveAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
       fprintf ( pf, " (%+7.3f)\n",
-                pmr->n.ml.amMoves[ pmr->n.iMove ].rScore -
-                pmr->n.ml.amMoves[ 0 ].rScore  );
+                pmr->ml.amMoves[ pmr->n.iMove ].rScore -
+                pmr->ml.amMoves[ 0 ].rScore  );
     else
       fprintf ( pf, " (%+6.3f%%)\n",
                 100.0f *
-                eq2mwc ( pmr->n.ml.amMoves[ pmr->n.iMove ].rScore, &ci ) - 
+                eq2mwc ( pmr->ml.amMoves[ pmr->n.iMove ].rScore, &ci ) - 
                 100.0f *
-                eq2mwc ( pmr->n.ml.amMoves[ 0 ].rScore, &ci ) );
+                eq2mwc ( pmr->ml.amMoves[ 0 ].rScore, &ci ) );
 
   }
 
-  if ( pmr->n.lt != LUCK_NONE ) {
+  if ( pmr->lt != LUCK_NONE ) {
 
     /* joker */
 
     fprintf ( pf, 
               _("Alert: %s roll!"),
-              gettext ( aszLuckType[ pmr->n.lt ] ) );
+              gettext ( aszLuckType[ pmr->lt ] ) );
     
     if ( !pms->nMatchTo || ( pms->nMatchTo && ! fOutputMWC ) )
-      fprintf ( pf, " (%+7.3f)\n", pmr->n.rLuck );
+      fprintf ( pf, " (%+7.3f)\n", pmr->rLuck );
     else
       fprintf ( pf, " (%+6.3f%%)\n",
-                100.0f * eq2mwc ( pmr->n.rLuck, &ci ) - 
+                100.0f * eq2mwc ( pmr->rLuck, &ci ) - 
                 100.0f * eq2mwc ( 0.0f, &ci ) );
 
   }
 
   fputs ( "\n", pf );
 
-  fprintf( pf, _("Rolled %d%d"), pmr->n.anRoll[ 0 ], pmr->n.anRoll[ 1 ] );
+  fprintf( pf, _("Rolled %d%d"), pmr->anDice[ 0 ], pmr->anDice[ 1 ] );
 
-  if( pmr->n.rLuck != ERR_VAL )
-    fprintf( pf, " (%s):\n", GetLuckAnalysis( pms, pmr->n.rLuck ) );
+  if( pmr->rLuck != ERR_VAL )
+    fprintf( pf, " (%s):\n", GetLuckAnalysis( pms, pmr->rLuck ) );
   else
     fprintf( pf, ":" );
 
-  if ( pmr->n.ml.cMoves ) {
+  if ( pmr->ml.cMoves ) {
 	
-    for( i = 0; i < pmr->n.ml.cMoves; i++ ) {
+    for( i = 0; i < pmr->ml.cMoves; i++ ) {
       if( i >= exsExport.nMoves && i != pmr->n.iMove )
         continue;
 
       fputc( i == pmr->n.iMove ? '*' : ' ', pf );
-      fputs( FormatMoveHint( szBuf, pms, &pmr->n.ml, i,
+      fputs( FormatMoveHint( szBuf, pms, &pmr->ml, i,
                              i != pmr->n.iMove ||
-                             i != pmr->n.ml.cMoves - 1 ||
-                             pmr->n.ml.cMoves == 1,
+                             i != pmr->ml.cMoves - 1 ||
+                             pmr->ml.cMoves == 1,
                              exsExport.fMovesDetailProb,
                              exsExport.afMovesParameters 
-                             [ pmr->n.ml.amMoves[ i ].esMove.et - 1 ] ), 
+                             [ pmr->ml.amMoves[ i ].esMove.et - 1 ] ), 
              pf );
 
 
@@ -564,12 +565,12 @@ TextAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
     if ( pmr->n.anMove[ 0 ] >= 0 )
       fprintf ( pf,
                 _("* %s moves %s"),
-                ap[ pmr->n.fPlayer ].szName,
+                ap[ pmr->fPlayer ].szName,
                 FormatMove ( sz, pms->anBoard, pmr->n.anMove ) );
-    else if ( ! pmr->n.ml.cMoves )
+    else if ( ! pmr->ml.cMoves )
       fprintf ( pf,
                 _("* %s cannot move"),
-                ap[ pmr->n.fPlayer ].szName );
+                ap[ pmr->fPlayer ].szName );
 
     fputs ( "\n", pf );
 
@@ -588,11 +589,11 @@ TextAnalysis ( FILE *pf, matchstate *pms, moverecord *pmr ) {
     if ( pmr->mt == MOVE_DOUBLE ) 
       fprintf ( pf,
                 "* %s doubles\n\n",
-                ap[ pmr->d.fPlayer ].szName );
+                ap[ pmr->fPlayer ].szName );
     else
       fprintf ( pf,
                 "* %s %s\n\n",
-                ap[ pmr->d.fPlayer ].szName,
+                ap[ pmr->fPlayer ].szName,
                 ( pmr->mt == MOVE_TAKE ) ? _("accepts") : _("rejects") );
     
     if ( exsExport.fIncludeAnalysis )
@@ -642,30 +643,30 @@ TextPrintComment ( FILE *pf, const moverecord *pmr ) {
   switch ( pmr->mt ) {
 
   case MOVE_GAMEINFO:
-    sz = pmr->g.sz;
+    sz = pmr->sz;
     break;
   case MOVE_DOUBLE:
   case MOVE_TAKE:
   case MOVE_DROP:
-    sz = pmr->d.sz;
+    sz = pmr->sz;
     break;
   case MOVE_NORMAL:
-    sz = pmr->n.sz;
+    sz = pmr->sz;
     break;
   case MOVE_RESIGN:
-    sz = pmr->r.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETBOARD:
-    sz = pmr->sb.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETDICE:
-    sz = pmr->sd.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETCUBEVAL:
-    sz = pmr->scv.sz;
+    sz = pmr->sz;
     break;
   case MOVE_SETCUBEPOS:
-    sz = pmr->scp.sz;
+    sz = pmr->sz;
     break;
   case MOVE_TIME:
 	  /* ignore */
@@ -806,7 +807,7 @@ static void ExportGameText ( FILE *pf, list *plGame,
     int iMove = 0;
     statcontext *psc = NULL;
     static statcontext scTotal;
-    movegameinfo *pmgi = NULL;
+    xmovegameinfo *pmgi = NULL;
 
     if ( ! iGame )
       IniStatcontext ( &scTotal );
@@ -842,14 +843,14 @@ static void ExportGameText ( FILE *pf, list *plGame,
 
       case MOVE_NORMAL:
 
-	if( pmr->n.fPlayer != msExport.fMove ) {
+	if( pmr->fPlayer != msExport.fMove ) {
 	    SwapSides( msExport.anBoard );
-	    msExport.fMove = pmr->n.fPlayer;
+	    msExport.fMove = pmr->fPlayer;
 	}
       
-        msExport.fTurn = msExport.fMove = pmr->n.fPlayer;
-        msExport.anDice[ 0 ] = pmr->n.anRoll[ 0 ];
-        msExport.anDice[ 1 ] = pmr->n.anRoll[ 1 ];
+        msExport.fTurn = msExport.fMove = pmr->fPlayer;
+        msExport.anDice[ 0 ] = pmr->anDice[ 0 ];
+        msExport.anDice[ 1 ] = pmr->anDice[ 1 ];
 
         TextBoardHeader ( pf, &msExport, iGame, iMove );
 
@@ -930,7 +931,7 @@ extern void CommandExportGameText( char *sz ) {
     }
     
     if( !sz || !*sz ) {
-	outputl( _("You must specify a file to export to (see `help export"
+	outputl( _("You must specify a file to export to (see `help export "
 		 "game text').") );
 	return;
     }
