@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: timecontrol.c,v 1.15 2004/01/19 10:50:30 uid68519 Exp $
+ * $Id: timecontrol.c,v 1.18 2004/05/07 14:28:23 thyssen Exp $
  */
 
 #include "config.h"
@@ -27,6 +27,10 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+#if HAVE_LIMITS_H
+#include <limits.h>
+#endif
+
 #include "backgammon.h"
 #include "eval.h"
 
@@ -753,7 +757,7 @@ int gettimeofday (struct timeval *tv, void * arg)
 
 	tv->tv_sec = (long) ms / 1000;
 	/* micro seconds */
-	tv->tv_usec = (long) ((ms - (tv->tv_sec * 1000)) / 1000);
+	tv->tv_usec = (long) ((ms - (tv->tv_sec * 1000)) * 1000);
 
 	return 0;
 }
@@ -869,14 +873,14 @@ extern int CheckGameClock(matchstate *pms, struct timeval *tvp)
     pgcOpp->tvStamp = *tvp;
 
     if (pen) {
-    moverecord *pmr;
-	pmr = malloc( sizeof( movetime ) );
+        moverecord *pmr = NewMoveRecord();
+
 	pmr->mt = MOVE_TIME;
-	pmr->t.sz = 0;
-	pmr->t.fPlayer = ms.fTurn;
-	pmr->t.tl[0] = ms.gc.pc[0].tvTimeleft;
-	pmr->t.tl[1] = ms.gc.pc[1].tvTimeleft;
+	pmr->fPlayer = ms.fTurn;
+	pmr->tl[0] = ms.gc.pc[0].tvTimeleft;
+	pmr->tl[1] = ms.gc.pc[1].tvTimeleft;
 	pmr->t.nPoints = pen;
+
 	AddMoveRecord( pmr );
    }
    return pen;
